package controller;

import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import parser.OPMLParser;
import parser.RSSParser;
import search.AdvancedSearchDialog;
import util.IconFactory;
import util.Pair;
import util.IconFactory.ICON;
import view.dialogs.AboutBox;
import view.dialogs.CraigslistImportDialog;
import view.dialogs.GeoRSSConverter;
import view.dialogs.GoogleMapShowFeeds;
import view.dialogs.LJImportDialog;
import view.dialogs.LJPostDialog;
import view.dialogs.NewChannelDialog;
import view.dialogs.OPMLImportDialog;
import view.dialogs.SubscribeDialog;
import view.dialogs.WebSearchDialog;
import view.dialogs.WriteArticleDialog;

import com.sun.syndication.io.FeedException;

import data.ArticleContent;
import data.ArticleInfo;
import data.Channel;
import data.DataException;
import data.Feed;
import data.Virtual;
import data.VirtualFactory;

/**
 * Action manager for toolbars and menus.
 * @author oleg
 */
public class ActionManager { 
	private final Shell parent;
	private final Controller controller;
	private final Map<Actions, Listener> actions = new HashMap<Actions, Listener>();
	/**
	 * Action ids.
	 * @author oleg
	 */
	public enum Actions {
		/**
		 * Searches the Web for feeds.
		 */
		WEB_SEARCH,
		/**
		 * Imports LJ friendlist.
		 */
		LJ_IMPORT,
		/**
		 * Shows feeds on a map.
		 */
		VISUALIZE_FEEDS,
		/**
		 * Imports from OPML
		 */
		IMPORT_OPML,
		/**
		 * Exports feeds as OPML
		 */
		EXPORT_OPML,
		/**
		 * Posts outbox to LJ. 
		 */
		POST_OUTBOX,
		/**
		 * Writes the contents of the outbox to a file
		 */
		WRITE_OUTBOX,
		/**
		 * Creates a new feed.
		 */
		NEW_FEED,
		/**
		 * Creates a new article.
		 */
		NEW_ARTICLE,
		/**
		 * Creates a new folder.
		 */
		NEW_FOLDER,
		/**
		 * Forces immediate synchronization for all feeds.
		 */
		FORCE_SYNC,
		/**
		 * Exits application.
		 */
		EXIT,
		/**
		 * Search
		 */
		SEARCH,
		/**
		 * Import craigslist feed
		 */
		CRAIGSLIST_SUBSCRIBE,
		/**
		 * Pipes a feed through GeoNames analyzer service to add GeoRSS information
		 */
		GEONAMES,
		/**
		 * Opens the application home page in the default browser
		 */
		VISIT_WEBSITE;
	}
	/**
	 * Creates a toolbar with standard actions on the given pane.
	 * @param composite parent pane
	 * @return a toolbar
	 */
	public ToolBar createStandardToolbar(final Composite composite) {
		ToolBar toolbar = new ToolBar(composite, SWT.NONE);
		toolbar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ToolItem ti = new ToolItem(toolbar,SWT.FLAT);
		ti.setImage(IconFactory.getIcon(composite.getDisplay(),ICON.NEW_FOLDER));
		ti.addListener(SWT.Selection, actions.get(Actions.NEW_FOLDER));
		ti.setToolTipText("Create a new folder");
		ti = new ToolItem(toolbar,SWT.FLAT);
		ti.setImage(IconFactory.getIcon(composite.getDisplay(),ICON.NEW_FEED));
		ti.setToolTipText("Subscribe to a feed");
		ti.addListener(SWT.Selection, actions.get(Actions.NEW_FEED));
		ti = new ToolItem(toolbar,SWT.FLAT);
		ti.setImage(IconFactory.getIcon(composite.getDisplay(),ICON.NEW_ARTICLE));
		ti.setToolTipText("Write an article");
		ti.addListener(SWT.Selection, actions.get(Actions.NEW_ARTICLE));
		ti = new ToolItem(toolbar,SWT.FLAT);
		ti.setImage(IconFactory.getIcon(composite.getDisplay(),ICON.UPDATE));
		ti.setToolTipText("Update all feeds");
		ti.addListener(SWT.Selection, actions.get(Actions.FORCE_SYNC));
		ti = new ToolItem(toolbar,SWT.FLAT);
		ti.setImage(IconFactory.getIcon(composite.getDisplay(),ICON.WEB_SEARCH));
		ti.addListener(SWT.Selection, actions.get(Actions.WEB_SEARCH));
		ti.setToolTipText("Search the web for feeds");
		return toolbar;
	}
	/**
	 * Creates a menu with standard submenus.
	 * @return a menu
	 */
	public StandardMenu createStandardMenuBar() {
		StandardMenu menu = new StandardMenu(parent);
		menu.addSubMenu("File");
		menu.addSubMenu("View");
		menu.addSubMenu("Tools");
		menu.addSubMenu("Help");
		menu.addItem("File", "Subscribe to a feed", actions.get(Actions.NEW_FEED),ICON.NEW_FEED);  
		menu.addItem("File", "New folder", actions.get(Actions.NEW_FOLDER), ICON.NEW_FOLDER);
		menu.addItem("File","Write an article...",actions.get(Actions.NEW_ARTICLE),ICON.NEW_ARTICLE);
		menu.addSeparator("File");
		menu.addItem("File","Update all feeds", actions.get(Actions.FORCE_SYNC), ICON.UPDATE);
		menu.addSeparator("File");
		menu.addItem("File", "Exit", actions.get(Actions.EXIT),ICON.EXIT);

		for(final String view : controller.getViews()) {
			menu.addItem("View", "Switch to " + view, new Listener() {
				public void handleEvent(Event e) {
					controller.switchView(view);
				}
			}, controller.getViewIcon(view));
		}
		
		menu.addItem("Tools", "Web search...", actions.get(Actions.WEB_SEARCH),ICON.WEB_SEARCH);
		menu.addItem("Tools", "Search", actions.get(Actions.SEARCH), ICON.SEARCH);
		menu.addSeparator("Tools");
		
		MenuItem mi = new MenuItem(menu.getSubMenu("Tools"),SWT.CASCADE);
		Menu ljMenu = new Menu(menu.getSWTMenu());
		mi.setImage(IconFactory.getIcon(parent.getDisplay(), ICON.LIVEJOURNAL));
		mi.setText("Livejournal");
		mi.setMenu(ljMenu);
		
		mi = new MenuItem(ljMenu,SWT.CASCADE);
		mi.setText("Import friend list...");
		mi.addListener(SWT.Selection, actions.get(Actions.LJ_IMPORT));
	
		mi = new MenuItem(ljMenu,SWT.CASCADE);
		mi.setText("Post outbox...");
		mi.addListener(SWT.Selection, actions.get(Actions.POST_OUTBOX));
		
		mi = new MenuItem(menu.getSubMenu("Tools"),SWT.CASCADE);
		Menu geoMenu = new Menu(menu.getSWTMenu());
		mi.setImage(IconFactory.getIcon(parent.getDisplay(), ICON.GOOGLE_MAPS));
		mi.setText("GeoRSS");
		mi.setMenu(geoMenu);
		
		mi = new MenuItem(geoMenu, SWT.CASCADE);
		mi.setText("Convert to GeoRSS...");
		mi.addListener(SWT.Selection, actions.get(Actions.GEONAMES));
		
		mi = new MenuItem(geoMenu, SWT.CASCADE);
		mi.setText("Subscribe to craigslist.org feed...");
		mi.addListener(SWT.Selection, actions.get(Actions.CRAIGSLIST_SUBSCRIBE));		

		menu.addItem("Tools", "Save outbox as...", actions.get(Actions.WRITE_OUTBOX), ICON.SAVE);
		
		menu.addItem("Tools", "Import feeds...", actions.get(Actions.IMPORT_OPML),ICON.STAR);
		
		menu.addItem("Tools", "Export feeds...", actions.get(Actions.EXPORT_OPML),ICON.SAVE);
		
		menu.addSeparator("Tools");

		menu.addItem("Tools","Show feeds on a map...", actions.get(Actions.VISUALIZE_FEEDS),ICON.GOOGLE_MAPS);
		
		menu.addItem("Help", "User manual", new Listener() {
			public void handleEvent(Event event) {
				Program.launch(GlobalPreferences.USER_MANUAL_URL);
			}
		},ICON.HELP);
		menu.addItem("Help", "Visit web site...", actions.get(Actions.VISIT_WEBSITE),ICON.LINK);
		menu.addItem("Help", "About", new Listener() {
			public void handleEvent(Event event) {
				new AboutBox(parent, controller);
			}
		},ICON.STAR);		
		return menu;
	}
	/**
	 * Creates a new action manager for the given shell.
	 * @param parent shell
	 * @param controller controller
	 */
	public ActionManager(final Shell parent, final Controller controller) {
		this.parent = parent;
		this.controller = controller;
		actions.put(Actions.VISIT_WEBSITE, new Listener() {
			public void handleEvent(Event event) {
				Program.launch(GlobalPreferences.WEB_PAGE);
			}
		});
		actions.put(Actions.WEB_SEARCH, new Listener() {
			public void handleEvent(Event event) {
				new WebSearchDialog(parent, controller);
			}
		});
		actions.put(Actions.VISUALIZE_FEEDS, new Listener() {
			public void handleEvent(Event event) {
				new GoogleMapShowFeeds(parent, controller);
			}
		});
		actions.put(Actions.POST_OUTBOX, new Listener() {
			public void handleEvent(Event event) {
				new LJPostDialog(parent, controller);
			}
		});
		actions.put(Actions.EXPORT_OPML, new Listener() {
			public void handleEvent(Event event) {
				FileDialog saveAs = new FileDialog(parent, SWT.SAVE);
				saveAs.setText("Export feeds as...");
				String fileName = saveAs.open();
				if (fileName != null) {
					Map<Channel, Collection<Feed>> feeds = new HashMap<Channel, Collection<Feed>>();
					for (Channel channel: controller.getDataManager().getChannels()) {
						if (!channel.getName().equals("")) feeds.put(channel, new LinkedList<Feed>());
					}
					for (Feed feed: controller.getDataManager().getFeeds()) {
						Collection<Feed> contents = feeds.get(feed.getChannel());
						if (contents != null) contents.add(feed);
					}
					try {
						OPMLParser.writeOPML(feeds, fileName);
					} catch (IOException e) {
						controller.reportError("Writing OPML summary caused an error:\n" + e.getMessage());
					}
				}
			}
		});
		actions.put(Actions.WRITE_OUTBOX, new Listener() {
			public void handleEvent(Event event) {
				Iterator<ArticleInfo> iter = controller.getDataManager().getArticles(controller.getDataManager().getFeed("rss-reader://outbox"));
				if (!iter.hasNext()) {
					MessageBox mb = new MessageBox(parent, SWT.ICON_WARNING | SWT.OK);
					mb.setText("Warning");
					mb.setMessage("Outbox is empty, operation aborted.");
					mb.open();
				} else {
					FileDialog fd = new FileDialog(parent, SWT.SAVE);
					fd.setText("Save outbox as...");
					String fileName = fd.open();
					if (fileName != null) {
						List<Pair<ArticleInfo, ArticleContent>> articles = new LinkedList<Pair<ArticleInfo,ArticleContent>>();
						while (iter.hasNext()) {
							ArticleInfo info = iter.next();
							articles.add(new Pair<ArticleInfo, ArticleContent>(info, controller.getDataManager().getContent(info)));
							try {
								RSSParser.writeFeed(controller.getDataManager().getFeed("rss-reader://outbox").getName(), articles, fileName);
							} catch (IOException e) {
								controller.reportError("Error writing outbox:\n" + e.getMessage());
							} catch (FeedException e) {
								controller.reportError("Error building the feed");
							}
						}
					}
				}
			}
		});
		actions.put(Actions.LJ_IMPORT, new Listener() {
			public void handleEvent(Event event) {
				new LJImportDialog(parent, controller);
			}
		});
		actions.put(Actions.NEW_FEED, new Listener() {
			public void handleEvent(Event event) {
				new SubscribeDialog(parent,controller);
			}
		});
		actions.put(Actions.NEW_ARTICLE, new Listener() {
			public void handleEvent(Event event) {
				new WriteArticleDialog(parent,controller);
			}
		});
		actions.put(Actions.FORCE_SYNC, new Listener() {
			public void handleEvent(Event event) {
				for (Feed feed: controller.getDataManager().getFeeds()) {
					controller.getDownloadManager().forceDownload(feed);
				}
			}
		});
		actions.put(Actions.NEW_FOLDER, new Listener() {
			public void handleEvent(Event event) {
				new NewChannelDialog(parent, controller);
			}
		});
		actions.put(Actions.EXIT, new Listener() {
			public void handleEvent(Event event) {
				parent.close();
			}
		});		
		actions.put(Actions.SEARCH, new Listener() {
			public void handleEvent(Event event) {
				Virtual result = new AdvancedSearchDialog(parent, controller).open();
				if (result != null) {
					// Remove search results first
					Virtual old = controller.getDataManager().getVirtual(GlobalPreferences.SEARCH_RESULTS);
					if (old != null) {
						try {
							controller.getDataManager().removeVirtual(old);
						} catch (DataException e) {
							controller.reportError("Could not remove the old search results");
							return;
						}
					}
					// Add new one
					try {
						controller.getDataManager().addVirtual(VirtualFactory.changeName(result, GlobalPreferences.SEARCH_RESULTS));
					} catch (DataException e) {
						controller.reportError("Could not save the search results");
						return;
					}				
				}
			}				
		});
		actions.put(Actions.CRAIGSLIST_SUBSCRIBE, new Listener() {
			public void handleEvent(Event event) {
				new CraigslistImportDialog(parent, controller);
			}
		});
		actions.put(Actions.GEONAMES, new Listener() {
			public void handleEvent(Event e) {
				new GeoRSSConverter(parent, controller);
			}
		});		
		actions.put(Actions.IMPORT_OPML, new Listener() {
			public void handleEvent(Event e) {
				new OPMLImportDialog(parent, controller); 
			}
		});
	}
	/**
	 * Accesses standard actions.
	 * @param id action id
	 * @return the action with that id
	 */
	public Listener getAction(Actions id) {
		return actions.get(id);
	}
}
