package controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolTip;
import org.eclipse.swt.widgets.Tray;
import org.eclipse.swt.widgets.TrayItem;

import util.IconFactory;
import util.ReflectionUtils;
import util.Trace;
import util.IconFactory.ICON;
import view.CalendarView;
import view.GoogleMapsView;
import view.MinimalisticView;
import view.NewspaperView;
import view.RSSMailView;
import view.View;
import data.ArticleInfo;
import data.DataException;
import data.DataManager;
import data.DataManagerDBImpl;
import data.Feed;
import data.Virtual;
import data.VirtualFactory;
import download.DownloadManager;
import download.DownloadObserver;

/**
 * Controller implementation.
 * Performs highest-level tasks such as maintaining settings, initializing, shutdown,
 * and managing multiple views.
 * 
 * Displays a tray icon in the system tray which allows view switching.
 * @author Kuat
 */
public class Controller implements DownloadObserver {

	private ControllerSettings config;	
	private DataManager dataManager;
	private DownloadManager downloadManager;
	private final ActionManager actionManager;
	
	private boolean isOnline;
	
	private Display display;
	private Shell shell;
	private View currentView;
	private TrayItem item;
	private ToolTip tip;	 
	
	private Image trayIcon;
	private Image trayIconBusy;
	private boolean trayEnabled;
	
	// Collection of registered views
	private Map<String, Class> views = new ConcurrentHashMap<String, Class>();
	
	// Collection of icons
	private Map<String, Image> viewIcons = new ConcurrentHashMap<String, Image>();
	
	/**
	 * Retrieves a view's icon.
	 * @param name the name of the view
	 * @return icon
	 */
	public Image getViewIcon(String name) {
		return viewIcons.get(name);
	}
	
	/**
	 * Retrieves collection of all views by their names.
	 * @return
	 */
	public Collection<String> getViews(){
		return Collections.unmodifiableCollection(views.keySet());
	}
	
	/**
	 * Switches to the view specified by the given name.
	 * If view is not found, then gives no effect. 
	 * @param id
	 */
	public synchronized void switchView (String id) {
		Class clazz = views.get(id);
		
		report("Switching to " + id);
		
		switchView(clazz);
	}
	
	private boolean switchView(Class clazz) {
		if (clazz == null) {
			return false;
		} else {
			if ((currentView != null) && (clazz.equals(currentView.getClass()))) {
				shell.setVisible(true);
				return false;
			}
			if (!ReflectionUtils.isSuperClass(View.class, clazz)) {
				Trace.printException("Object in views is not a view");
				return false;
			}
			
			shell.setVisible(false);
			Object o = null;
			try {
				Constructor constructor = clazz.getConstructor(new Class[] {Shell.class, Controller.class});
				o = constructor.newInstance(shell, this);				
			} catch (SecurityException e) {
				reportError("Security violation of constructor.");				
			} catch (NoSuchMethodException e) {
				reportError("Constructor cannot be found.");
			} catch (IllegalArgumentException e) {
				reportError(e.getMessage());
			} catch (InstantiationException e) {
				reportError("The view cannot be instantiated.");
			} catch (IllegalAccessException e) {
				reportError("Access violation.");
			} catch (InvocationTargetException e) {
				reportError("Constructor exception: " + e.getCause().toString());
			}
			boolean result = false;
			
			if (o != null) {
				closeView();
				//shell.pack();
				shell.setText(GlobalPreferences.APPLICATION_NAME);
				for (String key : views.keySet())
					if (views.get(key).equals(clazz)) {
						shell.setText(GlobalPreferences.APPLICATION_NAME + " : " + config.userName + " : " + key);
						break;
					}
				shell.setMenuBar(((View) o).getMenuBar());
				shell.pack();
				
				shell.open();		
				
				currentView = (View) o;
				// registering with Data Manager
				dataManager.addViewListener(currentView);
				// unregistering observers and saving configuration before GUI widgets disposal
				currentView.addDisposeListener(new DisposeListener() {
					public void widgetDisposed(DisposeEvent e) {
						currentView.terminate();
					}			
				});					
				result = true;
			} else {
				//Trace.printException("Object in views is not a view");
				Trace.printException("Error instantiating a view");
			}	
			
			shell.setVisible(true);
			return result;
		}
	}
	
	private void closeView(){
		if ((currentView != null) && (!currentView.isDisposed())) { 			
			currentView.dispose();
		}
		
		currentView = null;
	}
		
	/**
	 * Returns the data manager of the application. 
	 * Provides a context in which all views are running.
	 * @return
	 */
	public DataManager getDataManager(){
		return dataManager;
	} 	
	
	/** 
	 * Returns the download manager of the application.
	 * @return
	 */
	public DownloadManager getDownloadManager() {
		return downloadManager;
	}
	
	/**
	 * Returns the action manager for the main shell.
	 * @return
	 */
	public ActionManager getActionManager() {
		return actionManager;
	}
	
	/**
	 * Returns the currently loaded view.
	 * @return
	 */
	public View getView() {
		return currentView;
	}
	
	private void addMenuItemListener(MenuItem item, final String name) {
		item.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				switchView(name);
			}
		});		
	}

	private void configure(boolean forceDialog) {
		// Configuring
		ProfileManager pm = new ProfileManager(shell, this);
		ControllerSettings value = pm.open(forceDialog); 
		if (value != null) {
			config = value;
			isOnline = config.isOnline;
			loadProfile();
		} else {
			config = null;
			shell.close();
			return;
		}
	}
	
	private void loadProfile(){			
		// Initialize basic components		
		if (!dataManager.initialize(config.getProfileLocation())) {
			Trace.printException("Critical error: database failure");
			System.exit(0);
		}
		
		downloadManager = new DownloadManager(this);
		downloadManager.addDownloadObserver(this);

		switchView(config.view);
		if (item!=null) trayEnabled = true;
	}
		
	private void closeProfile(){
		if (config != null) {			
			Trace.printVerbose(config.toString());
			saveSettings(this.getClass().getName(), 
					new ControllerSettings(config.userName, config.purgePeriod, (currentView == null ? config.view : currentView.getClass()),isOnline));
			if (item!=null) trayEnabled = false;
			closeView();
			
			if ((shell!=null) && !shell.isDisposed()) shell.setVisible(false);
			
			downloadManager.removeDownloadObserver(this);
			downloadManager.terminate();

			// Purging removes all articles past currentdate - purgeperiod which are not starred
			if (config.purgePeriod > 0) {
				long date = System.currentTimeMillis() - config.purgePeriod;
				Virtual v = VirtualFactory.publishedUnstarredBeforeDate(date);
				Iterator<ArticleInfo> iterator = dataManager.getArticles(v);
				int count = 0;
				while(iterator.hasNext()) {
					try {
						dataManager.removeArticle(iterator.next());
						count++;
					} catch (DataException e) {
						Trace.printException(e);
					}
				}
				
				Trace.printVerbose("Purged " + count + " before " + new Date(date));
			}
			
			dataManager.terminate();
			config = null;
		}
	}

	
	Controller() {
		// GUI initialization
		display = new Display();
		shell = new Shell(display);
		actionManager = new ActionManager(shell, this);
		trayIcon = new Image(display, IconFactory.getIcon(IconFactory.ICON.TRAY_ICON));
		trayIconBusy = new Image(display, IconFactory.getIcon(IconFactory.ICON.TRAY_ICON_BUSY));
		shell.setImage(trayIcon);
		shell.setLayout(new FillLayout());
		
		// Setting variables
		initializeViews();
		dataManager = DataManagerDBImpl.getInstance();
		
		// Adding a tray icon
		Tray tray = display.getSystemTray();
		final Controller controller = this;
		if(tray != null) {
			item = new TrayItem(tray, SWT.NONE);
			trayEnabled = false;
			item.setImage(trayIcon);
			item.setToolTipText(GlobalPreferences.APPLICATION_NAME);
			item.addListener(SWT.MenuDetect, new Listener() {
				public void handleEvent(Event event) {
					if (!trayEnabled) return;
					if (shell.getShells().length > 0) return;
					
					// Style must be pop up
					final Menu m = new Menu(shell, SWT.POP_UP);

					for(String name : views.keySet()) {
						MenuItem item = new MenuItem(m, SWT.PUSH);
						item.setText("Switch to " + name);
						item.setImage(viewIcons.get(name));
						addMenuItemListener(item, name);
					}
					
					new MenuItem(m, SWT.SEPARATOR);
					MenuItem edit = new MenuItem(m, SWT.NONE);					
					edit.setText("Edit profile");
					edit.setImage(new Image(shell.getDisplay(), IconFactory.getIcon(IconFactory.ICON.RENAME)));
					edit.addListener(SWT.Selection, new Listener() {
						public void handleEvent(Event event) {		
							trayEnabled = false;
							ControllerSettings modified = new NewProfileDialog(shell, controller).open(config);
							if (modified != null) {
								config = modified;								
							}
							trayEnabled = true;					
						}
					});

					MenuItem login = new MenuItem(m, SWT.NONE);
					login.setText("Select different profile");	
					login.setImage(new Image(shell.getDisplay(), IconFactory.getIcon(IconFactory.ICON.PROFILE)));
					login.addListener(SWT.Selection, new Listener() {
						public void handleEvent(Event event) {
							closeProfile();
							configure(true);
						}
					});
					
					new MenuItem(m, SWT.SEPARATOR);
					MenuItem exit = new MenuItem(m, SWT.NONE);
					exit.setText("Exit");	
					exit.setImage(new Image(shell.getDisplay(), IconFactory.getIcon(ICON.EXIT)));
					exit.addListener(SWT.Selection, new Listener() {
						public void handleEvent(Event event) {
							shell.dispose();
						}
					});										
					
					// We need to make the menu visible
					m.setVisible(true);
				};
			});
			item.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					if (trayEnabled)
						shell.setVisible(!shell.getVisible());
				}
			});			
			tip = new ToolTip(shell, SWT.BALLOON | SWT.ICON_INFORMATION);
			tip.setVisible(false);
			tip.setAutoHide(true);
			item.setToolTip(tip);
			tip.setText("Message");			
		}	
		
		configure(false);

		// Read and dispatch loop
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		
		closeProfile();
		
		if (!display.isDisposed()) display.dispose();
	}
	
	private void initializeViews(){
		views.put("Widget View", MinimalisticView.class);
		viewIcons.put("Widget View", new Image(shell.getDisplay(), IconFactory.getIcon(ICON.WIDGET_VIEW)));
		views.put("Mail View", RSSMailView.class);
		viewIcons.put("Mail View", new Image(shell.getDisplay(), IconFactory.getIcon(ICON.MAIL_VIEW)));
		views.put("Newspaper View", NewspaperView.class);
		viewIcons.put("Newspaper View", new Image(shell.getDisplay(), IconFactory.getIcon(ICON.NEWSPAPER_VIEW)));
		views.put("Google Maps View", GoogleMapsView.class);
		viewIcons.put("Google Maps View", new Image(shell.getDisplay(), IconFactory.getIcon(ICON.GOOGLE_MAPS)));
		views.put("Calendar View", CalendarView.class);
		viewIcons.put("Calendar View", IconFactory.getIcon(shell.getDisplay(), ICON.CALENDAR));
	}

	/** 
	 * Retrieves settings by the name of the component.
	 * @param key
	 * @return null, if settings were not found for the component
	 */
	public Object getSettings(String key) {
		Object value = null;

		try {
			value =  getSettings(key, config.getProfileLocation());
		} catch (Exception e) {
			Trace.printVerbose("Could not find settings for " + key);			
		}

		return value;				
	}
	
	synchronized Object getSettings(String key, String location) throws FileNotFoundException, IOException, ClassNotFoundException {
		StringBuffer sb = new StringBuffer();
		sb.append(location).append("settings/").append(key);
		
		Object value = null;
		ObjectInputStream oi;
		oi = new ObjectInputStream(new FileInputStream(sb.toString()));
		value = oi.readObject();
		oi.close();
		return value;
	}
	
	/**
	 * Saves the settings of the component. They can be retrieved by asking for same name
	 * as key.
	 * @param key name of the module
	 * @param value settings
	 */
	public synchronized void saveSettings(String key, Serializable value) {

		try {
			ObjectOutputStream stream;

			StringBuffer sb = new StringBuffer();
			String configDir = sb.append(config.getProfileLocation()).append("settings/").toString();
			File dir = new File(configDir);
			if (!dir.isDirectory())
				dir.mkdir();
			
			stream = new ObjectOutputStream(new FileOutputStream(configDir + key,false));
			stream.writeObject(value);
			stream.close();
		} catch (FileNotFoundException e) {
			Trace.printException("Saving settings failed: file cannot be found");
			return;
		} catch (IOException e) {
			Trace.printException("Saving settings failed: I/O error");
			return;
		}

		Trace.printVerbose("Saved settings for " + key);
	}
	
	
	/**
	 * Reports an error through a relevant interface element.
	 * @param message
	 * @param shell a dialog shell in which the problem occurred and which must be blocked 
	 */
	public synchronized void reportError(String message, Shell parent) {
		if (currentView != null) {
			currentView.reportError(message, parent);
		} else {
			report(message);
		}
			
	}
	
	/**
	 * Reports an error through the view shell or the tray icon if the view is not chosen.
	 * @param message
	 */
	public void reportError(String message) {
		reportError(message, shell);
	}
	
	private void report(String message) {
		if ((tip != null) && !tip.isDisposed()) {
			tip.setMessage(message);
			tip.setVisible(true);
		} else
			Trace.printException(message);
	}
	
	public static void main(String[] args) {
		new Controller();
	}
	
	public void downloadFinished() {
		if (item != null)
			display.asyncExec(new Runnable() {
				public void run() {
					if (!item.isDisposed())
						item.setImage(trayIcon);	
				}
			});					
	}

	public void downloadStarted(Feed feed) {
		if (item != null)
			display.asyncExec(new Runnable() {
				public void run() {
					if (!item.isDisposed())
						item.setImage(trayIconBusy);
				}
			});					
	}

	/**
	 * Returns the name of the current user (or null if no profile loaded.)
	 * @return
	 */
	public String getUserName() {
		if (config != null) 
			return config.userName;
		else
			return null;
	}
	/**
	 * Checks the online/offline status.
	 * @return <code>true</code> if online, <code>false</code> if offline
	 */
	public boolean isOnline() {
		return isOnline;
	}
	/**
	 * Sets the online status.
	 * @param isOnline online status. 
	 */
	public void setOnline(boolean isOnline) {
		this.isOnline = isOnline;
	}
}
