package controller;

import java.io.File;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

import util.IconFactory;

/**
 * A prototype dialog which appears at the startup for profile selection and
 * management.
 * @author Kuat
 *
 */
class ProfileManager {
	private final Controller controller;
	private final java.util.List<ControllerSettings> profiles;
	private final List list;
	private final Shell shell;
	private ControllerSettings value;
	
	private void discoverProfiles() {		
		File dir = new File(GlobalPreferences.WORKING_LOCATION);
		
		if (!dir.exists())
			dir.mkdir();
		
		for (File file : dir.listFiles()) {
			if (file.isDirectory()) {
				try {
					String path = file.getCanonicalPath() + File.separator;
					Object o = controller.getSettings(Controller.class.getName(), path);
					if (o instanceof ControllerSettings) {
						ControllerSettings config = (ControllerSettings) o;
						if (config != null) {
							config.setProfileLocation(path);
							profiles.add(config);
						};
					}
				} catch (Exception e) {
					// do nothing
				} 
			}
		}		
	}
	
	public ProfileManager(final Shell parent, final Controller controller) {
		shell = new Shell(parent.getDisplay(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setImage(new Image(parent.getDisplay(), IconFactory.getIcon(IconFactory.ICON.PROFILE)));
		
		this.profiles = new ArrayList<ControllerSettings>();		
		this.controller = controller;
		discoverProfiles();

		shell.setText("Profile selection");
		shell.setLayout(new GridLayout(2,false));
		GridData gd;		
		
		Composite input = new Composite(shell, SWT.NONE);
		gd = new GridData(GridData.FILL_BOTH);
		input.setLayoutData(gd);
		input.setLayout(new GridLayout(1, false));
		list = new List(input, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		gd = new GridData(GridData.FILL_BOTH);
		gd.widthHint = 150;
		list.setLayoutData(gd);		
		list.setToolTipText("Select a profile to load it");
		for(ControllerSettings profile:profiles) {
			list.add(profile.userName, profiles.indexOf(profile));
		}
		if (list.getItemCount() > 0) 
			list.select(0);
		
		list.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent event) {
				if (event.keyCode == SWT.CR) {
					if (list.getSelectionIndex() != -1) {
						value = profiles.get(list.getSelectionIndex());
						shell.close();
					}
				}
			}
		});
		
		
		Composite buttons = new Composite(shell, SWT.NONE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		buttons.setLayoutData(gd);
		buttons.setLayout(new GridLayout(1, true));
		int style = GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL;
		
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Start");
		button.setImage(new Image(shell.getDisplay(), IconFactory.getIcon(IconFactory.ICON.STAR)));
		button.setLayoutData(new GridData(style));
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (list.getSelectionIndex() != -1) {
					value = profiles.get(list.getSelectionIndex());
					shell.close();
				}
			}
		});
		
		button = new Button(buttons, SWT.NONE);
		button.setText("Create");
		button.setImage(new Image(shell.getDisplay(), IconFactory.getIcon(IconFactory.ICON.NEW_FOLDER)));
		button.setLayoutData(new GridData(style));
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ControllerSettings config = createProfile(new ControllerSettings("", 0, GlobalPreferences.DEFAULT_VIEW, true));
				if (config != null) {
					value = config;
					shell.close();					
				}
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Delete");
		button.setImage(new Image(shell.getDisplay(), IconFactory.getIcon(IconFactory.ICON.DELETE)));
		button.setLayoutData(new GridData(style));
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				int index = list.getSelectionIndex(); 
				if (index != -1) {
					list.remove(index);
					ControllerSettings config = profiles.get(index);
					profiles.remove(index);
					deleteDir(new File(config.getProfileLocation()));										
				}
			}
		});
		
		button = new Button(buttons, SWT.NONE);
		button.setText("Exit");
		button.setImage(new Image(shell.getDisplay(), IconFactory.getIcon(IconFactory.ICON.EXIT)));
		button.setLayoutData(new GridData(style));
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});
		
		// Position shell in the center of the screen
		shell.pack();
	    Monitor primary = parent.getDisplay().getPrimaryMonitor();
	    Rectangle bounds = primary.getBounds();
	    Rectangle rect = shell.getBounds();
	    int x = bounds.x + (bounds.width - rect.width) / 2;
	    int y = bounds.y + (bounds.height - rect.height) / 2;
	    shell.setLocation(x, y);
	}
	
	ControllerSettings open(boolean forceDialog) {
		value = null;
		
		// Create a default profile if there is no other profile
		if (profiles.size() == 0) {
			value = new ControllerSettings(System.getProperty("user.name"), 0, GlobalPreferences.DEFAULT_VIEW,true);
			setupProfile(value);
			return value;
		}
		
		if (!forceDialog && (profiles.size() == 1)) {
			return profiles.get(0);
		}
		
		shell.open();
		
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}		
		
		return value;
	}
	
	void setupProfile(ControllerSettings config) {
		String location = "";
		try {
			MessageDigest md5hasher;

			md5hasher = MessageDigest.getInstance("MD5");
			md5hasher.update(config.userName.getBytes());
			md5hasher.update(String.valueOf(System.currentTimeMillis()).getBytes());
			md5hasher.update(config.view.getName().getBytes());
			
			location = new BigInteger(1, md5hasher.digest()).toString(16);
		} catch (NoSuchAlgorithmException e) {
			
		}
		
		boolean success = false;
		while(!success) {
			String path = GlobalPreferences.WORKING_LOCATION + location;
			File temp = new File(path);
			if (!temp.exists()) {
				config.setProfileLocation(path + File.separator);
				success = true;
			} else
				location = config.userName + System.currentTimeMillis();		
		}
	}
	
	ControllerSettings createProfile(ControllerSettings original) {		
		NewProfileDialog dialog = new NewProfileDialog(shell, controller);
		ControllerSettings config = dialog.open(original);
		
		if (config == null)
			return null;
		
		setupProfile(config);
		
		return config;
	}
	
    private static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
    
        // The directory is now empty so delete it
        return dir.delete();
    }
}
