package controller;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import util.IconFactory;

/**
 * A wrapper around the SWT menu allowing to access the toplevel submenus by name.
 * @author oleg
 */
public class StandardMenu {
	private final Menu menu; 
	private Map<String, Menu> subMenus = new HashMap<String, Menu>();
	/**
	 * Creates a new standard menu.
	 * @param parent parent shell
	 */
	public StandardMenu(Shell parent) {
		menu = new Menu(parent, SWT.BAR);
	}
	/**
	 * Retrieves the underlying SWT menu.
	 * @return SWT menu
	 */
	public Menu getSWTMenu() {
		return menu;
	}
	/**
	 * Accesses a toplevel submenu by its name.
	 * @param name the name of the menu
	 * @return the submenu with that name or null if no such submenu found
	 */
	public Menu getSubMenu(String name) {
		return subMenus.get(name);
	}
	/**
	 * Adds a new toplevel submenu with a unique name.
	 * @param name name of the submenu
	 * @return the created submenu or null if the name is not unique
	 */
	public Menu addSubMenu(String name) {
		if (subMenus.containsKey(name)) return null;
		else {
			MenuItem item = new MenuItem(menu, SWT.CASCADE);
			item.setText(name);
			Menu submenu = new Menu(menu);
			item.setMenu(submenu);
			subMenus.put(name, submenu);
			return submenu;
		}
	}
	/**
	 * Adds a separator to the toplevel submenu with a given name.
	 * @param menuName name of the submenu
	 */
	public void addSeparator(String menuName) {
		new MenuItem(subMenus.get(menuName),SWT.SEPARATOR);
	}
	/**
	 * Adds a menu item to the toplevel submenu with a given name. 
	 * @param menuName the name of the menu
	 * @param name the name of the item
	 * @param selectionListener a listener for the <code>SWT.Selected</code> event
	 * @param image image to display or null 
	 */
	public void addItem(String menuName, String name, Listener selectionListener, Image image) {
		Menu menu = subMenus.get(menuName);
		if (menu != null) {
			MenuItem item = new MenuItem(menu,SWT.CASCADE);
			item.setText(name);
			item.addListener(SWT.Selection, selectionListener);
			item.setImage(image);
		}
	}
	/**
	 * Adds a menu item to the toplevel submenu with a given name. 
	 * @param menuName the name of the menu
	 * @param name the name of the item
	 * @param selectionListener a listener for the <code>SWT.Selected</code> event
	 * @param icon standard icon to display
	 */
	public void addItem(String menuName, String name, Listener selectionListener, IconFactory.ICON icon) {
		Image image;
		if (icon == null) image = null;
		else image = IconFactory.getIcon(menu.getDisplay(), icon);
		addItem(menuName, name, selectionListener, image);
	}
	
}