package data;

/**
 * Class representing an article in a feed. 
 * Articles are considered to be equivalent if they have same:
 * -- title,
 * -- feed,
 * -- published date.
 * @author Kuat
 *
 */
public class ArticleInfo {

	private String title;
	private String authors;
	private String url;
	private Feed parent;
	private long publishedDate;
	private boolean isRead;
	private boolean isStarred;
	private GeoPoint geo;
	
	/**
	 * Returns the title of the articles.
	 * @return
	 */
	public String getTitle() {
		return title;
	}
	
	/**
	 * Returns the parent feed containing the feed.
	 * @return
	 */
	public Feed getFeed() {
		return parent;
	}
	
	/**
	 * Returns the published date of the article.
	 * @return
	 */
	public long getPublishedDate() {
		return publishedDate;
	}
	
	/**
	 * Indicates whether article is read (true) or not read (false).
	 * @return
	 */
	public boolean isRead() {
		return isRead;
	}
	
	/**
	 * Indicates whether article is starred (true) or not.
	 * @return
	 */
	public boolean isStarred() {
		return isStarred;		
	}
	
	/**
	 * Returns the authors of the article; can be null
	 * @return
	 */
	public String getAuthors() {
		return authors;
	}
	
	/**
	 * Web link to the content page; can be null
	 * @return
	 */
	public String getURL() {
		return url;
	}
		
	/** 
	 * Geographic position associated with the article. 
	 * @return
	 */
	public GeoPoint getPoint() {
		return geo;
	}
	
	/**
	 * Constructs ArticleInfo.
	 */
	public ArticleInfo(String title,
			Feed parent,
			long publishedDate,
			String authors,
			String url,
			boolean isRead,
			boolean isStarred) {
		assert title!= null;
		assert parent != null;
		assert publishedDate > 0;
		
		this.title = title;
		this.parent = parent;
		this.publishedDate = publishedDate;
		this.isRead = isRead;
		this.isStarred = isStarred;
		this.authors = authors;
		this.url = url;
	}

	/**
	 * Article constructor.
	 * @param title non-null
	 * @param parent non-null feed
	 * @param publishedDate  
	 * @param authors
	 * @param url
	 * @param isRead boolean
	 * @param isStarred boolean
	 * @param geo 
	 */
	public ArticleInfo(String title,
			Feed parent,
			long publishedDate,
			String authors,
			String url,
			boolean isRead,
			boolean isStarred,
			GeoPoint geo) {
		this(title, parent, publishedDate, authors, url, isRead, isStarred);
		this.geo = geo;
	}
	
	void setRead(boolean status) {
		isRead = status;
	}
	
	void setStarred(boolean status) {
		isStarred = status;
	}
	
	void setPoint(GeoPoint geo) {
		this.geo = geo;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof ArticleInfo)) 
			return false;
		ArticleInfo ai = (ArticleInfo) obj;
		return ai.getTitle().equals(title) && 
				ai.getPublishedDate() == publishedDate &&
				ai.getFeed().equals(parent);
	}

	@Override
	public int hashCode() {		
		return title.hashCode() + parent.hashCode();
	}

	@Override
	public String toString() {	
		return this.title + " DATE: " + this.publishedDate + " FEED: " + this.parent.getName(); 
	}
}
