package data;

import java.util.Collection;
import java.util.Iterator;

import view.RSSObserver;
import download.FeedChangeObserver;


/**
 * Interface to the data including containers, feeds, and articles.
 * Accesses backend to retrieve the objects.
 * Performs all persistent data operations.
 * @author Kuat
 *
 */
public interface DataManager {

	/**
	 * Performs initialization of external resources.
	 * @param profile profile location
	 * @return true if succeeds
	 */
	public boolean initialize(String profile);
	
	/**
	 * Safely releases any hold resources and performs
	 * persistence of current state (though not critical.)
	 */
	public void terminate();
	
	public Collection<Channel> getChannels ();	
	public Collection<Virtual> getVirtuals ();	
	public Collection<Feed> getFeeds();
	
	/**
	 * Retrieves all articles in the feed. 
	 * @param parent registered feed 
	 * @return an iterator over articles or null if an exception has occurred
	 */
	public Iterator<ArticleInfo> getArticles (Feed parent);

	/**
	 * Retrieves all articles in a virtual folder.
	 * @param virtual
	 * @return an iterator over articles or null if an exception has occurred
	 */
	public Iterator<ArticleInfo> getArticles (Virtual virtual);

	/**
	 * Retrieves the article by its title, feed, and published date.
	 * @param title article title
	 * @param feed feed containing the article
	 * @param date published date
	 * @return matching article or null if not found
	 */
	public ArticleInfo getArticle(String title, Feed feed, long date);
	
	/**
	 * Returns content of an article.
	 * @param article
	 * @return content of an article info or null if not found (or some exception occurred)
	 */
	public ArticleContent getContent(ArticleInfo article);
	
	public int unreadArticles(Feed parent);
	public int unreadArticles(Virtual virtual);
	public int numberArticles(Feed parent);
	public int numberArticles(Virtual virtual);
	
	/**
	 * Returns a registered feed by its URL.
	 * @param url
	 * @return a feed or null if no feed with such URL was registered.
	 */
	public Feed getFeed(String url);
	
	/**
	 * Returns a registered channel by its name.
	 * @param name
	 * @return a channel or null if there is no channel with given name.
	 */
	public Channel getChannel(String name);
	
	/**
	 * Returns a registered virtual by its name.
	 * @param name
	 * @return a virtual or null if there is no virtual with given name.
	 */
	public Virtual getVirtual(String name);
	
	/** Adds a feed.
	 * @param feed non-null feed to be added
	 * @requires channel is registered in the data service
	 * @throws DataException if duplicate feed with same URL is registered or persistence exception
	 */
	public void addFeed(Feed feed) throws DataException;
	
	/** Moves a feed to destination channel.
	 * @param feed non-null registered feed to be moved
	 * @param destination target registered channel
	 * @throws DataException
	 */
	public void moveFeed(Feed feed, Channel destination) throws DataException;
	
	/** Renames a feed.
	 * @param feed registered feed
	 * @param name non-null
	 * @throws DataException
	 */
	public void renameFeed(Feed feed, String name) throws DataException;
	
	/** Removes a feed with cascade delete of all feed's articles.
	 * @param feed registered feed 
	 * @throws DataException
	 */
	public void removeFeed(Feed feed) throws DataException;
	
	/** Adds a channel with the given name.
	 * @param name of a new channel
	 * @requires name is unique to channels
	 * @return a channel instance
	 * @throws DataException
	 */
	public Channel addChannel(String name) throws DataException;
	
	/** Renames a channel.
	 * @param channel registered channel
	 * @param name is unique to channels
	 * @throws DataException
	 */
	public void renameChannel(Channel channel, String name) throws DataException;
	
	/** Removes a channel. You need to remove all inner feed first.
	 * @param channel registered channel.
	 * @throws DataException
	 */
	public void removeChannel(Channel channel) throws DataException;	

	/** Adds a virtual.
	 * @param virtual non-null virtual with unique name among virtuals
	 * @throws DataException
	 */
	public void addVirtual(Virtual virtual) throws DataException;

	/** Removes a virtual.
	 * @param virtual registered virtual
	 * @throws DataException
	 */
	public void removeVirtual(Virtual virtual) throws DataException;
	
	/** Adds an article.
	 * @param article non-null meta-information 
	 * @param content non-null description
	 * @requires article's feed is registered
	 * @throws DataException
	 */
	public void addArticle(ArticleInfo article, ArticleContent content) throws DataException;

	/** Batch add of articles.
	 * @param articles non-null collection of articles
	 * @param contents non-null collection of descriptions
	 * @requires articles' feeds are registered
	 * @requires arrays are parallel
	 * @throws DataException
	 */
	public void addArticles(Collection<ArticleInfo> articles, Collection<ArticleContent> contents) throws DataException;
	
	/** Removes an article. 
	 * @param article non-null registered
	 * @throws DataException
	 */
	public void removeArticle(ArticleInfo article) throws DataException;
	
	/**
	 * Changes the update interval of a feed.
	 * @param feed non-null registered
	 * @param interval positive interval in milliseconds
	 * @throws DataException
	 */
	public void setUpdateInterval(Feed feed, long interval) throws DataException;
	
	/** Sets the read status of an article.
	 * @param article non-null registered
	 * @param status desired status
	 * @throws DataException
	 */
	public void setReadArticle(ArticleInfo article, boolean status) throws DataException;
		
	/**
	 * Sets the star status of an article.
	 * @param article non-null registered
	 * @param status desired
	 * @throws DataException
	 */
	public void setStarredArticle(ArticleInfo article, boolean status) throws DataException;

	/**
	 * Sets the geographic location of an article.
	 * @param article non-null registered
	 * @param point desired location
	 * @throws DataException
	 */
	public void setLocation(ArticleInfo article, GeoPoint point) throws DataException;
	
	/**
	 * Returns the most frequent terms appearing in titles and article contents.
	 * The order given by the iterator is the order of increasing frequency.
	 * @param count number of words to return
	 * @return a collection of strings
	 */
	public Collection<String> getMostFrequentWords(int count);
	
	/**
	 * Adds a view observer.
	 */
	public void addViewListener(RSSObserver ml);
	
	/**
	 * Removes a view observer.
	 */
	public void removeViewListener(RSSObserver ml);
		
	public void addFeedListener(FeedChangeObserver o);
	
	public void removeFeedListener(FeedChangeObserver o);		
}