package data;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

import util.Trace;
import view.RSSObserver;
import data.DataException.Cause;
import download.FeedChangeObserver;

/**
 * Implementation of DataManager interface which uses Derby database as a backend.
 * Serves as a factory of ADTs.
 * The objects kept in memory are collections of feeds, virtuals, and channels. 
 * The respective operations on these objects are performed on this transient objects as well as
 * their persistent copies on the back-end side using query parser.
 * The articles are instantiated on each request and not referenced in any way by this class.
 * This allows them to be quickly garbage-collected and also not having to load complete database on startup.
 * We may decide to implement our own cache later, but current implementation performs sufficiently well.
 * 
 * The class also employs full-text indexer to index all articles text. Its job is to synchronize
 * index and database.
 * 
 * The class notifies each listener on persistent data mutation event. 
 * 
 * @author Kuat
 *
 */
public class DataManagerDBImpl
implements DataManager {

	// Singleton instance
	private static DataManager singleton = new DataManagerDBImpl();

	private Database db;
	private Materializer parser;
	private FullTextIndexer indexer;
	
	// Name references
	static Map<String, Feed> feeds;	
	static Map<String, Virtual> virtuals;
	static Map<String, Channel> channels;
	
    // Observers
    private Vector<RSSObserver> listeners;
    private Vector<FeedChangeObserver> feedListeners;
        
    private boolean instanceExists = false; 
    
    public static DataManager getInstance() {
    	return singleton;
    }
    
	private DataManagerDBImpl() {
		db = Database.getInstance();
		parser = Materializer.getInstance();
		indexer = FullTextIndexer.getInstance();
	}
	
	public boolean initialize(String profile) {
		assert !instanceExists;

		feeds = new ConcurrentHashMap<String, Feed>();	
		virtuals = new ConcurrentHashMap<String, Virtual>();
		channels = new ConcurrentHashMap<String, Channel>();
	    listeners = new Vector<RSSObserver>();
	    feedListeners = new Vector<FeedChangeObserver>();
 
		try {
			db.start(profile);
		} catch (Exception e) {
			Trace.printException(e);
			return false;
		}
					
		parser.initialize(db);
		
		loadContainers();
		loadFeeds();
		
		boolean result = indexer.initialize(this, profile + "index/");		

		instanceExists = true;
		
		return result;
	}

	public void terminate() {
		assert instanceExists;
		
		flushFeeds();
		indexer.deinitialize();
		db.shutdown();		
		
		instanceExists = false;
	}
	
	private void flushFeeds() {
		for (Feed f: feeds.values()) {
			try {
				parser.updateFeed(f);
			} catch(SQLException e) {
				Trace.printException("Could not store feeds.");
			}
		}
	}

	private void loadFeeds() {
        try
        {
            Statement stmt = db.createStatement();
            ResultSet results = stmt.executeQuery("select * from FEED");
                            
            while(results.next())
            {
            	Feed feed = parser.newFeed(results);
            	feeds.put(feed.getURL(), feed);
    	    }
            results.close();
            stmt.close();
        }
        catch (SQLException sqlExcept)
        {
            Trace.printException(sqlExcept);
        }
	}
	
	private void loadContainers() {
        try
        {
            Statement stmt = db.createStatement();
            ResultSet results = stmt.executeQuery("select * from VIRTUAL");
                
            while(results.next())
            {
                Virtual result = parser.newVirtual(results);
                virtuals.put(result.getName(), result);
            }
            results.close();
            stmt.close();
            
            stmt = db.createStatement();
            results = stmt.executeQuery("select * from CHANNEL");
            
            while(results.next())
            {
                Channel result = parser.newChannel(results);                
                channels.put(result.getName(), result);
            }
            results.close();
            stmt.close();
        }
        catch (SQLException sqlExcept)
        {
            Trace.printException(sqlExcept);
        }		
	}
	
	/*******************************************************
	 * OBSERVERS
	 */
	
	private Iterator<ArticleInfo> getArticles(String query, int limit) {
		try
        {
            final Statement stmt = db.createStatement();           
            stmt.setMaxRows(limit);
            final ResultSet results = stmt.executeQuery(query);
            
            return (new Iterator<ArticleInfo>(){
				boolean gotNext = false;
				ArticleInfo info = null;
				boolean closed = false;

				public boolean hasNext() {
					try {
						if (closed)
							return false;
						
						if (!gotNext) { 
							if (results.next()) {
								gotNext = true;
								info = parser.newArticleInfo(results);
								return true;
							} else {
								results.close();
								stmt.close();
								closed = true;
								return false;
							} 
						} else
							return true;
					} catch (SQLException e) {
						Trace.printException(e);
						return false;
					}
				}

				public ArticleInfo next() {
			        try {
			        	ArticleInfo result;
			        	if (gotNext) {
			        		gotNext = false;
			        		result = info;
			        		info = null;
			        		return result;
			        	} else {
			        		if (!results.next())
			        			throw new NoSuchElementException();
			        		return parser.newArticleInfo(results);
			        	}
			        } catch (SQLException e) {
			        	Trace.printException(e);
			        	return null;
			        }
				}

				public void remove() {
				}
            	
            });
        }
        catch (SQLException sqlExcept)
        {
        	Trace.printException("FAILED QUERY: " + query);
        	Trace.printException(sqlExcept.getMessage());
    		return null;
        }        		
	}

	public Iterator<ArticleInfo> getArticles(Virtual virtual) {		
		Trace.printVerbose(virtual.getQuery());
		if (virtual.isFullText()) {
			Collection<ArticleInfo> articles = indexer.getArticles(virtual);
			return (articles == null ? null : articles.iterator());
		} else
			return getArticles(virtual.getQuery(), virtual.getLimit());
	}

	public Iterator<ArticleInfo> getArticles(Feed parent) {
		if (!feeds.containsValue(parent))
			return null;
		
		return getArticles("select " + Materializer.articleInfoColumns + " from ARTICLE where feed=" + parser.getId(parent), 0);
	}

	public Collection<Feed> getFeeds() {
		return Collections.unmodifiableCollection(feeds.values());
	}
	
	public Collection<Channel> getChannels() {
		return Collections.unmodifiableCollection(channels.values());
	}
	
	public Collection<Virtual> getVirtuals() {
		return Collections.unmodifiableCollection(virtuals.values());
	}
	
	public ArticleContent getContent(ArticleInfo article) {
		try {
			return parser.getContent(article);
		} catch (SQLException e) {
			Trace.printException(e);
    		return null;
		}
	}
	
	private int numberArticles(String query){
		int n = 0;
		
        try
        {
            Statement stmt = db.createStatement();
            ResultSet results = stmt.executeQuery(query);
                            
            if (results.next())
            	n = results.getInt(1);
            results.close();
            stmt.close();
        }
        catch (SQLException sqlExcept)
        {
        	Trace.printException(query);
            Trace.printException(sqlExcept);
        }
        
		return n;
	}
	
	public int unreadArticles(Feed feed){
		if (!feeds.containsValue(feed))
			return -1;
		
		return numberArticles(
			"select COUNT(*) from ARTICLE where readflag = 0 and feed=" + parser.getId(feed));
	}
	
	public int unreadArticles(Virtual virtual) {
		assert virtual != null;
		
		if (virtual.isFullText()) {
			return indexer.getArticles(VirtualFactory.unreadOnly(virtual)).size();
		} else
			return numberArticles(
					"select COUNT(*) from (" + virtual.getQuery() + ") as NUMBERARTICLES where NUMBERARTICLES.readflag = 0"); 
	}
	
	public int numberArticles(Feed feed) {
		if (!feeds.containsValue(feed))
			return -1;
		
		return numberArticles(
			"select COUNT(*) from ARTICLE where feed = " + parser.getId(feed));		
	}
	
	public int numberArticles(Virtual virtual) {
		assert virtual != null;
		
		if (virtual.isFullText()) {
			return indexer.getArticles(virtual).size();
		} else
			return numberArticles(
					"select COUNT(*) from (" + virtual.getQuery() + ") as NUMBERARTICLES");
	}
	

	public ArticleInfo getArticle(String title, Feed feed, long date) {
		if (!feeds.containsValue(feed))
			return null;

		ArticleInfo article = null;
        try
        {
        	article = parser.getArticle(title, feed, date);
        }
        catch (SQLException sqlExcept)
        {
            Trace.printException(sqlExcept);
        }		

		return article;
	}
	
	public Feed getFeed(String url){
		return feeds.get(url);
	}

	public Channel getChannel(String name) {
		return channels.get(name);
	}
	
	public Virtual getVirtual(String name) {
		return virtuals.get(name);
	}
	
	/*******************************************************
	 * MUTATORS
	 */	
	
	public synchronized void addFeed(Feed feed) throws DataException {
		if (feed == null) throw new DataException(null, Cause.NULL_POINTER);
		if (!channels.containsValue(feed.getChannel())) throw new DataException(feed.getChannel(), Cause.MISSING_PARENT);
		
		if (feeds.containsKey(feed.getURL())) {
			throw new DataException(feed, Cause.UNIQUENESS);
		} 
		
		try {
			parser.flushFeed(feed);
		} catch (SQLException e) {
			throw new DataException(e);
		}
		feeds.put(feed.getURL(), feed);

		synchronized(listeners) {
			for(RSSObserver o: this.listeners)
				o.newFeed(feed);
		}
	}

	private synchronized void persistArticle(ArticleInfo article, ArticleContent content) throws DataException{
		if (article == null) throw new DataException(null, Cause.NULL_POINTER);
		if (content == null) throw new DataException(null, Cause.NULL_POINTER);
		if (!feeds.containsValue(article.getFeed())) throw new DataException(article.getFeed(), Cause.MISSING_PARENT);
		
		try {
			parser.flushArticle(article, content);
		} catch (SQLException e) {				
			throw new DataException(e);
		}
		
		indexer.addArticle(article, content);
	}
	
	public void addArticle(ArticleInfo article, ArticleContent content) throws DataException{
		persistArticle(article, content);

		synchronized(listeners) {
			Collection<ArticleInfo> articles = new Vector<ArticleInfo>(1);
			articles.add(article);
			for (RSSObserver o: this.listeners)  
				o.newArticles(articles);
		}		
	}
	
	public void addArticles(Collection<ArticleInfo> articles, Collection<ArticleContent> contents) throws DataException{
		if (articles == null) throw new DataException(null, Cause.NULL_POINTER);
		if (contents == null) throw new DataException(null, Cause.NULL_POINTER);
		assert articles.size() == contents.size();
		
		Iterator<ArticleInfo> articlesIterator = articles.iterator();
		Iterator<ArticleContent> contentsIterator = contents.iterator();

		// We capture duplicate article violation, and whenever some unexpected exception occurs (which shouldn't
		// happen) we do notify listeners about successful insertions but after that throw an exception
		Collection<ArticleInfo> successful = new LinkedList<ArticleInfo> ();
		
		boolean notifyException = false;;
		while (articlesIterator.hasNext()) {
			boolean failed = false;
			ArticleInfo current = articlesIterator.next();
			ArticleContent currentContent = contentsIterator.next();
			try {
				persistArticle(current, currentContent);
			} catch (DataException e) {
				failed = true;
				if (!e.isUniquenessViolation()) notifyException = true;
			}
			if (!failed) successful.add(current);
		}

		synchronized(listeners) {
			for (RSSObserver o: this.listeners)  
				o.newArticles(successful);
		}		
		
		if (notifyException) throw new DataException("Unexpected error while persisting an article");
	}

	public synchronized Channel addChannel(String name) throws DataException{		
		if (channels.containsKey(name))
			throw new DataException(name, Cause.UNIQUENESS);
		
		Channel result = null;
		try {
			result = parser.flushChannel(name);
		} catch (SQLException e) {
			throw new DataException(e);
		}
		channels.put(name, result);
		
		synchronized (listeners) {
			for(RSSObserver r: listeners)
				r.newChannel(result);
		}
		
		return result;		
	}

	public synchronized void addVirtual(Virtual virtual) throws DataException {
		if (virtual == null) throw new DataException(null, Cause.NULL_POINTER);		
		if (virtuals.containsKey(virtual.getName()))
			throw new DataException(virtual, Cause.UNIQUENESS);
		
		try {
			parser.flushVirtual(virtual);			
		} catch (SQLException e) {
			throw new DataException(e);
		}
		virtuals.put(virtual.getName(), virtual);
		
		synchronized(listeners) {
			for(RSSObserver r: listeners)
				r.newVirtual(virtual);
		}
	}

	public synchronized void moveFeed(Feed feed, Channel destination) throws DataException{
		if (!feeds.containsValue(feed)) throw new DataException(feed, Cause.MISSING);
		if (!channels.containsValue(destination)) throw new DataException(destination, Cause.MISSING);		
		
		Channel old = feed.getChannel();
		feed.setChannel(destination);		

		try {
			parser.updateFeed(feed);
		} catch (SQLException e) {
			feed.setChannel(old);
			throw new DataException(e);
		}
	}
	
	public synchronized void renameFeed(Feed feed, String name) throws DataException {
		if (name == null) throw new DataException(null, Cause.NULL_POINTER);
		if (!feeds.containsValue(feed)) throw new DataException(feed, Cause.MISSING);
				
		feed.setName(name);
		
		try {
			parser.updateFeed(feed);
		} catch (SQLException e) {
			throw new DataException(e);
		}
	}

	public synchronized void removeArticle(ArticleInfo article) throws DataException {
		if (article == null) throw new DataException(null, Cause.NULL_POINTER);
		if (!feeds.containsValue(article.getFeed())) throw new DataException(article.getFeed(), Cause.MISSING_PARENT);
		
		try {
			parser.removeArticle(article);
		} catch (SQLException e) {
			throw new DataException(e);
		}
		
		indexer.removeArticle(article);
	}

	public synchronized void removeChannel(Channel channel) throws DataException {
		if (!channels.containsValue(channel)) throw new DataException(channel, Cause.MISSING);
		
		try {
			parser.removeChannel(channel);
		} catch (SQLException e) {
			throw new DataException(e);
		}
		channels.remove(channel.getName());
	}
	
	public synchronized void removeFeed(Feed feed) throws DataException {
		if (!feeds.containsValue(feed)) throw new DataException(feed, Cause.MISSING);
		
		try {
			parser.removeFeed(feed);
		} catch (SQLException e) {
			throw new DataException(e);
		}
		feeds.remove(feed.getURL());
		
		// DB removes all articles in the feed by cascade but we
		// need to do same thing with index
		indexer.removeFeed(feed);
		
		synchronized (feedListeners) {
			for(FeedChangeObserver r: feedListeners)
				r.feedRemoved(feed);
		}
	}

	public synchronized void removeVirtual(Virtual virtual) throws DataException {
		if (!virtuals.containsValue(virtual)) throw new DataException(virtual, Cause.MISSING);
		
		try {
			parser.removeVirtual(virtual);
		} catch (SQLException e) {
			throw new DataException(e.getMessage());
		}
		virtuals.remove(virtual.getName());
	}

	public synchronized void renameChannel(Channel channel, String name) throws DataException{
		if (!channels.containsValue(channel)) throw new DataException(channel, Cause.MISSING);
		
		if (channels.containsKey(name))
			throw new DataException(name, Cause.UNIQUENESS);
		
		channels.remove(channel.getName());
		channel.setName(name);
		channels.put(name, channel);
		
		try {
			parser.updateChannel(channel);
		} catch (SQLException e) {
			throw new DataException(e);
		}
	}
	
	public void setUpdateInterval(Feed feed, long interval) throws DataException {
		if (!feeds.containsValue(feed)) throw new DataException(feed, Cause.MISSING);
		assert interval > 0;
		
		feed.setUpdateInterval(interval);
		
		synchronized (feedListeners) {
			for(FeedChangeObserver r: feedListeners)
				r.updateIntervalChanged(feed);
		}
	}


	public void setReadArticle(ArticleInfo article, boolean status) throws DataException{
		if (article == null) throw new DataException(null, Cause.NULL_POINTER);
		if (!feeds.containsValue(article.getFeed())) throw new DataException(article.getFeed(), Cause.MISSING_PARENT);		
		
		article.setRead(status);
		try {
			parser.updateArticle(article);
		} catch (SQLException e) {
			throw new DataException(e.getMessage());
		}
	}
	
	public void setStarredArticle(ArticleInfo article, boolean status) throws DataException {
		if (article == null) throw new DataException(null, Cause.NULL_POINTER);
		if (!feeds.containsValue(article.getFeed())) throw new DataException(article.getFeed(), Cause.MISSING_PARENT);		

		article.setStarred(status);
		try {
			parser.updateArticle(article);			
		} catch (SQLException e) {
			throw new DataException(e.getMessage());
		}
	}
	
	public void setLocation(ArticleInfo article, GeoPoint point) throws DataException {
		if (article == null) throw new DataException(null, Cause.NULL_POINTER);
		if (!feeds.containsValue(article.getFeed())) throw new DataException(article.getFeed(), Cause.MISSING_PARENT);		

		article.setPoint(point);
		try {
			parser.updateArticle(article);			
		} catch (SQLException e) {
			throw new DataException(e.getMessage());
		}
	}
	
	public Collection<String> getMostFrequentWords(int count) {
		return indexer.mostFrequentTerms(count);
	}

	/*******************************************************
	 * LISTENER REGISTRATION
	 */

	public void addViewListener(RSSObserver ml) {
		synchronized(listeners) {
			listeners.add(ml);
		}
	}
	
	public void removeViewListener(RSSObserver ml) {
		synchronized(listeners) {
			listeners.remove(ml);
		}
	}

	public void addFeedListener(FeedChangeObserver o) {
		synchronized(feedListeners) {
			feedListeners.add(o);
		}		
	}

	public void removeFeedListener(FeedChangeObserver o) {
		synchronized(feedListeners) {
			feedListeners.remove(o);
		}		
	}
}

