package data;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import util.Trace;

/**
 * Derby database backend
 * @author Kuat
 *
 */
class Database {

	private static Database singleton = new Database();
	
	// Connection string to embedded driver
	private static String dbURL = "jdbc:derby:archive;user=se12;password=mine";
	
    // jdbc Connection
    private static Connection conn = null;   
        
    /**
     * Return singleton instance.
     * @return
     */
    public static Database getInstance() {
    	return singleton;
    }
        
    private Database() { 
    }
    
    /**
     * Initializes by establishing a connection to the database.
     * @param dir location of the database
     */
    public void start(String dir) throws Exception {
    	if (conn != null) 
    		throw new Exception("Database connection is already established");
    	
        // Set the db system directory.
        System.setProperty("derby.system.home", dir);              

        createConnection();
    }

	private void createConnection() throws Exception{
    	boolean tryCreate = false;
    	
        try
        {
            Class.forName("org.apache.derby.jdbc.EmbeddedDriver").newInstance();
            //Get a connection          
            conn = DriverManager.getConnection(dbURL + ";create=false"); 
        }
        catch (SQLException except)
        {
        	tryCreate = true;            
        } catch (InstantiationException e) {
			Trace.printException(e);
		} catch (IllegalAccessException e) {
			Trace.printException(e);
		} catch (ClassNotFoundException e) {
			Trace.printException(e);
		}        
		
		if (tryCreate) {
	        try
	        {
	            Class.forName("org.apache.derby.jdbc.EmbeddedDriver").newInstance();
	            //Get a connection          
	            conn = DriverManager.getConnection(dbURL + ";create=true"); 
	        }
	        catch (Exception except)
	        {
	            Trace.printException(except);       
	            throw new Exception();
	        }
	        
	        Trace.printVerbose("Creating a new database.");
	        createTables();	        
		}
    }
    
	/**
	 * Safe release of connection
	 *
	 */
    public void shutdown() {
        try
        {
            if (conn != null)
            {
            	conn.close();
                DriverManager.getConnection("jdbc:derby:;shutdown=true");                               
            }           
        }
        catch (SQLException sqlExcept)
        {
            Trace.printVerbose("Database is successfully closed");
            conn = null;
        }
    }
    
    private void createTables() {

		//FileReader fin;
    	InputStreamReader fin;
		// TODO: change to InputStreamReader when packing in a jar -- Oleg
		try {
			// TODO: use this when packing in a jar -- Oleg
			fin = new InputStreamReader(Database.class.getResourceAsStream("/res/DB.sql"));
			// fin = new FileReader(GlobalPreferences.RESOURCE_LOCATION + "DB.sql");
			BufferedReader reader = new BufferedReader(fin);			

			String line;
			StringBuffer buffer = new StringBuffer();
			while((line = reader.readLine()) != null) {
				if (line.startsWith("--")) {			
					String command = buffer.toString().trim();
					if (command.length() > 0)
						executeCommand(command);
					buffer.setLength(0);
				} else {
					line = line.trim();
					buffer.append(" " + line);
				}
			}
			String command = buffer.toString().trim();
			if (command.length() > 0)
				executeCommand(command);

			conn.setAutoCommit(true);
			
		} catch (FileNotFoundException e) {
			Trace.printException(e);
		} catch (IOException e) {
			Trace.printException(e);
		} catch (SQLException e) {
			Trace.printException(e);
		}
    }
    
    /**
     * Executes a command
     * @param command command string
     * @return true if succeeds, false otherwise
     */
    private boolean executeCommand(String command) {
    	boolean result = true;
    	try
    	{
    		Statement stmt = conn.createStatement();
    		Trace.printVerbose(command);
    		stmt.execute(command);
    		stmt.close();
    	}
    	catch (SQLException sqlExcept)
    	{
    		Trace.printException(sqlExcept);
    		result = false;
    	}    	
    	
    	return result;
    }
    
    /**
     * Creates a statement from the database connection.
     * @return
     * @throws SQLException
     */
    public Statement createStatement() throws SQLException {
    	return conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
    }
    
    /**
     * Prepares a statement from the database connection.
     * @param query format string 
     * @return
     * @throws SQLException
     */
    public PreparedStatement createPreparedStatement(String arg) throws SQLException {
    	return conn.prepareStatement(arg);
    }

	public PreparedStatement createPreparedStatement(String string, int key) throws SQLException {
		return conn.prepareStatement(string, key);
	}
}
