package data;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.BooleanClause.Occur;
import org.apache.lucene.util.PriorityQueue;
import org.htmlparser.Parser;
import org.htmlparser.beans.StringBean;
import org.htmlparser.util.ParserException;

import util.Trace;
import controller.GlobalPreferences;

/**
 * Full text indexer based on Lucene search engine.
 * Indexes content of articles and some of its relevant meta-data.
 * Assumes that the mutable part of articles (read status) is not indexed.
 * @author Kuat
 *
 */
class FullTextIndexer {

	private static final FullTextIndexer singleton = new FullTextIndexer(); 
		
	private static ExecutorService worker;
	
	private String indexDir;
	private boolean ENABLED = GlobalPreferences.INDEX_ENABLED;
	
	private IndexWriter writer;
	private IndexReader reader;
	private Analyzer writerAnalyzer = new StandardAnalyzer();
	private Collection<String> stopWords;
	
	private DataManager data; 
	
	private FullTextIndexer() {		
	}
	
	public static FullTextIndexer getInstance(){		
		return singleton;
	}
	
	private void recover() {
		Trace.printVerbose("Synchronizing with database.");
		Iterator<ArticleInfo> iterator = data.getArticles(VirtualFactory.empty("Random"));						
		while(iterator.hasNext()) {
			final ArticleInfo article = iterator.next();
			final ArticleContent content= data.getContent(article);

			worker.submit(new Runnable() {
				public void run() {
					addArticle(article, content);						
				}
			});				
		}
	}
	
	/**
	 * Initializes the indexing service. If index cannot be opened,
	 * tries to recover by pushing all elements in the database into the index.
	 * @param data data manager
	 * @param indexDir location of the index
	 * @return true iff succeeds
	 */
	public boolean initialize(final DataManager data, String indexDir) {
		if (!ENABLED) return true;
		
		this.data = data;
		this.indexDir = indexDir;
		this.reader = null;
		
        boolean failed = false;
        
        try {
			writer = getWriter();
		} catch (IOException e) {
			failed = true;
		}      
		
		worker = Executors.newSingleThreadExecutor();

        if (failed) {
       		// create a new index writer and overwrite the old index
       		try {
				writer = new IndexWriter(indexDir, writerAnalyzer, true);
			} catch (IOException e) {
				Trace.printException(e);
				return false;
			}

			recover();
			
			worker.submit(new Runnable() {
				public void run() {
					Trace.printVerbose("Synchronization complete.");
				}
			});
        }
        
        return true;
  	}
	
	public void deinitialize() {
		if (data != null) {
			
			worker.shutdown();
			try {
				while(!worker.awaitTermination(500, TimeUnit.MILLISECONDS)) {
					Trace.printVerbose("Indexing service is still running...");
				}
			} catch (InterruptedException e1) {
				Trace.printException("Indexing service failed to stop");
			}

			try {
				if (writer != null) {
					writer.optimize();
					writer.close();
				}
				if (reader != null) {
					reader.close();
				}
			} catch (IOException e) {
				Trace.printVerbose("Failed to close index.");
				return;
			}

			assert syncOk();
			
			data = null;
			
			Trace.printVerbose("Index is closed.");			
		}
	}
	
	private IndexWriter getWriter() throws IOException {
        // create a new index writer.
        // the false means it won't overwrite the old index
		return new IndexWriter(indexDir, writerAnalyzer, false);   	    
	}

	private int getId(ArticleInfo article) throws IOException {
		int result = -1;	
		
		IndexReader readonly = IndexReader.open(indexDir);
		IndexSearcher search = new IndexSearcher(readonly);
				
		BooleanQuery query = new BooleanQuery();
		query.add(new TermQuery(new Term("id", article.getTitle())), Occur.MUST);
		query.add(new TermQuery(new Term("date", Long.toString(article.getPublishedDate()))), Occur.MUST);
		query.add(new TermQuery(new Term("feed", article.getFeed().getURL())), Occur.MUST);
		Hits hits = search.search(query);

		if (hits.length() > 0)
			result = hits.id(0);
		
		search.close();
		readonly.close();
		
		return result;
	}
	
	/**
	 * Returns the articles in the given virtual. The virtual must be
	 * adapted for lucene.
	 * @param virtual
	 * @return collection of articles.
	 */
	public Collection<ArticleInfo> getArticles(Virtual virtual) {
		if (!ENABLED) return null;

		if (!virtual.isFullText())
			return null;
		
		try {			
			worker.submit(new Runnable() {
				public void run() {
					try {
						writer.optimize();
					} catch (IOException e) {
						Trace.printException(e);
					}
				}
			});
			
			IndexReader readonly = IndexReader.open(indexDir);
			IndexSearcher search = new IndexSearcher(readonly);
			
			boolean unreadOnly;
			if (virtual.getQuery().startsWith("+"))
				unreadOnly = true;
			else 
				unreadOnly = false;
			
			String q = virtual.getQuery().substring(1);
			
			Query query = (new QueryParser("content", new StandardAnalyzer())).parse(q);
			Hits hits = search.search(query);

			int count = hits.length();
			
			if (virtual.getLimit() > 0)
				count = Math.min(virtual.getLimit(), hits.length());

			List<ArticleInfo> articles = new ArrayList<ArticleInfo>();

			for(int i = 0; i< count; i++) {
				Document doc = hits.doc(i);
				ArticleInfo info = data.getArticle(doc.getField("title").stringValue(),
						data.getFeed(doc.getField("feed").stringValue()),
						Long.parseLong(doc.getField("date").stringValue()));
				if ((info != null) && ((unreadOnly && !info.isRead()) || !unreadOnly))
					articles.add(info);
			}

			search.close();
			readonly.close();
			
			return articles;
		} catch (IOException e) {
			Trace.printException("Failed to search");
		} catch (ParseException e) {
			Trace.printException("Query is wrong");
		}		
		
		return null;
	}

	/**
	 * Removes an article from an index.
	 * @param article non-null
	 */
	public void removeArticle(final ArticleInfo article) {
		if (!ENABLED) return;
		
		assert article!=null;
		
		worker.submit(new Runnable() { public void run() {
			try {
				int id = getId(article);

				if (id >= 0) {
					if (writer != null) {						
						writer.optimize();
						writer.close();
						writer = null;
					}

					if (reader == null) 
						reader = IndexReader.open(indexDir);				 

					reader.deleteDocument(id);
				} else {
					Trace.printException("Failure of getId() in index: index and db are out of sync now. Please let Kuat now.\nArticle details: " + article.toString());										
				}
			} catch (IOException e) {
				Trace.printException(e);
			}}});		
	}

	/**
	 * Remove all articles in the given feed.
	 * @param feed to be removed
	 */
	public void removeFeed(final Feed feed) {
		if (!ENABLED) return;

		assert feed != null;

		worker.submit(new Runnable() {public void run() {
			try {
				if (writer != null) {						
					writer.optimize();
					writer.close();
					writer = null;
				}

				if (reader == null) 
					reader = IndexReader.open(indexDir);				 

				reader.deleteDocuments(new Term("feed", feed.getURL()));
			} catch (IOException e) {
				Trace.printException(e);
			}
		}});
		
	}
	
	/**
	 * Creates a lucene document for a given article and its summary.
	 * @param article
	 * @param content
	 * @return a document
	 */
	private Document createDocument(ArticleInfo article, ArticleContent content) {
		final Document doc = new Document();

		//	pull out all of the fields and put them
		//	in the document

		doc.add(new Field("id", article.getTitle(), Field.Store.YES, Field.Index.UN_TOKENIZED));
		doc.add(new Field("date", Long.toString(article.getPublishedDate()), Field.Store.YES, Field.Index.UN_TOKENIZED));
		doc.add(new Field("feed", article.getFeed().getURL(), Field.Store.YES, Field.Index.UN_TOKENIZED));
		
		doc.add(new Field("authors", validate(article.getAuthors()), Field.Store.NO, Field.Index.TOKENIZED));
		doc.add(new Field("feedurl", article.getFeed().getURL(), Field.Store.YES, Field.Index.TOKENIZED));
		doc.add(new Field("url", validate(article.getURL()), Field.Store.NO, Field.Index.TOKENIZED));
		doc.add(new Field("title", article.getTitle(), Field.Store.YES, Field.Index.TOKENIZED));
  
		String summary = content.getSummary();
		
		try {
			StringBean sb = new StringBean ();
			sb.setLinks (false);
			Parser parser;
			parser = new Parser ();
			parser.setInputHTML(summary);
			parser.visitAllNodesWith (sb);
			summary = sb.getStrings ();
		} catch (ParserException e) {
			Trace.printException(e);
		}				
		
		if (summary == null) summary = "";
		
		doc.add(new Field("content", summary, Field.Store.NO, Field.Index.TOKENIZED));
		
		return doc;
	}
	
	/**
	 * Indexes an article and its content and puts it into the index.
	 * @param article
	 * @param content
	 */
	public void addArticle(ArticleInfo article, ArticleContent content) {		
		if (!ENABLED) return;
		
		final Document doc = createDocument(article, content);

		worker.submit(new Runnable() { public void run() {		
			try {
				if (reader != null) {
					reader.close();
					reader = null;
				}
				if (writer == null)
					writer = getWriter();
				writer.addDocument(doc);			
			} catch (IOException e) {
				Trace.printException(e);
			}}});
	}
	
	/**
	 * Checks for sync status between db and index
	 *
	 */
	private boolean syncOk() {
		int count = 0;
		for (Feed feed : data.getFeeds()) 
			count += data.numberArticles(feed);
		
		int indexCount = 0;
		IndexReader readonly;
		try {
			readonly = IndexReader.open(indexDir);
			indexCount = readonly.numDocs();
			readonly.close();
		} catch (IOException e) {
			Trace.printException(e);
		}
		
		if (count != indexCount) {
			Trace.printException("Database and index are not synchronized!!!\nIf application crashed last time then recovery did not succeed. If not let me (Kuat) know.");
			Trace.printVerbose("Number of articles in database: " + count);
			Trace.printVerbose("Number of articles in index: " + indexCount);
			Trace.printVerbose("Starting recovery and diagnostics");
			try {
				IndexReader reader = IndexReader.open(indexDir);
				for (int i = 0; i < indexCount; i++) {
					Document doc = reader.document(i);

					String url = doc.getField("feed").stringValue();
					String title = doc.getField("title").stringValue();
					long date = Long.parseLong(doc.getField("date").stringValue());
					Feed feed = data.getFeed(url);
					if (feed != null) {
						if (data.getArticle(title, feed, date) != null)
							continue;
					}
					
					Trace.printVerbose("Missing document: " + title + " : " +
							doc.getField("feed").stringValue() + " : " +
							date);				
										
					reader.deleteDocument(i);
				}
				reader.close();
			} catch (IOException e) {
				Trace.printException(e);
			}
			
			Iterator<ArticleInfo> iterator = data.getArticles(VirtualFactory.empty("Random"));
			try {
				writer = getWriter();

				while(iterator.hasNext()) {
					final ArticleInfo article = iterator.next();
					final ArticleContent content= data.getContent(article);
					final Document doc = createDocument(article, content);

					writer.addDocument(doc);			
				}
				
				writer.optimize();
				writer.close();
			} catch (IOException e) {
				Trace.printException(e);
			}


			return false;
		}
		
		return true;
	}
	
	/**
	 * Returns most frequent terms in the index in the order of increasing frequency.
	 * @param count number of terms to returns
	 * @return
	 */
	public Collection<String> mostFrequentTerms(int count) {
		if (!ENABLED) return null;
		
		if (stopWords == null) {
			 stopWords = new HashSet<String>();
			 for (String s: StandardAnalyzer.STOP_WORDS) {
				 stopWords.add(s.toUpperCase());
			 }
				 
		}
		
		Collection<String> result = new ArrayList<String>();

		IndexReader readonly;
		try {
			readonly = IndexReader.open(indexDir);

			TermFreqQueue tiq = new TermFreqQueue(count);
			TermEnum terms = readonly.terms();

			int minFreq = 0;
			while (terms.next()) {
				if ((terms.docFreq() > minFreq) && heuristics(terms.term())) {
					tiq.put(new TermFreq(terms.term(), terms.docFreq()));
					if (tiq.size() >= count) { // if tiq overfull
						tiq.pop(); // remove lowest in tiq
						minFreq = ((TermFreq)tiq.top()).docFreq; // reset minFreq
					}
				}
			}

			while (tiq.size() != 0) {
				TermFreq termInfo = (TermFreq)tiq.pop();
				result.add(termInfo.term.text());			 
			}

			readonly.close();

		} catch (IOException e) {
			Trace.printException(e);
		}

		
		return result;
	}
	
	/**
	 * Returns true if the term is a human-readable and meaningful term according to the heuristics.
	 * @param term
	 * @return
	 */
	private boolean heuristics(Term term) {
		
		return (term.field().equals("title") || term.field().equals("content")) && 
			(!stopWords.contains(term.text().toUpperCase())) &&
			(term.text().length() > 5);
	}
	
	private static class TermFreqQueue extends PriorityQueue {
		TermFreqQueue(int size) {
			initialize(size);
		}

		protected final boolean lessThan(Object a, Object b) {
			TermFreq termInfoA = (TermFreq)a;
			TermFreq termInfoB = (TermFreq)b;
			return termInfoA.docFreq < termInfoB.docFreq;
		}
	}

	private static class TermFreq {
		TermFreq(Term t, int df) {
			term = t;
			docFreq = df;
		}
		int docFreq;
		Term term;
	}

	private String validate(String s) {
		return (s == null ? "" : s);
	}
}
