package data;

import java.io.ByteArrayOutputStream;
import java.sql.Blob;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.sql.rowset.serial.SerialBlob;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;

import util.Trace;

/**
 * This class serves as an adaptor to database.
 * Manages queries to synchronize states of memory objects with their persistent instances.
 * @author Kuat
 *
 */
class Materializer {
	
	public static final String articleInfoColumns = "title, authors, url, feed, readflag, star, date, latitude, longitude";
	public static final String articleInfoCompleteColumns = "title, authors, ARTICLE.url, ARTICLE.feed, readflag, star, ARTICLE.date, latitude, longitude";
	
    private PreparedStatement addFeed, addVirtual, addChannel, addArticle, removeArticle, removeVirtual, removeChannel, removeFeed;
    
    private PreparedStatement getContent, getArticle, updateFeed, updateArticle, updateChannel;
    
    private static Materializer singleton = new Materializer();
    
	// Internal feed-id mappings
	private Map<Integer, Feed> idFeed;
	private Map<Feed, Integer> feedId;
	private Map<Integer, Channel> idChannel;

	private Materializer() {
	}
	
	public static Materializer getInstance(){
		return singleton;
	}
	
    public void initialize(Database db) {

    	idFeed = new ConcurrentHashMap<Integer, Feed>();
		feedId = new ConcurrentHashMap<Feed, Integer>();
		idChannel = new ConcurrentHashMap<Integer, Channel>();
		
		try {
			addFeed = db.createPreparedStatement("insert into FEED " +
			" 			(name, url, channel, lastsync, updateperiod, icon)" + 
			" VALUES	(?,?,?,?,?,?)", Statement.RETURN_GENERATED_KEYS);
			
			addVirtual = db.createPreparedStatement("insert into VIRTUAL "+ 
			" 			(name, query, limit, lucene)" + 
			" VALUES 	(?, ?, ?, ?)");
			
			addChannel = db.createPreparedStatement("insert into CHANNEL "+
			"			(name) VALUES (?)", Statement.RETURN_GENERATED_KEYS);
			
			addArticle = db.createPreparedStatement("insert into ARTICLE " +
			"			(title, authors, url, feed, readflag, date, committed, summary, star, latitude, longitude)" +
			" VALUES	(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
			
			removeVirtual = db.createPreparedStatement("delete from VIRTUAL" +
			" where name = ?");
			
			removeChannel = db.createPreparedStatement("delete from CHANNEL where id = ?");
			
			removeFeed = db.createPreparedStatement("delete from FEED where id = ?");
			
			removeArticle = db.createPreparedStatement("delete from ARTICLE where title = ? and feed = ? and date = ?");
			
			getContent = db.createPreparedStatement("select summary from ARTICLE where title=? and feed=? and date=?");
			
			getArticle = db.createPreparedStatement("select " + articleInfoColumns + "  from ARTICLE where title = ? and feed = ? and date = ?");
			
			updateChannel = db.createPreparedStatement("update CHANNEL set name = ? where id = ?");
			
			updateFeed = db.createPreparedStatement("update FEED set name = ?, channel = ?, lastsync = ?, updateperiod = ? where id = ?");
			
			updateArticle = db.createPreparedStatement("update ARTICLE set readflag =? , star = ?,  latitude=?, longitude=? where title=? and feed=? and date=?");
			
		} catch (SQLException e) {
			Trace.printException(e);
		}		
    }
    
    Integer getId(Feed feed) {
    	return feedId.get(feed);
    }
    
    Integer getId(Channel channel) {
    	return channel.getId();
    }
    
	
	/*******************************************************************************
	 * 
	 */
    
	Virtual newVirtual(ResultSet results) throws SQLException {
		boolean lucene = false;
		if (results.getShort(4) == 1) 
			lucene = true;
		Virtual result = new Virtual(results.getString(1), results.getString(2), results.getInt(3), lucene);		
		return result;
	}
	
	void flushVirtual(Virtual v) throws SQLException {
		addVirtual.clearParameters();
		addVirtual.setString(1, v.getName());
		addVirtual.setString(2, v.getQuery());
		addVirtual.setInt(3, v.getLimit());
		short lucene = 0;
		if (v.isFullText())
			lucene = 1;
		addVirtual.setShort(4, lucene);
		addVirtual.executeUpdate();
	}
	
	void removeVirtual(Virtual v) throws SQLException {
		removeVirtual.clearParameters();
		removeVirtual.setString(1, v.getName());
		removeVirtual.executeUpdate();
	}
	
	
	/*******************************************************************************
	 * 
	 */
	
	Channel newChannel(ResultSet results) throws SQLException {
		int id = results.getInt(1);
		Channel c = new Channel(results.getString(2), id);
		idChannel.put(id, c);    
		return c;
	}
	
	Channel flushChannel(String name) throws SQLException {
		addChannel.clearParameters();
		addChannel.setString(1, name);
		addChannel.executeUpdate();
		ResultSet results = addChannel.getGeneratedKeys();
        
        int id = -1;
        if (results.next()) {
            id = results.getInt(1);
        }
        
        if (id == -1) {
        	throw new SQLException();
        }
        
        Channel c = new Channel(name, id);        
        idChannel.put(id, c);        	
        return c;
	}
	
	void removeChannel(Channel c) throws SQLException {
		removeChannel.clearParameters();
		int id = c.getId();
		removeChannel.setInt(1, id);
		removeChannel.executeUpdate();			
		idChannel.remove(id);
	}
	
	void updateChannel(Channel channel) throws SQLException {
		updateChannel.clearParameters();
		updateChannel.setString(1, channel.getName());
		updateChannel.setInt(2, channel.getId());
		updateChannel.executeUpdate();
	}
	
	/*******************************************************************************
	 * 
	 */
	
	Feed newFeed(ResultSet results) throws SQLException {
    	int id = results.getInt(1);
    	String name = results.getString(2);
    	String url = results.getString(3);
    	int channel = results.getInt(4);
    	Timestamp lastSyncDate = results.getTimestamp(5);
    	long lastSync;
    	if (lastSyncDate != null) {
    		lastSync = lastSyncDate.getTime();
    	} else {
    		lastSync = -1;
    	}
    	long update = results.getLong(6);
    	Blob blob = results.getBlob(7);
    	ImageData icon;
    	if (blob != null){
			ImageLoader imageLoader = new ImageLoader();
			imageLoader.load(blob.getBinaryStream());
    		icon = imageLoader.data[0];
    	} else{
    		icon = null;
    	}
    	

    	Feed feed = new Feed(name,url,lastSync,idChannel.get(channel),update,icon);
    	feedId.put(feed, id);
    	idFeed.put(id, feed);

    	return feed;
	}
	
	void flushFeed(Feed feed) throws SQLException {
		addFeed.clearParameters();
		addFeed.setString(1, feed.getName());
		addFeed.setString(2, feed.getURL());
		addFeed.setInt(3, feed.getChannel().getId());
		addFeed.setTimestamp(4, new Timestamp(feed.getSyncTime()));
		addFeed.setLong(5, feed.getUpdateInterval());
		
		if (null != feed.getIcon()) {
			ImageLoader imageLoader = new ImageLoader();
			imageLoader.data = new ImageData[] {feed.getIcon()};
			ByteArrayOutputStream os = new ByteArrayOutputStream();
			imageLoader.save(os, SWT.IMAGE_ICO);

			Blob blob = new SerialBlob(os.toByteArray());			
			addFeed.setBlob(6, blob);
		} else
			addFeed.setBlob(6, null);
		
		addFeed.executeUpdate();
        ResultSet results = addFeed.getGeneratedKeys();
        
        int id = -1;
        if (results.next()) {
            id = results.getInt(1);
        }
        
        if (id == -1) {
        	throw new SQLException();
        } else {
        	idFeed.put(id, feed);
        	feedId.put(feed, id);
        }        
	}
	
	void removeFeed(Feed f) throws SQLException {
		int id = feedId.get(f);
		removeFeed.clearParameters();
		removeFeed.setInt(1, id);
		removeFeed.executeUpdate();
		feedId.remove(f);
		idFeed.remove(id);
	}

	void updateFeed(Feed feed) throws SQLException {
		updateFeed.clearParameters();
		updateFeed.setString(1, feed.getName());
		updateFeed.setInt(2, feed.getChannel().getId());
		updateFeed.setTimestamp(3, new Timestamp(feed.getSyncTime()));
		updateFeed.setLong(4, feed.getUpdateInterval());
		updateFeed.setInt(5, feedId.get(feed));
		updateFeed.executeUpdate();
		
	}	

	
	/*******************************************************************************
	 * 
	 */
	
	ArticleInfo newArticleInfo(ResultSet results) throws SQLException {
		String title = results.getString(1);
		String authors = results.getString(2);
		String url = results.getString(3);
		Feed parent = idFeed.get(results.getInt(4));
		boolean read = false;
		short status = results.getShort(5);
		if (status == 1)
			read = true;
		
		boolean star = false;
		status = results.getShort(6);
		if (status == 1)
			star = true;	
		
		Timestamp date = results.getTimestamp(7);
		
		double latitude = results.getDouble(8);
		double longitude = results.getDouble(9);
		GeoPoint geo = null;
		if ((latitude != 0) || (longitude != 0))
			geo = new GeoPoint(latitude, longitude);
		
		if (parent == null) throw new SQLException("Feed was deleted");
		
		return new ArticleInfo(title,parent,date.getTime(), authors, url, read, star, geo);
	}

	ArticleContent newArticleContent(ResultSet results) throws SQLException {
		String summary = results.getString(1);
		return new ArticleContent(summary);
	}
	
	void flushArticle(ArticleInfo ai, ArticleContent ac) throws SQLException {
		addArticle.clearParameters();
		addArticle.setString(1, ai.getTitle());
		addArticle.setString(2, ai.getAuthors());
		addArticle.setString(3, ai.getURL());
		addArticle.setInt(4, feedId.get(ai.getFeed()));
		short read = 0;
		if (ai.isRead())
			read = 1;
		addArticle.setShort(5, read);
		addArticle.setTimestamp(6, new Timestamp(ai.getPublishedDate()));
		addArticle.setTimestamp(7, new Timestamp(System.currentTimeMillis()));
		addArticle.setString(8, ac.getSummary());
		short star = 0;
		if (ai.isStarred())
			star = 1;
		addArticle.setShort(9, star);
		if (ai.getPoint() != null) {
			addArticle.setDouble(10, ai.getPoint().getLatitude());
			addArticle.setDouble(11, ai.getPoint().getLongitude());
		} else {
			addArticle.setDouble(10, 0);
			addArticle.setDouble(11, 0);
		}
			
		addArticle.executeUpdate();		
	}
	
	ArticleInfo getArticle(String title, Feed feed, long date) throws SQLException {
		getArticle.clearParameters();
		getArticle.setString(1, title);
		getArticle.setInt(2, feedId.get(feed));
		getArticle.setTimestamp(3, new Timestamp(date));
		ResultSet results = getArticle.executeQuery();
		
		ArticleInfo article;
		if (results.next())
			article = newArticleInfo(results);
		else 
			article = null;
		
		results.close();
		
		return article;
	}
	
	ArticleContent getContent(ArticleInfo ai) throws SQLException {
		getContent.clearParameters();
		getContent.setString(1, ai.getTitle());
		getContent.setInt(2, feedId.get(ai.getFeed()));
		getContent.setTimestamp(3, new Timestamp(ai.getPublishedDate()));
		ResultSet results = getContent.executeQuery();
		
		ArticleContent result;
		if (results.next()) 
			result = newArticleContent(results);
		else
			result = null;
		
		results.close();
		return result;
	}

	void removeArticle(ArticleInfo article) throws SQLException{
		removeArticle.clearParameters();
		removeArticle.setString(1, article.getTitle());
		removeArticle.setInt(2, feedId.get(article.getFeed())); 
		removeArticle.setTimestamp(3,new Timestamp(article.getPublishedDate())); 
		removeArticle.executeUpdate();		
	}

	void updateArticle(ArticleInfo article) throws SQLException {
		updateArticle.clearParameters();
		short read = 0;
		if (article.isRead())
			read = 1;
		updateArticle.setShort(1, read);
		short star = 0;
		if (article.isStarred())
			star = 1;
		updateArticle.setShort(2, star);
		if (article.getPoint() != null) {
			updateArticle.setDouble(3, article.getPoint().getLatitude());
			updateArticle.setDouble(4, article.getPoint().getLongitude());
		} else {
			updateArticle.setDouble(3, 0);
			updateArticle.setDouble(4, 0);
		}
		updateArticle.setString(5, article.getTitle());
		updateArticle.setInt(6, feedId.get(article.getFeed())); 
		updateArticle.setTimestamp(7,new Timestamp(article.getPublishedDate())); 
		updateArticle.executeUpdate();		
	}
}
