package data;

import java.sql.Timestamp;
import java.util.Date;

import org.apache.commons.lang.StringEscapeUtils;

import util.Trace;

/**
 * An utility factory of Virtual objects.
 * Provides a variety of constructors.
 * The default limiting factor of results is 1000.
 * @author Kuat
 *
 */
public class VirtualFactory {
	

	private static final String keywordFormat = "select " + Materializer.articleInfoColumns +" from ARTICLE where (UPPER(title) like '%%%1$S%%' or UPPER(summary) like '%%%1$S%%' or UPPER(authors) like '%%%1$S%%')";
	private static final String all = "select " + Materializer.articleInfoColumns +" from ARTICLE";
	private static final String allOrdered = "select " + Materializer.articleInfoColumns +" from ARTICLE order by date desc";
	
	private static int limit = 1000;
	
	private static String luceneSpecialCharacters = "'+-!(){}[]^\"~*?:\\";
		
	/**
	 * Creates a virtual folder consisting of results of keyword search in
	 * articles titles and descriptions.
	 * SQL query.
	 * @param name name of the virtual
	 * @param keyword 
	 * @return
	 */
	public static Virtual keywordSQLSearch(String name, String keyword) {
		return new Virtual(name, String.format(keywordFormat, StringEscapeUtils.escapeSql(keyword.trim())), limit);
	}
	
	/**
	 * Creates a virtual folder consisting of results of keyword search in
	 * articles titles and descriptions. Tokenizes the keyword string and 
	 * creates a boolean query for containment of each token in content, title, or feed's url.
	 * LUCENE query.
	 * @param name desired name of the virtual
	 * @param keyword
	 * @return
	 */
	public static Virtual keywordLuceneSearch(String name, String keyword) {
		String[] tokens = keyword.split("\\s");
		boolean first = true;
		StringBuffer sb = new StringBuffer();
		sb.append("-");
		
		if (tokens.length > 0)
			sb.append("(");
		
		for(String token: tokens) {
			if (token.length() > 0) {
				token = escapeLucene(token);
				if (first) {
					sb.append("content: "+ token +"* OR title: " + token + "* OR feedurl: " + token + "*");
					first = false;
				} else {
					sb.append(") AND ");
					sb.append("(content: "+ token +"* OR title: " + token + "* OR feedurl: " + token + "*");
				}
			}
		}
		
		if (tokens.length > 0)
			sb.append(")");

		
		return new Virtual(name, sb.toString(), limit, true);
	}
	
	private static String escapeLucene(String text) {
		StringBuffer sb = new StringBuffer();
				
		for (char c : text.toCharArray()) {
			if (luceneSpecialCharacters.indexOf(c) != -1) {
				sb.append("\\");
			} 
			sb.append(c);
		}
		return sb.toString();
	}
	
	/**
	 * Lucene term query (based on query parser)
	 * @param field	field name 
	 * @param text expression
	 * @param b true if it is positive relation or negative
	 * @return
	 */
	public static Virtual luceneTermQuery(String field, String text, boolean b) {
		StringBuffer sb = new StringBuffer();
		sb.append("-");
		if (!b)
			sb.append("NOT ");
		String unesc = text;
		if ((unesc != null) && (unesc.length() > 0)) { 
			sb.append(field).append(":(").append(unesc.replace(" ", "+")).append(")");
			return new Virtual(field + (b ? " contains " : " does not contain ") + text, sb.toString(), 0, true);
		} else
			return null;		
	}

	
	// SQL / not combinable
	public static Virtual allOrdered(String name) {
		return new Virtual(name, allOrdered, 0);
	}
	
	// SQL 
	public static Virtual empty(String name) {
		return new Virtual(name, all, 0);
	}
	
	/**
	 * Limits the results of a search to the given number of elements.
	 * UNIVERSAL.
	 * @param v virtual
	 * @param n positive number to set a limit or 0 if remove any limits
	 * @return a new virtual
	 */
	public static Virtual limitVirtual(Virtual v, int n) {
		if (v == null) return null;
		
		return new Virtual(v.getName(), v.getQuery(), n, v.isFullText());
	}
	
	/**
	 * Returns a virtual for articles published before the given date and not starred.
	 * SQL query.
	 * @param date
	 * @return
	 */
	public static Virtual publishedUnstarredBeforeDate(long date) {
		return new Virtual("Before " + new Date(date), all + " where star = 0 and date < '" + new Timestamp(date) + "'", 0);
	}
	
	/**
	 * Returns a virtual of articles in the given virtual which are not read.
	 * UNIVERSAL.
	 * @param v a given virtual
	 * @return a virtual
	 */
	public static Virtual unreadOnly(Virtual v) {
		if (v == null) return null;
		
		if (!v.isFullText())
			return new Virtual(v.getName() + " (unread)", "select "+ Materializer.articleInfoColumns +" from (" + v.getQuery() + ") as A where A.readflag=0", v.getLimit());
		else 
			return new Virtual(v.getName() + " (unread)", "+" + v.getQuery().substring(1), v.getLimit(), true);
	}
	
	private static int getLimit(Virtual v1, Virtual v2) {
		int limit = 0;
		if (v1.getLimit() == 0) 
			limit = v2.getLimit();
		else {
			if (v2.getLimit() > 0)
				limit = Math.min(v2.getLimit(), v1.getLimit());
			else
				limit = v1.getLimit();
		}
		return limit;
	}
	
	// SQL / Lucene (not mixed) 
	public static Virtual combine(Virtual v1, Virtual v2) {
		if ((v1 == null) || (v2 == null)) return null;
		
		int limit = getLimit(v1, v2);
		String name = v1.getName() + " or " + v2.getName();
		
		if (v1.isFullText() && v2.isFullText()) {
			// TODO: this is a hack
			boolean unreadOnly1 = v1.getQuery().startsWith("+");
			boolean unreadOnly2 = v2.getQuery().startsWith("+");
			String prefix = (unreadOnly1 && unreadOnly2) ? "+" : "-";

			return new Virtual(name,
					prefix + "(" + v1.getQuery().substring(1) + ") OR (" + v2.getQuery().substring(1) + ")", limit, true);
		} else if (!v1.isFullText() && !v2.isFullText()) {						
			return new Virtual(name,
					v1.getQuery() + " UNION " + v2.getQuery(), limit);
		} else {
			Trace.printException("Lucene/SQL queries cross combination is not yet supported");
			return null;			
		}
	}
	
	/**
	 * Returns a virtual which contains articles contained both in v1 and v2.
	 * For SQL queries, the results of a virtual must not be ordered due to limitations of Derby.
	 * Lucene and SQL queries cannot be intersected at this moment.
	 */
	public static Virtual intersect(Virtual v1, Virtual v2) {
		if ((v1 == null) || (v2 == null)) return null;
		
		int limit = getLimit(v1, v2);
		String name = v1.getName() + " and " + v2.getName();

		if (v1.isFullText() && v2.isFullText()) {
			boolean unreadOnly1 = v1.getQuery().startsWith("+");
			boolean unreadOnly2 = v2.getQuery().startsWith("+");
			String prefix = (unreadOnly1 || unreadOnly2) ? "+" : "-";
			
			return new Virtual(name,
					prefix + "(" + v1.getQuery().substring(1) + ") AND (" + v2.getQuery().substring(1) + ")", limit, true);
		} else if (!v1.isFullText() && !v2.isFullText()) {						
			return new Virtual(name,
					v1.getQuery() + " INTERSECT " + v2.getQuery(), limit);
		} else {
			Trace.printException("Lucene/SQL queries cross intersection is not yet supported");
			return null;			
		}
	}
	
	/**
	 * Renames a virtual to the given name.
	 * UNIVERSAL.
	 * @param v virtual
	 * @param name new name
	 * @return
	 */
	public static Virtual changeName(Virtual v, String name) {
		return new Virtual(name, v.getQuery(), v.getLimit(), v.isFullText());
	}
	
	/**
	 * Orders the results of the virtual by date in the descending order.
	 * Works only for SQL queries and compatible with intersection operation.
	 * @param v
	 * @return a virtual
	 */
	public static Virtual orderByDate(Virtual v) {
		if (v.isFullText()) return null;
		
		return new Virtual(v.getName(), v.getQuery() + " order by date desc", v.getLimit());
	}

	/**
	 * Returns a virtual which contains all articles published after date x till date y.
	 * SQL query.
	 * @param x a date (in milliseconds)
	 * @param y a date (in milliseconds)
	 * @return a virtual
	 */
	public static Virtual dateRange(long x, long y) {
		return new Virtual("Between " + new Date(x) + " and " + new Date(y), all + " where date < '" + new Timestamp(y) + "' and date > '" + new Timestamp(x) + "'", 0);
	}
	
	/**
	 * Returns a virtual equivalent to a feed search.
	 * @param parent a feed
	 * @param isFullText true if want to use faster index queries
	 * @return a virtual
	 */
	public static Virtual createVirtualFromFeed(Feed parent, boolean isFullText) {
		if (isFullText) {
		 	return new Virtual(parent.getName(),"-feedurl:\"" + parent.getURL() + "\"", 0, true);
		} else
			return new Virtual(parent.getName(), all + " where feed=" + Materializer.getInstance().getId(parent));
	}	
	
	/**
	 * Returns a virtual containing all articles in the given channel.
	 * Uses SQL search backend.
	 * @param channel a channel
	 * @return a virtual
	 */
	public static Virtual createVirtualFromChannel(Channel channel) {
		if (channel == null) return null;
		return new Virtual(channel.getName(), "select " + Materializer.articleInfoCompleteColumns + " from " + 
				"ARTICLE, FEED "  + 
				"where ARTICLE.feed = FEED.id and FEED.channel = " + channel.getId());		
	}
}
