package download;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import parser.RSSParser;
import util.IconFactory;
import util.Pair;
import util.Trace;

import com.sun.syndication.io.FeedException;

import controller.Controller;
import data.ArticleContent;
import data.ArticleInfo;
import data.Channel;
import data.DataException;
import data.DataManager;
import data.Feed;

/**
 * The download manager. No more than one instance may exist. 
 * @author oleg
 */
public class DownloadManager implements FeedChangeObserver {
	// download tasks
	private final ConcurrentMap<String, TimerTask> tasks = new ConcurrentHashMap<String, TimerTask>();
	private final Timer timer;
	private final Object timerLock = new Object();
	private boolean timerWorking;
	private final DataManager dataManager;
	private static boolean instanceExists = false;

	// controller needed for error report
	private final Controller controller;

	// observers
	private final Vector<DownloadObserver> observers = new Vector<DownloadObserver>();

	/**
	 * Terminates all download tasks.
	 */
	public void terminate() {
		dataManager.removeFeedListener(this);
		synchronized (timerLock) {
			timer.cancel();
			timerWorking = false;
		}
		instanceExists = false;
	}

	/**
	 * Registers an observer to be notified about the download manager status.
	 * @param observer the observer to be notified
	 */
	public synchronized void addDownloadObserver(DownloadObserver observer) {
			observers.add(observer);
	}

	/**
	 * Unregisters an observer.
	 * @param observer
	 */
	public synchronized void removeDownloadObserver(DownloadObserver observer) {
			observers.remove(observer);
	}
	@SuppressWarnings("unchecked")
	private void notifyObservers(Feed feed, boolean started) {
		Vector<DownloadObserver> copy;
		synchronized (this) {
			copy = (Vector<DownloadObserver>) observers.clone();
		}
		for (DownloadObserver observer: copy) {
			if (started) observer.downloadStarted(feed);
			else observer.downloadFinished();
		}
	}

	private TimerTask createDownloadTask(final Feed feed) {
		return new TimerTask() {
			public void run() {
				if (controller.isOnline()) {
					notifyObservers(feed, true);				
					long newSyncTime = System.currentTimeMillis();
					long lastSyncTime = feed.getSyncTime(); 
					try {
						try {
							Collection<ArticleInfo> articles = new ArrayList<ArticleInfo>();;
							Collection<ArticleContent> contents = new ArrayList<ArticleContent>();;

							for (Pair<ArticleInfo, ArticleContent> article: RSSParser.parseFeed(feed,conditionalGet(feed.getURL(), lastSyncTime),newSyncTime)) {

								if (article.car.getPublishedDate() > lastSyncTime) {
									articles.add(article.car);
									contents.add(article.cdr);
								} 											
							}

							if (articles.size() > 0 && timerWorking)
								dataManager.addArticles(articles, contents);
						} catch (DataException e) {
							Trace.printException(e);
						}						

						feed.setSyncTime(newSyncTime);
						notifyObservers(feed, false);
					} catch (FeedException e) {
						//	e.printStackTrace();
					} catch (IOException e) {
						//	e.printStackTrace();
					} finally {					
						notifyObservers(feed, false);
					}
				}
			}
		};
	}
	/**
	 * Creates a new download manager. Currently can be called only once.
	 * @param dataManager
	 */
	public DownloadManager(Controller controller) {
		if (instanceExists) throw new IllegalArgumentException("Only one download manager may exist");
		else {
			instanceExists = true;
			synchronized (timerLock) {
				timer = new Timer();
				timerWorking = true;
			}
			this.controller = controller;
			this.dataManager = controller.getDataManager();			
			for (Feed feed: dataManager.getFeeds()) {
				if (!feed.isLocal()){
					TimerTask task = createDownloadTask(feed);
					tasks.put(feed.getURL(), task);
					timer.schedule(task, 0, feed.getUpdateInterval());
				}
			}
			dataManager.addFeedListener(this);
		}
	}
	// conditional get
	// null if not modified, inputStream otherwise
	// throws IOException
	private static InputStream conditionalGet(String url, long isModifiedSince) throws IOException {
		URLConnection connection = new URL(url).openConnection();
		if (connection instanceof HttpURLConnection) {
			connection.setIfModifiedSince(isModifiedSince);
			connection.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)");
			connection.connect();
			if (((HttpURLConnection) connection).getResponseCode() == HttpURLConnection.HTTP_NOT_MODIFIED) return null;
		}
		return connection.getInputStream();
	}

	/**
	 * Changes the download task execution interval.
	 * @param feed
	 */
	public void updateIntervalChanged(Feed feed) {
		tasks.remove(feed.getURL()).cancel();
		TimerTask task = createDownloadTask(feed);
		timer.scheduleAtFixedRate(task, feed.getUpdateInterval(), feed.getUpdateInterval());
		tasks.put(feed.getURL(), task);
	}
	/**
	 * Subscribes to a new feed
	 * @param url
	 * @param name
	 * @param channel
	 * @param updateInterval
	 */
	public void newSubscription(String url, String name, Channel channel, long updateInterval) {
		Feed feed = new Feed(name, url, 0, channel, updateInterval, IconFactory.getFavicon(url));
		long firstSyncTime = System.currentTimeMillis();
		Collection<Pair<ArticleInfo, ArticleContent>> articles = null;
		try {
			notifyObservers(null, true);
			articles = RSSParser.parseFeed(feed, conditionalGet(url, 0),firstSyncTime);
			feed.setSyncTime(firstSyncTime);
			dataManager.addFeed(feed);
			notifyObservers(null, false);
			notifyObservers(feed, true);
			TimerTask task = createDownloadTask(feed);
			synchronized (timerLock) {
				if (timerWorking) timer.scheduleAtFixedRate(task, updateInterval, updateInterval);
			}
			
			tasks.put(url, task);
			if (articles != null) {
				try {
					Collection<ArticleInfo> articlesList = new ArrayList<ArticleInfo>();
					Collection<ArticleContent> contents = new ArrayList<ArticleContent>();

					for (Pair<ArticleInfo, ArticleContent> article: articles) {
						articlesList.add(article.car);
						contents.add(article.cdr);
					}

					if (timerWorking) dataManager.addArticles(articlesList, contents);
				} catch (DataException e) {
					if (e.isUniquenessViolation()) {
						// ignore
					} else {
						controller.reportError(e.getMessage());
					}
				}
			}
		} catch (FeedException e) {
			controller.reportError("The feed is incorrect: " + e.getMessage());
		} catch (DataException e) {
			if (e.isUniquenessViolation()) {
				controller.reportError("You are already subscribed to the feed \n" + name + " (" + url + ")");
			} else
				controller.reportError(e.getMessage());
		} catch (IOException e) {
			controller.reportError("Could not connect to the web server: " + e.getMessage());
		} finally {
			notifyObservers(null, false);
		}
	}

	public void forceDownload(Feed feed) {
		if (!feed.isLocal()) synchronized (timerLock) {
			if (timerWorking) timer.schedule(createDownloadTask(feed),0);
		}
	}

	public void feedRemoved(Feed feed) {
		tasks.remove(feed.getURL()).cancel();
	}

}
