package parser;


import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.swt.graphics.ImageData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import util.Pair;
import util.Trace;
import data.Channel;
import data.Feed;






/**
 * OPML reader/writer
 * 
 * OPML SpecsGoals of the OPML format 

The purpose of this format is to provide a way to exchange information between outliners and Internet services that can be browsed or controlled through an outliner.

The design goal is to have a transparently simple, self-documenting, extensible and human readable format that's capable of representing a wide variety of data that's easily browsed and edited. As the format evolves this goal will be preserved. It should be possible for a reasonably technical person to fully understand the format with a quick read of a single Web page.

It's an open format, meaning that other outliner vendors and service developers are free to use the format to be compatible with Radio UserLand or for any other purpose.

What is an <opml>? 

<opml> is an XML element, with a single required attribute, version; a <head> element and a <body> element, both of which are required.

The version attribute is a version string, of the form, x.y, where x and y are both numeric strings.

What is a <head>? 

A <head> contains zero or more optional elements, described below.

<title> is the title of the document.

<dateCreated> is a date-time, indicating when the document was created.

<dateModified> is a date-time, indicating when the document was last modified.

<ownerName> is a string, the owner of the document.

<ownerEmail> is a string, the email address of the owner of the document.

<expansionState> is a comma-separated list of line numbers that are expanded. The line numbers in the list tell you which headlines to expand. The order is important. For each element in the list, X, starting at the first summit, navigate flatdown X times and expand. Repeat for each element in the list.

<vertScrollState> is a number, saying which line of the outline is displayed on the top line of the window. This number is calculated with the expansion state already applied.

<windowTop> is a number, the pixel location of the top edge of the window.

<windowLeft> is a number, the pixel location of the left edge of the window.

<windowBottom> is a number, the pixel location of the bottom edge of the window.

<windowRight> is a number, the pixel location of the right edge of the window.

<head> notes 

All the sub-elements of <head> may be ignored by the processor. If an outline is opened within another outline, the processor must ignore the windowXxx elements, those elements only control the size and position of outlines that are opened in their own windows.

All date-times conform to the Date and Time Specification of RFC 822.

If you load an OPML document into your client, you may choose to respect expansionState, or not. We're not in any way trying to dictate user experience. The expansionState info is there because it's needed in certain contexts. It's easy to imagine contexts where it would make sense to completely ignore it.

What is a <body>? 

A <body> contains one or more <outline> elements.

What is an <outline>? 

An <outline> is an XML element, possibly containing one or more attributes, and containing any number of <outline> sub-elements.

Common attributes 

text is the string of characters that's displayed when the outline is being browsed or edited. There is no specific limit on the length of the text attribute.

type is a string, it says how the other attributes of the <outline> are interpreted.

isComment is a string, either "true" or "false", indicating whether the outline is commented or not. By convention if an outline is commented, all subordinate outlines are considered to be commented as well. If it's not present, the value is false.

isBreakpoint is a string, either "true" or "false", indicating whether a breakpoint is set on this outline. This attribute is mainly necessary for outlines used to edit scripts that execute. If it's not present, the value is false.

Compatibility 

Before the 1.0 format was frozen the top-level element of the format was called outlineDocument. Radio UserLand will continue to read such documents.

Limits 

There are no documented limits to the number of attributes an <outline> element can have, or the number of <outline> elements it can contain.

Notes 

OPML is a file format, not a protocol. When you click on a link in an HTML document it doesn't in any way change the document stored on the server. OPML is used in much the same way.
 * 
 * 
 */
public class OPMLParser {
	
	/**
	 * Parses and OMPL file on import. Feeds are constructed from the parameter value input
	 * to the method. **Please handle the channel names as they have been instantiated with null
	 * to avoid issues with DataManager synchronizability
	 * 
	 * 
	 * @param fileName 
	 * @param defaultLastModified for created feeds
	 * @param defaultUpdateInterval for created feeds
	 * @param defaultImage for created feeds
	 * @return Map<ChannelName, Collection<Pair<feedTitle, xmlURl>>>
	 */
	
	public static Map<String, Collection<Pair<String,String>>> readOPML(String fileName, long defaultLastModified,
			long defaultUpdateInterval, ImageData defaultImage) throws IOException{
		//throw new UnsupportedOperationException("Not implemented yet");
		Map<String, Collection<Pair<String,String>>> map = new TreeMap<String,Collection<Pair<String,String>>>();
		DOMParser parser = new DOMParser();
			try {
			   parser.parse(new InputSource(new FileReader(fileName)));
               Document document = parser.getDocument();
               
               //check for empty structure
               if (document==null){
            	   return null;
               }
                              
                             
               Node bodyNode = document.getElementsByTagName("body").item(0);
               //System.out.println(document.getElementsByTagName("body").getLength());
               NodeList nodeList = bodyNode.getChildNodes();
               List<Node> topLayerOutlineList = new LinkedList<Node>();
               for(int k= 0; k< nodeList.getLength(); k++){
            	   if(nodeList.item(k).getNodeName().equals("outline")){
            		   topLayerOutlineList.add(nodeList.item(k));
            	   }
               }
              
               int topLayerOutlineListSize = topLayerOutlineList.size();
               //System.out.println(topLayerOutlineListSize);
               //System.out.println(nodeList.item(2).getNodeName());
               
               for(int i = 0; i<topLayerOutlineListSize; i++){
            	   Element topLayerOutline = (Element)topLayerOutlineList.get(i);
            	   //System.out.println(topLayerOutline.getAttribute("title"));
            	   //System.out.println(topLayerOutline.hasChildNodes());
            	   String folderTitle = topLayerOutline.getAttribute("title");
            	   Collection<Pair<String,String>> listFeed = new LinkedList<Pair<String,String>>();
            	   
            	   if(topLayerOutline.hasChildNodes()){
            		   
            	   
            	   NodeList secondLayerOutlineNodes = topLayerOutline.getElementsByTagName("outline");
            	   int listSecondLayerOutlineNodesSize = secondLayerOutlineNodes.getLength();
            	   for (int j= 0; j<listSecondLayerOutlineNodesSize; j++){
            		   
            		   Element secondLayerOutlineNode= (Element)secondLayerOutlineNodes.item(j);
            	   String feedTitle = secondLayerOutlineNode.getAttribute("title");
            	   //System.out.println("feedTitle:"+feedTitle);
            	   String xmlUrl = secondLayerOutlineNode.getAttribute("xmlUrl");
            	   if(!(feedTitle.equals("") || xmlUrl.equals(""))){
            		   listFeed.add(new Pair<String,String>(feedTitle,xmlUrl));
            	   }
            	   }
            	   map.put(folderTitle,listFeed);
            	   }
            	   else{
            		  folderTitle="";
            		  String feedTitle = topLayerOutline.getAttribute("title");
            		  String xmlUrl = topLayerOutline.getAttribute("xmlUrl");
            		  //System.out.println("feedTitle:"+feedTitle);
            		  if(!(feedTitle.equals("") || xmlUrl.equals(""))){
            			  
            				  if(map.get("")!=null){
            					  listFeed = map.get("");
            				  }
            		  
            					  listFeed.add(new Pair<String,String>(feedTitle,xmlUrl));
            					  map.put(folderTitle,listFeed);
            				  
            			  }
            		  
            		  
            		  
            		  
               	   
            	   }
            	   //System.out.println("add  " + folderTitle);
            	   
            	   
            		   
            	   
            	   //System.out.println(map.size());
               }
               
            	   
            	   
            	   return map;
            	 
		   
		   
		   
		   }
			
			
           catch (SAXParseException e) {
               // ignore
           }
           catch (SAXException e) {
               System.err.println("error: Parse error occurred - "+e.getMessage());
               Exception se = e;
               
                   se = ((SAXException)e).getException();
               
               if (se != null)
                 se.printStackTrace(System.err);
               else
                 e.printStackTrace(System.err);
           }
           

		
		return null;
		 

	}
	
	
	public static Map<String, Collection<Feed>> readOPMLwithoutLayers(String fileName, long defaultLastModified,
			long defaultUpdateInterval, ImageData defaultImage) throws IOException{
		//throw new UnsupportedOperationException("Not implemented yet");
		Map<String, Collection<Feed>> map = new TreeMap<String,Collection<Feed>>();
		Collection<Feed> listFeed = new LinkedList<Feed>();
		DOMParser parser = new DOMParser();
			try {
			   parser.parse(fileName);
               Document document = parser.getDocument();
               
               //check for empty structure
               if (document==null){
            	   return null;
               }
               //in case of multiple titles, the first title holds
               Node importTitle = document.getElementsByTagName("title").item(0);
               String importTitleString = importTitle.getTextContent();
               
               if (importTitleString==null){
            	   importTitleString = "Imported Feeds";
               }
               
               
               NodeList nodeList = document.getElementsByTagName("outline");
               //System.out.println("NodeList size: " + nodeList.getLength());
               int listSize = nodeList.getLength();
               
               for(int i = 0; i<listSize; i++){
            	   Element elem = (Element)nodeList.item(i);
            	   String feedTitle = elem.getAttribute("title");
            	   //System.out.println("feedTitle:"+feedTitle);
            	   String xmlUrl = elem.getAttribute("xmlUrl");
            	   String htmlUrl = elem.getAttribute("htmlUrl");
            	   if(!(feedTitle.equals("") || xmlUrl.equals("") || htmlUrl.equals(""))){
            		   Feed feed = new Feed(feedTitle,xmlUrl,defaultLastModified, null,defaultUpdateInterval,defaultImage);
            		   listFeed.add(feed);
            	   }
            	   
            	   map.put(importTitleString, listFeed);
               }
            	   
            	   
            	   return map;
            	 
		   
		   
		   
		   }
			
			
           catch (SAXParseException e) {
               // ignore
           }
           catch (SAXException e) {
               System.err.println("error: Parse error occurred - "+e.getMessage());
               Exception se = e;
               
                   se = ((SAXException)e).getException();
               
               if (se != null)
                 se.printStackTrace(System.err);
               else
                 e.printStackTrace(System.err);
           }
           

		
		return null;
		 

	}
	/**
	 * Exports a collection of feeds in the OPML format.
	 * @param feeds folders and corresponding feeds
	 * @param stream output stream where the OPML data is to be written
	 * @throws IOException 
	 * @author oleg
	 */
	public static void writeOPML(Map<Channel, Collection<Feed>> feeds, OutputStream stream) throws IOException,
	TransformerConfigurationException, ParserConfigurationException{
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        Document opml = db.newDocument();
        
        //Document opml = new DocumentImpl();
		Element root = opml.createElement("opml");
		opml.appendChild(root);
		Element body = opml.createElement("body");
		for (Channel channel: feeds.keySet()) {
			Element folder = opml.createElement("outline");
			folder.setAttribute("type", "folder");
			folder.setAttribute("title", channel.getName());
			for (Feed feed: feeds.get(channel)) {
				Trace.printVerbose("processing " + feed.getName());
				Element elt = opml.createElement("outline");
				elt.setAttribute("type","rss");
				elt.setAttribute("title",feed.getName());
				elt.setAttribute("xmlUrl", feed.getURL());
				//folder.addcontent(elt);
				folder.appendChild(elt);
			}
			body.appendChild(folder);
		}
		root.appendChild(body);
		//XMLOutputter out = new XMLOutputter();
		//out.output(opml,stream);
		TransformerFactory	tranFactory = TransformerFactory.newInstance();
		
        Transformer aTransformer = tranFactory.newTransformer();

        Source src = new DOMSource(opml);
        StreamResult dest = new StreamResult(stream);
        try{
        
        aTransformer.transform(src, dest); 
        }
        catch(TransformerException e){
        	e.printStackTrace();
        }
        stream = dest.getOutputStream();
			
	}
	/**
	 * Exports a collection of feeds in the OPML format.
	 * @param feeds folders and corresponding feeds
	 * @param fileName the name of the file where the OPML data is to be written
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @author oleg
	 */
	public static void writeOPML(Map<Channel, Collection<Feed>> feeds, String fileName) throws FileNotFoundException, IOException {
		try{
		writeOPML(feeds, new FileOutputStream(fileName));
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
}
