package parser;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang.StringEscapeUtils;

import util.Pair;

import com.sun.syndication.feed.module.georss.GeoRSSModule;
import com.sun.syndication.feed.module.georss.GeoRSSUtils;
import com.sun.syndication.feed.synd.SyndContent;
import com.sun.syndication.feed.synd.SyndContentImpl;
import com.sun.syndication.feed.synd.SyndEntry;
import com.sun.syndication.feed.synd.SyndEntryImpl;
import com.sun.syndication.feed.synd.SyndFeed;
import com.sun.syndication.feed.synd.SyndFeedImpl;
import com.sun.syndication.feed.synd.SyndPerson;
import com.sun.syndication.io.FeedException;
import com.sun.syndication.io.SyndFeedInput;
import com.sun.syndication.io.SyndFeedOutput;
import com.sun.syndication.io.XmlReader;

import data.ArticleContent;
import data.ArticleInfo;
import data.Feed;
import data.GeoPoint;

/**
 * RSS Parser.
 */
public class RSSParser {
	/**
	 * Produces a short summary of a collection of articles suitable for posting to blogs.
	 * @param articles
	 * @return
	 * @author oleg
	 */
	public static String getShortHTMLSummary(Iterator<ArticleInfo> articles) {
		StringBuffer sb = new StringBuffer();
		while (articles.hasNext()) {
			ArticleInfo a = articles.next();
			sb.append("<a href=\"").append(a.getURL()).append("\">").append(a.getTitle()).append("</a><br>");
		}
		return sb.toString();
	}
	
	/**
	 * Parses an RSS stream into a collection of articles.
	 * @param feed parent feed for new articles
	 * @param is RSS stream
	 * @return a collection of RSS articles
	 * @throws IllegalArgumentException
	 * @throws FeedException
	 * @throws IOException
	 * @author oleg 
	 */
	public static Collection<Pair<ArticleInfo, ArticleContent>> parseFeed(Feed feed, InputStream is, long fallbackDate) throws IllegalArgumentException, FeedException, IOException {
			SyndFeed romeFeed = new SyndFeedInput().build(new XmlReader(is));
			List<Pair<ArticleInfo, ArticleContent>> articles = new LinkedList<Pair<ArticleInfo,ArticleContent>>();
			Date feedPublishedDate = romeFeed.getPublishedDate();
			final long defaultDate = feedPublishedDate == null? fallbackDate : feedPublishedDate.getTime();
			for (Object o: romeFeed.getEntries()) {				 
				SyndEntry entry = (SyndEntry) o;
				String author = entry.getAuthor();
				if (author == null) {
					List authors = entry.getAuthors();
					if (authors == null || authors.isEmpty()) author = "";
					else {
						Iterator iter = authors.iterator();
						StringBuffer sb= new StringBuffer(((SyndPerson) iter.next()).getName());
						while (iter.hasNext()) {
							sb.append(", ").append(((SyndPerson) iter.next()).getName());
						}
						author = sb.toString();
					}
				} 
				String unescapedTitle = entry.getTitle();
				if (unescapedTitle == null) {
					unescapedTitle = "Untitled";
				} else unescapedTitle = StringEscapeUtils.unescapeXml(unescapedTitle);
				
				GeoRSSModule geoRSSModule = GeoRSSUtils.getGeoRSS(entry);
				GeoPoint point = null;
				if (geoRSSModule != null)
					point = new GeoPoint(geoRSSModule.getLatitude(), geoRSSModule.getLongitude());
				
				Date articlePublished = entry.getPublishedDate(); 
				ArticleInfo info = new ArticleInfo(unescapedTitle,feed,articlePublished==null? defaultDate : articlePublished.getTime(),StringEscapeUtils.unescapeXml(author),entry.getLink(),false, false, point);
				StringBuffer sb = new StringBuffer();
				for (Object c: entry.getContents()) sb.append(((SyndContent) c).getValue());
				ArticleContent content = new ArticleContent(sb.toString());
				articles.add(new Pair<ArticleInfo, ArticleContent>(info,content));
			}
			return articles;
	}
	/**
	 * Writes a feed to a file
	 * @param title feed title
	 * @param articles a collection of articles
	 * @param fileName the name of the output file
	 * @throws IOException if an I/O error occurs while writing the file
	 * @throws FeedException if an error occurs while building the xml representation of a feed
	 * @author oleg
	 */
	public static void writeFeed(String title, Collection<Pair<ArticleInfo, ArticleContent>> articles, String fileName) throws IOException, FeedException {
		SyndFeed romeFeed = new SyndFeedImpl();
		romeFeed.setFeedType("atom_0.3");
		romeFeed.setTitle(title);
		romeFeed.setPublishedDate(new Date(System.currentTimeMillis()));
		List<SyndEntry> entries = new LinkedList<SyndEntry>();
		for (Pair<ArticleInfo,ArticleContent> article: articles) {
			SyndEntry romeEntry = new SyndEntryImpl();
			romeEntry.setTitle(article.car.getTitle());
			romeEntry.setAuthor(article.car.getAuthors());
			romeEntry.setLink(article.car.getURL());
			//romeEntry.setPublishedDate(new Date(article.car.getPublishedDate()));
			romeEntry.setPublishedDate(new Date(System.currentTimeMillis()));
			List<SyndContent> contents = new LinkedList<SyndContent>();
			SyndContent content = new SyndContentImpl();
			content.setValue(article.cdr.getSummary());
			content.setType("text/html");
			contents.add(content);
			romeEntry.setContents(contents);
			entries.add(romeEntry);
		}
		romeFeed.setEntries(entries);
		SyndFeedOutput feedOut = new SyndFeedOutput();
		File out = new File(fileName);
		feedOut.output(romeFeed, out);
	}
}
