package search;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import view.dialogs.NewVirtualDialog;
import controller.Controller;
import data.Virtual;
import data.VirtualFactory;

/**
 * A dialog which gives a user detailed options for a search.
 * @author Kuat
 *
 */
public class AdvancedSearchDialog {
	private final Shell shell;
	private final Composite criteriaPane;
	private Virtual value;
	private Map<String, String> fields;
	private List<Criterion> criteria;
	private boolean isAnd;
	private boolean unreadOnly;
	private boolean firstSet = false;
	private final String help = "You can use the mask symbols such as * and ? in the query.\nIf you would like to use them literally you should precede the character with \\";

	private static enum Relation {
		YES ("Contains"),
		NO ("Does not");
		final String name;
		Relation(String name) {
			this.name = name;
		}
	}
	
	private void generateMaps() {
		fields = new HashMap<String, String>();
		fields.put("Authors", "authors");		
		fields.put("Summary", "content");
		fields.put("URI", "url");
		fields.put("Feed URL", "feedurl");
		fields.put("Title", "title");
	}	

	private Virtual getVirtual() {
		boolean first = true;
		Virtual result = null;
		for (Criterion c : criteria) {
			Virtual v = c.getVirtual();
			if (v!= null) {
				if (first) { 
					result =v;
					first = false;
				} else {
					if (isAnd) 
						result = VirtualFactory.intersect(result, v);	
					else
						result = VirtualFactory.combine(result, v);
				}
			}
		}				
		
		if (unreadOnly) 
			result = VirtualFactory.unreadOnly(result);
		
		return result;
	}
	
	private class Criterion extends Composite {
		final Combo field;
		final Combo relation;
		final Text text;
		
		Criterion(Composite parent) {
			super(parent, SWT.None);
			criteria.add(this);

			setLayout(new GridLayout(6, false));
			setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

			field = new Combo(this, SWT.BORDER | SWT.READ_ONLY);
			field.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING));
			
			for (String name:fields.keySet()) 
				field.add(name);
			
			field.select(field.indexOf("Title"));
						
			relation = new Combo(this, SWT.BORDER | SWT.READ_ONLY);
			relation.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING));
			
			for (Relation rel : Relation.values()) {
				relation.add(rel.name);
			}
			
			relation.select(0);
			
			text = new Text(this, SWT.BORDER);
			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.widthHint = 250;			
			text.setLayoutData(gd);
			text.setToolTipText(help);
			
			Button remove = new Button(this, SWT.PUSH | SWT.FLAT);
			remove.setText("Remove");
			remove.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
			remove.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event e) {
					criteria.remove(Criterion.this);
					Criterion.this.dispose();
					shell.pack();
				}
			});
			
			if (!firstSet) {
				remove.setEnabled(false);
				firstSet = true;
			}
		}
		
		Virtual getVirtual() {
			return VirtualFactory.luceneTermQuery(fields.get(field.getText()), text.getText(), relation.getText().equals(Relation.YES.name));
		}
	}
	
	/**
	 * Creates a new advanced search dialog.
	 * @param parent parent shell
	 */
	public AdvancedSearchDialog(final Shell parent, final Controller controller) {
		shell = new Shell(parent, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
		shell.setText("Advanced search");		
		shell.setLayout(new GridLayout(1, false));
		new Label(shell, SWT.None).setText("Please select criteria:");
		
		generateMaps();
		
		Composite logic = new Composite(shell, SWT.None);
		logic.setLayout(new GridLayout(2, false));
		logic.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));	
		Button and = new Button(logic, SWT.RADIO);
		and.setText("All criteria must be satisfied");
		and.setSelection(true);
		isAnd = true;
		and.addListener(SWT.Selection, new Listener() { public void handleEvent(Event e) {
			isAnd = true;}});
		Button or = new Button(logic, SWT.RADIO);
		or.setText("Some criterion must be satisfied");
		or.addListener(SWT.Selection, new Listener() { public void handleEvent(Event e) {
			isAnd = false;}});
		
		Button readStatus = new Button(shell, SWT.CHECK);
		readStatus.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		readStatus.setText("Search among unread messages only");
		unreadOnly = false;
		readStatus.setSelection(false);
		readStatus.addListener(SWT.Selection, new Listener() {public void handleEvent(Event e) {
			unreadOnly = !unreadOnly;}});
		
		criteria = new ArrayList<Criterion>();
		criteriaPane = new Composite(shell, SWT.None);
		criteriaPane.setLayout(new GridLayout(1, false));
		criteriaPane.setLayoutData(new GridData(GridData.FILL_BOTH));
		new Criterion(criteriaPane);
		
		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		buttons.setLayout(new GridLayout(4, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Search");
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				value = getVirtual();
				shell.close();
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Save..");	
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {				
				new NewVirtualDialog(shell, controller, getVirtual());
				shell.close();
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Add criterion");		
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				new Criterion(criteriaPane);
				shell.pack();
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");
		
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});

		shell.pack();		
	}
	
	/**
	 * Displays a dialog and returns a new virtual representing the composite search.
	 * @return a virtual or null if cancelled or saved
	 */
	public Virtual open() {		
		value = null;
		shell.open();
		
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}		
		
		return value;
	}
}
