package search;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import search.SearchPreferences.SEARCH_OPTION;
import util.IconFactory;
import view.dialogs.NewVirtualDialog;
import controller.ActionManager;
import controller.Controller;
import data.Channel;
import data.Container;
import data.Feed;
import data.Virtual;
import data.VirtualFactory;

/**
 * Implementation of pluggable search control.
 * @author Kuat
 *
 */
public class SearchBox {

	private Text search; 
	private final SearchObserver display;
	private SearchPreferences config;
	private final Shell shell;
	private final Controller controller;
	private final boolean forcedSQL;
	
	private void search() { 
		if (search.getText().trim().length() == 0) {
			display.cancelSearch();
			return;
		}
		Virtual v = generateVirtual();
		if (v != null)
			display.displayResults(v);
	}
	
	private Virtual generateVirtual() {
		Virtual result = null;
		Container current = display.currentView();
		
		// Auto-decide which search option to use
		if (forcedSQL || ((config.searchWithinView && (current != null) && 
				(((current instanceof Virtual) && (!((Virtual) current).isFullText())) || (current instanceof Channel))))) 
			config.searchOption = SEARCH_OPTION.SQL;
		else
			config.searchOption = SEARCH_OPTION.LUCENE;
		
		
		switch(config.searchOption) {
		case SQL:
			result = VirtualFactory.keywordSQLSearch("Keyword search for '" + search.getText() + "'", search.getText());
			break;
		case LUCENE:
			result = VirtualFactory.keywordLuceneSearch("Keyword search for '" + search.getText() + "'", search.getText());
			break;
		}

		if (config.searchWithinView) {			
			if (current != null) {
				if (current instanceof Virtual) {
					result = VirtualFactory.intersect((Virtual) current, result);
				} else if (current instanceof Feed) {
					result = VirtualFactory.intersect(VirtualFactory.createVirtualFromFeed((Feed) current, config.searchOption == SEARCH_OPTION.LUCENE), result);				
				} else if (current instanceof Channel) {
					result = VirtualFactory.intersect(VirtualFactory.createVirtualFromChannel((Channel) current), result);
				}
			}				
		} 		
		
		if (config.unreadOnly)
			result = VirtualFactory.unreadOnly(result);

		if (config.limit) 
			result = VirtualFactory.limitVirtual(result, config.limitFactor);

		return result;
	}
		
	/**
	 * Constructs a search box on the parent composite.
	 * @param c controller
	 * @param parent
	 * @param incremental enables incremental search
	 * @param fullText enables index search 
	 * @param searchWithinView forces 'search within view' option
	 */
	public SearchBox(Controller c, Composite parent, SearchObserver view, boolean incremental, boolean fullText, boolean searchWithinView) {
		// Controller
		this.controller = c;
		this.shell = parent.getShell();
		this.display = view;
		
		// Settings
		String name = this.getClass().getName() + "." + display.getClass().getName();
		config = (SearchPreferences) controller.getSettings(name);
		if (config == null) {		
			config = new SearchPreferences();
		}
		
		// Forced settings
		if (!incremental) config.incremental = false;
		if (!fullText) {
			config.searchOption = SEARCH_OPTION.SQL;
			forcedSQL = true;
		} else
			forcedSQL = false;
		if (searchWithinView)
			config.searchWithinView = true;
		
		// GUI elements
		final ToolBar bar = new ToolBar(parent, SWT.FLAT);		
		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gd.minimumWidth = 200;
		bar.setLayoutData(gd);
		
		// Menu
		final Menu menu = new Menu (shell, SWT.POP_UP);
		
		MenuItem option;
		option = new MenuItem (menu, SWT.CHECK);
		option.setText ("Limit search results");
	    option.setSelection(config.limit);
		option.addListener(SWT.Selection, new Listener() {
		      public void handleEvent(Event event) {
		        config.limit = !config.limit;
		        search();
		      }
		    });
		
		option = new MenuItem (menu, SWT.CHECK);
		option.setText ("Search within the current view");
		option.setSelection(config.searchWithinView);
		option.addListener(SWT.Selection, new Listener() {
		      public void handleEvent(Event event) {
		        config.searchWithinView = !config.searchWithinView;
		        search();
		      }
		    });

		if (incremental) {
			option = new MenuItem (menu, SWT.CHECK);
			option.setText("Incremental search");
			option.setSelection(config.incremental);
			option.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					config.incremental = !config.incremental;
				}
			});
		}
	
		option = new MenuItem (menu, SWT.CHECK);
		option.setText("Display only unread articles");
		option.setSelection(config.unreadOnly);
		option.addListener(SWT.Selection, new Listener() {
		      public void handleEvent(Event event) {
		        config.unreadOnly = !config.unreadOnly;
		        search();
		      }
		    });

//		option = new MenuItem (menu, SWT.SEPARATOR);
//		option = new MenuItem (menu, SWT.RADIO);
//		option.setText("SQL search");
//		if (config.searchOption == SearchPreferences.SEARCH_OPTION.SQL)
//			option.setSelection(true);
//		option.addListener(SWT.Selection, new Listener() {
//			public void handleEvent(Event event) {
//				config.searchOption = SearchPreferences.SEARCH_OPTION.SQL;
//				search();
//			}
//		});		
//		option = new MenuItem (menu, SWT.RADIO);
//		option.setText("Use Lucene index");
//		if (config.searchOption == SearchPreferences.SEARCH_OPTION.LUCENE)
//			option.setSelection(true);
//		option.addListener(SWT.Selection, new Listener() {
//			public void handleEvent(Event event) {
//				config.searchOption = SearchPreferences.SEARCH_OPTION.LUCENE;
//				search();
//			}
//		});
		
		option = new MenuItem (menu, SWT.SEPARATOR);
		option = new MenuItem (menu, SWT.PUSH);
		option.setText("Save search...");
		option.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					new NewVirtualDialog(shell, controller, generateVirtual());
				}
		});
		option = new MenuItem (menu, SWT.PUSH);
		option.setText("Advanced search...");
		option.addListener(SWT.Selection, controller.getActionManager().getAction(ActionManager.Actions.SEARCH));
				
		final ToolItem item = new ToolItem (bar, SWT.DROP_DOWN);
		item.setImage(new Image(parent.getDisplay(),IconFactory.getIcon(IconFactory.ICON.SEARCH)));
		item.setToolTipText("Search");
		item.addListener (SWT.Selection, new Listener () {
			public void handleEvent (Event event) {
				if (event.detail == SWT.ARROW) {
					Rectangle rect = item.getBounds ();
					Point pt = new Point (rect.x, rect.y + rect.height);
					pt = bar.toDisplay (pt);
					menu.setLocation (pt.x, pt.y);
					menu.setVisible (true);
				} else
					search();
			}
		});		
		
		ToolItem sep = new ToolItem (bar, SWT.SEPARATOR);	
		sep.setWidth(165);
		search = new Text(bar, SWT.SINGLE | SWT.BORDER);		
		sep.setControl (search);
		
		final ToolItem clear = new ToolItem (bar, SWT.PUSH);		
		clear.setImage(new Image(parent.getDisplay(),IconFactory.getIcon(IconFactory.ICON.SEARCH_CLEAR)));
		clear.setToolTipText("Clear search results");
		clear.addListener(SWT.Selection, new Listener () {
			public void handleEvent (Event event) {
				display.cancelSearch();
				search.setText("");
			}
		});
		
		bar.pack();
		
		search.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (config.incremental) {
					String text = search.getText();
					if (text.length() > 0){
						if (display != null)
							search();
					} else
						if (display != null)
							display.cancelSearch();
				}
			}
		});		
	    search.addKeyListener(new KeyAdapter() {
	        public void keyPressed(KeyEvent event) {
	          switch (event.keyCode) {
	          case SWT.CR:
	        	  if (display != null)
	        		  search();
	        	  break;
	          case SWT.ESC:
	        	  if (display != null)
	        		  display.cancelSearch();
	        	  break;
	          }
	        }
	      });
	   	    
	}

	/**
	 * Terminates the component and saves its settings.
	 *
	 */
	public void terminate() {
		String name = this.getClass().getName() + "." + display.getClass().getName();
		controller.saveSettings(name, config);
	}

}
