package settings;

import java.lang.reflect.Field;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import util.SFontData;
import util.SRGB;

/**
 * An abstract class representing a dialog for modifying font and color CSS styles.
 * @author oleg
 */
abstract class AppearanceDialog {
	/**
	 * A browser component to display changes.
	 */
	final protected Browser browser;
	/**
	 * The dialog shell.
	 */
	final protected Shell shell;
	/**
	 * The composite containing the browser component.
	 */
	final protected Composite input;
	/**
	 * Displays style changes. 
	 */
	abstract protected void showAppearance();
	/**
	 * Cancel button callback.
	 */
	protected void cancelClicked() {
		shell.close();
	}
	/**
	 * OK button callback.
	 */
	protected void okClicked() {
		applyClicked();
		shell.close();
	}
	/**
	 * Apply button callback.
	 */
	abstract protected void applyClicked();
	
	/**
	 * Adds a button for selection a font.
	 * @param buttonText text to be displayed on the button
	 * @param dialogText text to be displayed in the font selection dialog title
	 * @param object the object to modify upon selecting a font
	 * @param fontField the name of the font field to modify
	 * @param colorField the name of the color field to modify (on some platforms one can select both color and font in one dialog)
	 */
	protected void addFontButton(final String buttonText, final String dialogText, final Object object, final String fontField, final String colorField) {
		Button button = new Button(input, SWT.NONE);
		button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		button.setText(buttonText);
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				FontDialog fd = new FontDialog(shell);
				fd.setText(dialogText);
				Field ff;
				try {
					ff = object.getClass().getDeclaredField(fontField);
					fd.setFontList(new FontData[] {((SFontData) ff.get(object)).toFontData()});
					FontData newdata = fd.open();
					if (newdata != null) {
						ff.set(object, new SFontData(newdata));
						RGB newcolor = fd.getRGB(); 
						if (newcolor != null) object.getClass().getDeclaredField(colorField).set(object, new SRGB(newcolor)); 
						showAppearance();
					}
				} catch (SecurityException e) {
					e.printStackTrace();
				} catch (NoSuchFieldException e) {
					e.printStackTrace();
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
			}
		});
	}
	/**
	 * Adds a button for selecting color.
	 * @param buttonText the text to be displayed on the button
	 * @param dialogText the text to be displayed in the color selection dialog title
	 * @param object object to modify upon selecting a color
	 * @param colorField the name of the field of the object to set
	 */
	protected void addColorButton(final String buttonText, final String dialogText, final Object object, final String colorField) {
		Button button = new Button(input, SWT.NONE);
		button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		button.setText(buttonText);
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ColorDialog cd = new ColorDialog(shell);
				cd.setText(dialogText);
				try {
					Field cf = object.getClass().getDeclaredField(colorField);
					cd.setRGB(((SRGB) cf.get(object)).toRGB());
					RGB newdata = cd.open();
					if (newdata != null) {
						cf.set(object, new SRGB(newdata)); 
						showAppearance();
					}
				} catch (SecurityException e) {
					e.printStackTrace();
				} catch (NoSuchFieldException e) {
					e.printStackTrace();
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
			}
		});
	}
	/**
	 * Creates a new dialog. A subclass should create an event-dispatching loop itself.
	 * @param parent parent shell
	 * @param rows the number of buttons to adjust the layout for
	 */
	protected AppearanceDialog(Shell parent, int rows) {
		shell = new Shell(parent,SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
		shell.setText("Fonts and colors");
		shell.setLayout(new GridLayout(1,false));
		input = new Composite(shell,SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));
		input.setLayout(new GridLayout(2,false));
		browser = new Browser(input,SWT.NONE);
		GridData browserData = new GridData(GridData.FILL_BOTH);
		browserData.widthHint = 300;
		browserData.verticalSpan = rows;
		browser.setLayoutData(browserData);

		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttons.setLayout(new GridLayout(3,false));
		
		Button button = new Button(buttons, SWT.NONE);
		button.setText("OK");
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				okClicked();
			}
		});
		
		button = new Button(buttons, SWT.NONE);
		button.setText("Apply");
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				applyClicked();
			}
		});
		
		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				cancelClicked();
			}
		});
		
	}

}
