package test;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import data.ArticleContent;
import data.ArticleInfo;
import data.Channel;
import data.DataException;
import data.DataManager;
import data.DataManagerDBImpl;
import data.Feed;
import data.GeoPoint;
import data.Virtual;
import data.VirtualFactory;

public class DBTest extends TestCase {
	DataManager data = DataManagerDBImpl.getInstance();
	Feed f;
	Channel c;
	long fixedTime = System.currentTimeMillis();
	
    public static Test suite() {
        TestSuite suite = new TestSuite();

        suite.addTest(new DBTest("testAccess"));
        suite.addTest(new DBTest("testChannels"));
        suite.addTest(new DBTest("testFeeds"));
        suite.addTest(new DBTest("testChannelAdd"));
        suite.addTest(new DBTest("testFeedAdd"));
        suite.addTest(new DBTest("testArticles"));
        suite.addTest(new DBTest("testConcurrency"));
        suite.addTest(new DBTest("testRemoveFeed"));
        suite.addTest(new DBTest("testRemoveChannel"));
        suite.addTest(new DBTest("testClose"));
        
        return suite;
    }
	
    public DBTest(String name) {
        super(name);
    }    

    /**
     * Tests initialization.
     * 
     */
    public void testAccess() {	
    	data.initialize(System.getProperty("user.home", ".")+ "/.rss/TEST/");
    	
    	//cleanup
    	try{
    		Feed feed = data.getFeed("TEST");
    		if (feed != null) 
    			data.removeFeed(feed);
    		Channel c = data.getChannel("TEST");
    		if (c!=null) 
    			data.removeChannel(c);    		
    	} catch (DataException e){
    		e.printStackTrace();
    	}
    }
    
    /**
     * Tests termination.
     *
     */
    public void testClose() {
    	data.terminate();
    }
    
    /**
     * Shows list of default channels.
     *
     */
    public void testChannels() {
    	for(Channel c: data.getChannels()) {
    		System.out.println("Channel " + c.getName());
    	}
    }
    
    /**
     * Shows list of default feeds (nicely formatted)
     *
     */
    public void testFeeds() {
    	for(Feed f: data.getFeeds()) {
    		System.out.println(f.getName() + " " + f.getURL());
    		System.out.println("	" + f.getChannel().getName() + " " + f.getUpdateInterval());    		
    	}
    }
    
    /**
     * Tests addition and renaming of a channel.
     *
     */
    public void testChannelAdd() {    	
    	if (data.getChannel("TEST") == null) {
    		try {
    			c = data.addChannel("TEST2");
    			data.renameChannel(c, "TEST");
    			assertTrue(c.getName().equals("TEST"));
    		} catch (DataException e) {
    			fail("Addition failed");
    		}
    	}
    }
    
    /**
     * Tests addition, renaming, and moving of a feed.
     * Also tests deleting a feed.
     *
     */
    public void testFeedAdd() {
    	c = data.getChannel("TEST");
    	f = new Feed("TEST2", "TEST", 0, c, 1000000, null);
    	if (data.getFeed(f.getURL()) == null)
    		try {
    			data.addFeed(f);
    			data.renameFeed(f, "TEST");
    			assertTrue(f.getName().equals("TEST"));
    			
    			Channel d = data.addChannel("TEST2");
    			data.moveFeed(f, d);
    			assertTrue(f.getChannel().equals(d));
    			data.moveFeed(f, c);
    			data.removeChannel(d);
    			
    		} catch (DataException e) {
    			fail("Feed add failed");
    		}
    }

    /**
     * Tests article addition and changing read status.
     * Also tests all message counters provided.
     *
     */
    public void testArticles() {
    	f = data.getFeed("TEST");
    	List<ArticleInfo> ais = new ArrayList<ArticleInfo>();
    	for(int i = 0; i < 1000; i++) {
    		ArticleInfo ai = new ArticleInfo("Article" + i, f, fixedTime, "Kuat", "mit.edu", false, false, new GeoPoint(1,1));
    		ais.add(ai);
    		ArticleContent ac = new ArticleContent("BLAH");
    		try {
				data.addArticle(ai, ac);
			} catch (DataException e) {
				e.printStackTrace();
			}
    	}
    	
    	assertTrue(data.unreadArticles(f) == 1000);
    	assertTrue(data.unreadArticles(VirtualFactory.createVirtualFromChannel(data.getChannel("TEST"))) == 1000);
    	
    	for (ArticleInfo ai: ais) {
    		try {
    			data.setReadArticle(ai, true);
    		} catch (DataException e) {
    			e.printStackTrace();
    			fail("Cannot set article to read");
    		}
    	}
    	assertTrue(data.unreadArticles(f) == 0);
    	assertTrue(data.unreadArticles(VirtualFactory.createVirtualFromChannel(data.getChannel("TEST"))) == 0);
    	assertTrue(data.numberArticles(f) == 1000);
    	assertTrue(data.numberArticles(VirtualFactory.createVirtualFromChannel(data.getChannel("TEST"))) == 1000);

    	
    	Iterator<ArticleInfo> iter = data.getArticles(f);
    	int count = 0;
    	while(iter.hasNext()) {
    		count ++ ;
    		ArticleInfo ai = iter.next();  
    		assertTrue(ais.contains(ai));
    		assertTrue(ai.getPublishedDate() == fixedTime);
    		assertTrue(ai.getURL().equals("mit.edu"));
    		assertTrue(ai.getAuthors().equals("Kuat"));
    		assertTrue(ai.getPoint().getLatitude() == 1);
    		assertTrue(ai.getPoint().getLongitude() == 1);
    	}
    	
    	assertTrue(count == 1000);
    }
   
    /**
     * Tests removing a feed and removing articles.
     * Tests iterator observers.
     * Tests retrieving a channel.
     * Additional test of counters.
     *
     */
    public void testRemoveFeed() {
    	f = data.getFeed("TEST");
    	assertTrue(f!= null);
    	
    	// removing half of them
    	Iterator<ArticleInfo> iter = data.getArticles(f);
    	int i = 0;
    	while(iter.hasNext()) {
    		i++;
    		if (i > 2000) break;
    		ArticleInfo ai = iter.next();
    		try {
				data.removeArticle(ai);
			} catch (DataException e) {
				e.printStackTrace();
				fail("cannot remove article");
			}
    	}
    	
    	assertTrue(data.unreadArticles(f) == 0);

    	try {
			data.removeFeed(f);
		} catch (DataException e) {
			e.printStackTrace();
			fail("Cannot remove feed");
		}
		
		assertTrue(data.getFeed("TEST") == null);
    }
    
    /**
     * Tests removing a channel.
     * Tests retrieving a channel.
     */
    public void testRemoveChannel() {
    	c = data.getChannel("TEST");
    	assertTrue(c!=null);
    	try {
    		data.removeChannel(c);
    	} catch (DataException e){
    		fail("Cannot remove channel");
    	}
    }
    
    public void testConcurrency() {
    	final Virtual all = VirtualFactory.allOrdered("haha");
    	Thread t1 = new Thread () { 
    		public void run() {
    			Iterator<ArticleInfo> iterator = data.getArticles(all);
    			while(iterator.hasNext()) 
    				iterator.next().toString();
    		}
    	};
    	t1.start();
    	
    	Thread t2 = new Thread () { 
    		public void run() {
    			Iterator<ArticleInfo> iterator = data.getArticles(all);
    			while(iterator.hasNext())
    				iterator.next().toString();
    		}
    	};
    	t2.start();
    	
    	Thread t3 = new Thread () { 
    		public void run() {
    			Iterator<ArticleInfo> iterator = data.getArticles(all);
    			while(iterator.hasNext())
    				iterator.next().toString();
    		}
    	};
    	t3.start();
    	
    	List<Thread> t = new ArrayList<Thread>();
    	for(final Feed feed : data.getFeeds()) {
        	Thread tx = new Thread () { 
        		public void run() {
        			Iterator<ArticleInfo> iterator = data.getArticles(feed);
        			while(iterator.hasNext())
        				iterator.next().toString();
        		}
        	};
        	tx.start();
        	t.add(tx);    		
    	}
    	for(final Virtual feed : data.getVirtuals()) {
        	Thread tx = new Thread () { 
        		public void run() {
        			Iterator<ArticleInfo> iterator = data.getArticles(feed);
        			while(iterator.hasNext())
        				iterator.next().toString();
        		}
        	};
        	tx.start();
        	t.add(tx);    		
    	}
    	
    	Thread t4 = new Thread() {
    		public void run() {
    			f = data.getFeed("TEST");
    	    	for(int i = 0; i < 1000; i++) {
    	    		ArticleInfo ai = new ArticleInfo("Article" + i + "000", f, fixedTime, "Kuat", "mit.edu", false, false, null);
    	    		ArticleContent ac = new ArticleContent("BLAH");
    	    		try {
    					data.addArticle(ai, ac);
    				} catch (DataException e) {
    					e.printStackTrace();
    				}
    	    	}
    		}
    	};
    	
    	t4.start();    		
    	
    	
    	try {
			t1.join();
	    	t2.join();
	    	t3.join();
	    	t4.join();
	    	for(Thread tx : t)
	    		tx.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
    }
}
