package util;

import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;

/**
 * Array/list utilites.
 * @author oleg
 */
public class SortUtils {
	/**
	 * Merges an unsorted collection into a sorted list. Efficient for random-access lists. 
	 * @param <T> generic type 
	 * @param sortedList sorted list to merge into
	 * @param collection unsorted collection
	 * @param compare sort criterion
	 * @requires the order in <code>sortedList</code> to be consistent with <code>compare</code>
	 * @modifies <code>sortedList</code>  
	 */
	@SuppressWarnings("unchecked")
	public static <T> void addToSorted(List<T> sortedList, Collection<T> collection, Comparator<? super T> compare) {
		T[] array = (T[]) collection.toArray();
		Arrays.sort(array, compare);
		int sortedUp = sortedList.size() - 1;
		int arrayUp = array.length - 1;
		int mergeUp = sortedUp + arrayUp + 1;
		for (int i = 0; i< array.length; i++) sortedList.add(null);
		while (arrayUp >= 0 && sortedUp >= 0) {
			T sortMax = sortedList.get(sortedUp);
			T arrayMax = array[arrayUp];
			int res = compare.compare(sortMax, arrayMax);
			if (res > 0) {
				sortedList.set(mergeUp, sortMax);
				sortedUp --;
			} else {
				sortedList.set(mergeUp, arrayMax);
				arrayUp --;
			} 
			mergeUp --;
		}
		if (sortedUp == -1) {
			for (int i = 0; i<=arrayUp; i++) {
				sortedList.set(i, array[i]);
			}
		}
		
	}
	/**
	 * Removes elements from the given positions of a list. Efficient for random-access lists.
	 * @param <T> generic type
	 * @param list a list
	 * @param indices an array of indices
	 * @requires all elements of <code>indices</code> to be from 0 to <code>list.size() - 1</code>
	 * @modifies list, sorts indices
	 */
	public static <T> void removeAll(List<T> list, int[] indices) {
		Arrays.sort(indices);
		for (int i=1; i<indices.length; i++) {
			for (int j=indices[i-1]+1; j<indices[i]; j++) {
				list.set(j-i, list.get(j));
			}
		}
		final int listSize = list.size();
		int lastInd = indices[indices.length - 1];
		for (int j=lastInd + 1; j< listSize;j++) {
			list.set(j-indices.length, list.get(j));
		}
		for (int j=listSize-1; j >= listSize - indices.length; j--) {
			list.remove(j);
		}
	}
	
	/**
	 * Searches for the position in a sorted part of an array where an element would be inserted to preserve the order.
	 * @param <T> generic type
	 * @param a sorted array
	 * @param fromIndex the index of the first element of the sorted part of the array
	 * @param toIndex the index of the last element of the sorted part of the array <b>plus 1</b>
	 * @param key the element which position is to be found 
	 * @param comparator comparator giving the order of the sorted part of the array
	 * @return an index within <code>fromIndex</code> and <code>toIndex</code> such that if <code>key</code> were inserted there, the order in the sorted part of the array would be preserved. 
	 */
	public static <T> int binaryPositionSearch(T[] a, int fromIndex, int toIndex, T key, Comparator<? super T> comparator) {
		int start = fromIndex;
		int end = toIndex;
		while (start < end) {
			int middle = (start + end) >> 1;
			int result = comparator.compare(key, a[middle]);
			if (result == 0) return middle;
			else if (result < 0) end = middle;
			else if (result > 0) start = middle + 1;
		} return start;
	}
}
