package view;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import search.SearchBox;
import settings.GoogleMapsViewSettings;
import util.IconFactory;
import util.IconFactory.ICON;
import view.components.FolderPane;
import view.components.MapPane;
import view.components.StatusBar;
import controller.Controller;
import controller.StandardMenu;
import data.ArticleInfo;
import data.Channel;
import data.Feed;
import data.Virtual;

/**
 * View targeted for GeoRSS extension and uses the spacial information from the article
 * to identify its position on the Google Maps.
 * @author Kuat
 *
 */
public class GoogleMapsView extends View {
	
	private final MapPane mapPane;
	private final FolderPane folderPane;
	private final SearchBox search;
	private final StatusBar statusBar;
	private final Label status;
	private final Menu menuBar;
	private final SashForm sash;
	private final Combo combo;
	public Menu getMenuBar() {
		return menuBar;
	}
	
	public GoogleMapsView(Shell shell, final Controller controller) {
		super(shell, controller);		
		GridLayout layout = new GridLayout(1,false);
		layout.verticalSpacing = 0;
		layout.horizontalSpacing = 0;
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		this.setLayout(layout);
		
		GoogleMapsViewSettings settings = (GoogleMapsViewSettings) controller.getSettings(getClass().getName());
		if (settings == null) settings = GoogleMapsViewSettings.defaultSettings();
		
		StandardMenu stMenu = controller.getActionManager().createStandardMenuBar();
		
		menuBar = stMenu.getSWTMenu();		
		
		Composite input = new Composite(this, SWT.None);
		GridLayout barLayout = new GridLayout(2, false);
		barLayout.verticalSpacing = 0;
		barLayout.marginHeight = 0;
		input.setLayout(barLayout);
		input.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ToolBar toolbar = controller.getActionManager().createStandardToolbar(input);
		toolbar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		

		
		ToolItem ti = new ToolItem(toolbar, SWT.SEPARATOR);
		
		ti = new ToolItem(toolbar,SWT.FLAT);
		ti.setImage(IconFactory.getIcon(getDisplay(),ICON.DELETE));
		ti.setToolTipText("Stop");
		ti.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				mapPane.stopDisplayOperation();
				status.setText("Stopped.");
			}
		});

		ti = new ToolItem(toolbar,SWT.FLAT);
		ti.setImage(IconFactory.getIcon(getDisplay(),ICON.GOOGLE_MAPS));
		ti.setToolTipText("Refresh view");		
		ti.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				mapPane.refresh();
			}
		});
		
		new ToolItem(toolbar, SWT.SEPARATOR);
		
		ToolItem sep = new ToolItem (toolbar, SWT.SEPARATOR);
		Label label = new Label(toolbar, SWT.None);
		label.setText("Maximum number of articles to show:");
		label.pack();
		sep.setWidth(label.getSize().x);
		sep.setControl(label);		
		
		sep = new ToolItem(toolbar, SWT.SEPARATOR);
		combo = new Combo(toolbar, SWT.READ_ONLY | SWT.BORDER);
		combo.add("25"); combo.add("50"); combo.add("100"); 
		combo.select(combo.indexOf(String.valueOf(settings.limit)));
		
		
		combo.pack();
		sep.setWidth(combo.getSize().x);
		sep.setControl(combo);
		combo.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				mapPane.setLimit(Integer.valueOf(combo.getText()));
			}
		});				
				
		sash = new SashForm(this,SWT.HORIZONTAL);
		
		folderPane = new FolderPane(sash,controller);
		
		
		Composite bottom = new Composite(this, SWT.NONE);
		GridLayout bLayout = new GridLayout(3, false);
		bLayout.verticalSpacing = 0;
		bLayout.marginHeight = 0;
		bottom.setLayout(bLayout);
		bottom.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		status = new Label(bottom, SWT.NONE);
		status.setText("Select a feed to show articles on the map");
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = settings.statusWidth;
		status.setLayoutData(gd);
		
		mapPane = new MapPane(sash,dataManager,folderPane,status,settings.limit);
		
		gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 2;
		gd.heightHint = settings.mainHeight;
		gd.widthHint = settings.mainWidth;
		sash.setLayoutData(gd);		
		sash.setWeights(settings.mainWeights);
		
		folderPane.addContainerSelectionObserver(mapPane);
		
		for(Channel c: dataManager.getChannels()) {
			if (!c.getName().equals("")) folderPane.addChannel(c);
		}

		for(Feed f: dataManager.getFeeds()) {
			folderPane.addFeed(f);
		}
		
		for(Virtual v: dataManager.getVirtuals()) {
			folderPane.addVirtual(v);
		}

		
		
		search = new SearchBox(controller, input, mapPane, false, true, true);

		statusBar= new StatusBar(bottom, controller, SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		statusBar.setLayoutData(gd);
		
		folderPane.setSelection(settings.folderPaneSelection[0], settings.folderPaneSelection[1]);
		
		downloadManager.addDownloadObserver(statusBar);
	}
			
		@Override
	public void terminate() {
		controller.saveSettings(getClass().getName(), new GoogleMapsViewSettings(sash.getSize().x,sash.getSize().y,status.getSize().x,sash.getWeights(),Integer.parseInt(combo.getText()),folderPane.getSelection()));
		super.terminate();
		downloadManager.removeDownloadObserver(statusBar);
		search.terminate();
		mapPane.stopDisplayOperation();
	}

	public void newChannel(final Channel channel) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addChannel(channel);
			}
		});
	}

	public void newFeed(final Feed feed) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addFeed(feed);
			}
		});
	}

	public void newVirtual(final Virtual virtual) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addVirtual(virtual);
			}
		});
	}
	
	public void newArticles(Collection<ArticleInfo> info) {
		// same code as in other views
		Iterator<ArticleInfo> iter = info.iterator();
		if (iter.hasNext()) folderPane.feedUpdated(iter.next().getFeed(), true);
		mapPane.cancelSearch();
	}

	
	
}
