package view;


import java.util.Collection;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import search.SearchBox;
import settings.MailPaneAppearanceDialog;
import settings.MailViewSettings;
import util.IconFactory;
import util.IconFactory.ICON;
import view.components.FolderPane;
import view.components.RSSMailPane;
import view.components.StatusBar;
import controller.Controller;
import controller.StandardMenu;
import data.ArticleInfo;
import data.Channel;
import data.DNDArticle;
import data.Feed;
import data.Virtual;

/**
 * The standard "e-mail client" view.
 * @author oleg
 */
public class RSSMailView extends View {
	private final FolderPane folderPane;
	private final RSSMailPane mailPane;
	private final SearchBox search;
	private final StatusBar statusBar;	
	private final SashForm sashForm; 

	private final Menu menuBar;

	public Menu getMenuBar() {
		return menuBar;
	}

	/**
	 * Creates a new view on a given shell with a given controller
	 * @param parent parent shell
	 * @param ctrl the controller
	 */
	@SuppressWarnings("unchecked")
	public RSSMailView(final Shell parent, Controller ctrl) {
		super(parent, ctrl);
		MailViewSettings settings = (MailViewSettings) controller.getSettings(this.getClass().getName());
		if (settings == null) settings = MailViewSettings.defaultSettings();
		GridLayout mailViewLayout = new GridLayout(2,false);
		mailViewLayout.horizontalSpacing = 0;
		mailViewLayout.marginWidth = 0;
		mailViewLayout.marginHeight = 0;
		mailViewLayout.verticalSpacing = 0;
		this.setLayout(mailViewLayout);

		Composite bar = new Composite(this, SWT.None);
		bar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout barLayout = new GridLayout(2, false);
		barLayout.verticalSpacing = 0;
		barLayout.marginHeight = 0;
		bar.setLayout(barLayout);
		controller.getActionManager().createStandardToolbar(bar).setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		sashForm = new SashForm(this,SWT.HORIZONTAL);
		folderPane = new FolderPane(sashForm,controller);
		mailPane = new RSSMailPane(sashForm,controller,folderPane,settings.appearance,settings.sortCriterion, settings.sortDirection,settings.columnWidthData,settings.mailPaneOrientation,settings.weights);
//		mailPane.setWeights(settings.weights.get(settings.mailPaneOrientation));
		sashForm.setWeights(settings.mainWeights);
		search = new SearchBox(controller, bar, mailPane, true, true, false);

//		folderPane.addFolderPaneSelectionObserver(mailPane);

		for(Channel c: dataManager.getChannels()) {
			if (!c.getName().equals("")) folderPane.addChannel(c);
		}

		for(Feed f: dataManager.getFeeds()) {
			folderPane.addFeed(f);
		}

		for(Virtual v: dataManager.getVirtuals())
			folderPane.addVirtual(v);

		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 2;
		gd.heightHint = settings.mailPaneHeight;
		gd.widthHint = settings.mailPaneWidth;
		sashForm.setLayoutData(gd);

		statusBar = new StatusBar(this, controller, SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		statusBar.setLayoutData(gd);

		if (folderPane.setSelection(settings.folderPaneSelection[0], settings.folderPaneSelection[1])) {
			mailPane.setSelectedArticle(DNDArticle.toArticleInfo(settings.mailPaneSelection,controller.getDataManager()));
		}

		StandardMenu stMenu = controller.getActionManager().createStandardMenuBar();
		Menu viewMenu = stMenu.getSubMenu("View");
		MenuItem mi = new MenuItem(viewMenu,SWT.CASCADE,0);
		mi.setText("Fonts and colors...");
		mi.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				new MailPaneAppearanceDialog(parent,mailPane);
			}
		});
		mi.setImage(IconFactory.getIcon(getDisplay(), ICON.FONTS_AND_COLORS));
		
		mi = new MenuItem(viewMenu, SWT.CASCADE,1);
		mi.setText("Layout");
		Menu layoutMenu = new Menu(stMenu.getSWTMenu());
		mi.setMenu(layoutMenu);
		mi.setImage(IconFactory.getIcon(getDisplay(), ICON.LAYOUT));
		
		mi = new MenuItem(layoutMenu, SWT.CASCADE | SWT.RADIO);
		mi.setText("Standard");
		mi.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				mailPane.setMailPaneOrientation(SWT.VERTICAL);
			}
		});
		mi.setSelection(settings.mailPaneOrientation == SWT.VERTICAL);

		mi = new MenuItem(layoutMenu, SWT.CASCADE | SWT.RADIO);
		mi.setText("Three-column");
		mi.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				mailPane.setMailPaneOrientation(SWT.HORIZONTAL);
			}
		});
		mi.setSelection(settings.mailPaneOrientation == SWT.HORIZONTAL);

		new MenuItem(viewMenu,  SWT.SEPARATOR, 2);

		menuBar = stMenu.getSWTMenu();

//		mailPane.setColumnWidthData(settings.columnWidthData);

		this.pack();

		downloadManager.addDownloadObserver(statusBar);
	}
	/**
	 * Displays new articles
	 */
	public void newArticles(Collection<ArticleInfo> infos) {
		Iterator<ArticleInfo> iter = infos.iterator();
		if (iter.hasNext()) {
			Feed feed = iter.next().getFeed();
			folderPane.feedUpdated(feed, true);
			mailPane.newArticles(infos,feed);
		}

	}
	/**
	 * Displays a new feed.
	 */
	public void newFeed(final Feed feed) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addFeed(feed);
			}
		});
	}
	/**
	 * Reflects download finish.
	 */
	public void downloadFinished() {
		statusBar.downloadFinished();
	}
	/**
	 * Reflects download start.
	 */
	public void downloadStarted(Feed feed) {
		statusBar.downloadStarted(feed);
	}
	/**
	 * Displays a new folder.
	 */
	public void newChannel(final Channel channel) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addChannel(channel);
			}
		});
	}	
	/**
	 * Terminates the view.
	 */
	@Override
	public void terminate() {	
		super.terminate();
		downloadManager.removeDownloadObserver(statusBar);
		search.terminate();
		//weights.put(mailPane.getOrientation(), mailPane.getWeights());
		controller.saveSettings(this.getClass().getName(), 
				new MailViewSettings(
						sashForm.getSize().y,
						sashForm.getSize().x,
						sashForm.getWeights(),
						mailPane.getOrientation(),
						mailPane.getMailPaneWeights(),
						getShell().getLocation(),
						mailPane.getSortCriterion(),
						mailPane.getSortDirection(),
						folderPane.getSelection(),
						DNDArticle.fromArticleInfo(mailPane.getSelectedArticle()),
						mailPane.getColumnWidthData(),
						mailPane.getAppearance()));
	}
	/**
	 * Displays a new virtual
	 */
	public void newVirtual(final Virtual virtual) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addVirtual(virtual);
			}
		});
	}
}
