package view.components;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TreeEditor;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.HTMLTransfer;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import util.IconFactory;
import util.SortUtils;
import util.IconFactory.ICON;
import view.dialogs.ChangeIntervalDialog;
import view.dialogs.SubscribeDialog;
import view.dnd.ArticleTransfer;
import view.dnd.FeedTransfer;
import view.dnd.IEURLTransfer;
import controller.ActionManager;
import controller.Controller;
import controller.GlobalPreferences;
import data.Channel;
import data.Container;
import data.DNDArticle;
import data.DataException;
import data.Feed;
import data.Virtual;

/**
 * A pane with folders, virtuals and feeds.
 * @author oleg
 */
public class FolderPane extends Composite implements ContainerSelector {
	private final Tree treePane;
	private final Controller controller;
	private final Map<Channel, TreeItem> folders = new HashMap<Channel, TreeItem>();
	private final Map<Feed, TreeItem> feeds = new HashMap<Feed, TreeItem>(); 
	private static final String CONTAINER_KEY = "container"; 
	private final TreeEditor treeEditor;
	private final Font normalFont;
	private final Font boldFont;
	private int virtualCount = 0;
	private int folderCount = 0;
	private int localCount = 0;
	private Vector<ContainerSelectionObserver> selectionListeners = new Vector<ContainerSelectionObserver>(1);
	private TreeItem searchResults;
	// compares strings and tree items,
	// clearly violates the .equals contract and thus should only be used 
	// for binary search
	private static final Comparator<Object> treeItemComparator = new Comparator<Object>() {
		public int compare(Object o1, Object o2) {
			String s1 = o1 instanceof TreeItem? ((Container) ((TreeItem) o1).getData(CONTAINER_KEY)).getName() : (String) o1;
			String s2 = o2 instanceof TreeItem? ((Container) ((TreeItem) o2).getData(CONTAINER_KEY)).getName() : (String) o2;
			return s1.compareTo(s2);
		}
	};

	/**
	 * Sets selection in the folder pane and notifies observers.
	 * @param index index of the parent item or the item itself if there is no parent item or -1 if no selection should be set
	 * @param subindex index of the item itself among the siblings if there is a parent item or -1 if there is no parent item
	 * @return <code>true</code> if setting selection succeeded (including the case when no selection should be set), <code>false</code> if setting selection failed
	 */
	public boolean setSelection(int index, int subindex) {
		if (index >= 0) {
			try {
				TreeItem item = treePane.getItem(index);
				if (subindex >= 0) {
					TreeItem subitem = item.getItem(subindex);
					item.setExpanded(true);
					treePane.setSelection(subitem);
					Object o = subitem.getData(CONTAINER_KEY);
					if (o instanceof Container) notifyObserversAboutSelection((Container) o);
				} else {
					treePane.setSelection(item);
					Object o = item.getData(CONTAINER_KEY);
					if (o instanceof Container) notifyObserversAboutSelection((Container) o);
				}
				return true;
			} catch (IllegalArgumentException e) {
				return false;
			}
		}
		return true;
	}
	/**
	 * Retrieves information about the item selected in the folder pane. 
	 * @return an array of two numbers: 
	 * <ul> <li>if no item is selected, the first element is -1 </li>
	 * <li> if the selected item has no parent item, the first element is the index of the selected item and the second element is -1 </li>
	 * <li> if the selected item has a parent item, the first element is the index of the parent item and the second element is the index of the selected item among its siblings</li> </ul>
	 */
	public int[] getSelection() {
		if (treePane.getSelectionCount() > 0) {
			TreeItem item = treePane.getSelection()[0];
			TreeItem parentItem = item.getParentItem();
			if (parentItem == null) return new int[] {treePane.indexOf(item), -1};
			else return new int[] {treePane.indexOf(parentItem), parentItem.indexOf(item)};
		} else return new int[] {-1,-1};
	}

	/**
	 * Displays a new folder. 
	 * @param channel folder 
	 */
	public void addChannel(Channel channel) {
		TreeItem item = new TreeItem(treePane,SWT.NONE,SortUtils.binaryPositionSearch(treePane.getItems(), 0, folderCount, channel.getName(), treeItemComparator));
		item.setData(CONTAINER_KEY,channel);
		item.setImage(new Image(this.getDisplay(),IconFactory.getIcon(IconFactory.ICON.FOLDER)));
		item.setText(channel.getName());
		folders.put(channel, item);
		folderCount ++;
	}
	/**
	 * Refreshes the unread counter for a feed.
	 * @param feed the feed
	 */
	public void feedUpdated(final Feed feed, boolean newArticles) {
		getDisplay().asyncExec(new Runnable() {
			public void run() {
				if (!isDisposed()) {
					TreeItem feedItem = feeds.get(feed);
					if (feedItem != null) {
						int unread = controller.getDataManager().unreadArticles(feed);
						if (unread == 0) {
							feedItem.setFont(normalFont);
							feedItem.setText(feed.getName());
						} else {
							feedItem.setFont(boldFont);
							feedItem.setText(new StringBuffer(feed.getName()).append(" (").append(unread).append(")").toString());
						}
					}
				}
			}
		});

	}
	/**
	 * Displays a new feed
	 * @param feed the feed to display
	 */
	public void addFeed(Feed feed) {
		TreeItem subitem;
		if (feed.isLocal()) {
			subitem = new TreeItem(treePane,SWT.NONE);
			subitem.setImage(new Image(this.getDisplay(), IconFactory.getIcon(IconFactory.ICON.FOLDER_OUTBOX)));
			localCount ++;
		} else {
			TreeItem item = folders.get(feed.getChannel());
			subitem = new TreeItem(item, SWT.NONE, SortUtils.binaryPositionSearch(item.getItems(), 0, item.getItemCount(), feed.getName(), treeItemComparator));
			if (feed.getIcon() != null) {
				subitem.setImage(new Image(this.getDisplay(),feed.getIcon()));
			} else {
				subitem.setImage(new Image(this.getDisplay(),IconFactory.getIcon(IconFactory.ICON.FAVICON)));
			}
		}
		//subitem.setText(feed.getName());
		feeds.put(feed, subitem);
		feedUpdated(feed,true);
		subitem.setData(CONTAINER_KEY, feed);
	}
	/**
	 * Displays a new virtual.
	 * @param v the virtual to display
	 */
	public void addVirtual(Virtual v) {
		if (v.getName().equals(GlobalPreferences.SEARCH_RESULTS) && (searchResults != null)) {
			searchResults.setData(CONTAINER_KEY, v);
			setSelection(treePane.indexOf(searchResults), -1);
			return;
		}


		TreeItem subitem = new TreeItem(treePane, SWT.NONE, SortUtils.binaryPositionSearch(treePane.getItems(), folderCount + localCount, folderCount + localCount + virtualCount, v.getName(), treeItemComparator));
		subitem.setText(v.getName());
		subitem.setData(CONTAINER_KEY, v);
		subitem.setImage(new Image(this.getDisplay(),IconFactory.getIcon(IconFactory.ICON.FOLDER_SEARCH)));
		virtualCount ++;

		if (v.getName().equals(GlobalPreferences.SEARCH_RESULTS)) {
			searchResults = subitem;
			setSelection(treePane.indexOf(searchResults), -1);
		}
	}
	/**
	 * Adds a new selection observer.
	 * @param l selection observer to add
	 */
	public void addContainerSelectionObserver(ContainerSelectionObserver l) {
		synchronized (this) {
			selectionListeners.add(l);
		}
	}
	/**
	 * Removes a selection observer.
	 * @param l selection observer to remove
	 */
	public void removeFolderPaneSelectionObserver(ContainerSelectionObserver l) {
		synchronized (this) {
			selectionListeners.remove(l);
		}
	}

	@SuppressWarnings("unchecked")
	private void notifyObserversAboutSelection(Container container) {
		Vector<ContainerSelectionObserver> l;
		synchronized (this) {
			l = (Vector<ContainerSelectionObserver>) selectionListeners.clone();
		}
		for (ContainerSelectionObserver sl: l) {
			if (container instanceof Feed) {
				Feed feed = (Feed) container;
				if (feed.isLocal()) sl.outboxSelected(feed);
				else sl.feedSelected(feed);
			} else if (container instanceof Virtual) {
				sl.virtualSelected((Virtual) container, false);
			} else if (container instanceof Channel) {
				sl.channelSelected((Channel) container);
			}
		}
	}

	/**
	 * Creates a new folder/feed pane.
	 * @param parent parent pane
	 * @param controller the controller
	 */
	public FolderPane(final Composite parent, final Controller controller) {
		super(parent,SWT.NONE);
		this.controller = controller;
		this.setLayout(new FillLayout());
		treePane = new Tree(this, SWT.BORDER);
		treeEditor = new TreeEditor(treePane);
		treeEditor.horizontalAlignment = SWT.LEFT;
		treeEditor.grabHorizontal = true;
		setupDND();

		normalFont = treePane.getFont();
		FontData fontData = normalFont.getFontData()[0];
		fontData.setStyle(SWT.BOLD);
		boldFont = new Font(this.getDisplay(),fontData);

		new FolderPaneMenu();

		treePane.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				Object o = event.item.getData(CONTAINER_KEY);
				if (o instanceof Container) notifyObserversAboutSelection((Container) o);
			}
		});
	}
//	move feed
	private void tryMoveFeed(Feed feed, Channel channel) {
		try {
			controller.getDataManager().moveFeed(feed, channel);

			TreeItem item = folders.get(feed.getChannel());

			TreeItem subitem = new TreeItem(item, SWT.NONE, SortUtils.binaryPositionSearch(item.getItems(), 0, item.getItemCount(), feed.getName(), treeItemComparator));
			subitem.setData(CONTAINER_KEY, feed);
			subitem.setText(feed.getName());
			subitem.setImage(new Image(getDisplay(), feed.getIcon() == null? IconFactory.getIcon(IconFactory.ICON.FAVICON): feed.getIcon()));
			feeds.put(feed, subitem).dispose();
			feedUpdated(feed, false);
		} catch (DataException e) {
			controller.reportError("Moving the feed caused an error\n" + e.getMessage());
		}
	}

	private class FolderPaneMenu {
		final Image moveImage; 
		final Image renameImage;
		final Image deleteImage;
		final Image updateImage;
		final Image timeImage;
		final Image ljImage;
		final Image saveImage;
		final Menu popupMenu;
		FolderPaneMenu() {
			popupMenu = new Menu(treePane);
			treePane.setMenu(popupMenu);
			moveImage = IconFactory.getIcon(getDisplay(),ICON.MOVE);
			renameImage = IconFactory.getIcon(getDisplay(),ICON.RENAME);
			deleteImage = IconFactory.getIcon(getDisplay(),ICON.DELETE);
			updateImage = IconFactory.getIcon(getDisplay(),ICON.UPDATE);
			timeImage = IconFactory.getIcon(getDisplay(),ICON.TIME);
			ljImage = IconFactory.getIcon(getDisplay(),ICON.LIVEJOURNAL);
			saveImage = IconFactory.getIcon(getDisplay(), ICON.SAVE);

			popupMenu.addListener(SWT.Show, new Listener() {
				public void handleEvent(Event event) {
					TreeItem[] selection = treePane.getSelection();
					if (selection.length == 0) event.doit = false;
					else {
						for (MenuItem item: popupMenu.getItems()) item.dispose();
						Object container = ((TreeItem) treePane.getSelection()[0]).getData(CONTAINER_KEY);
						if (container instanceof Channel) {
							createRenameItem();
							createDeleteItem();
						} else if (container instanceof Virtual) {
							Virtual v = (Virtual) container;
							if (! v.getName().equals("Starred") && ! v.getName().equals("Unread")) {
								createDeleteItem();
							}
						} else if (container instanceof Feed) {
							Feed selected = (Feed) container;
							if (selected.isLocal()) {
								createRenameItem();
								createSaveItem();
								createLJItem();
							} else {
								createMoveItem();
								createRenameItem();
								createDeleteItem();
								createUpdateItem();
								createTimeItem();
							}
						}
					}
				}
			});
		}
		private void createMoveItem() {
			final MenuItem moveItem = new MenuItem(popupMenu,SWT.CASCADE);
			moveItem.setText("Move");
			moveItem.setImage(moveImage);
			final Menu moveTo = new Menu(popupMenu);
			moveItem.setMenu(moveTo); 
			moveTo.addListener(SWT.Show, new Listener() {
				public void handleEvent(Event event) {
					for (MenuItem item: moveTo.getItems()) item.dispose();
					final Feed feed = (Feed) treePane.getSelection()[0].getData(CONTAINER_KEY);
					for (final Channel channel: folders.keySet()) {
						if (!channel.equals(feed.getChannel())) {
							MenuItem item = new MenuItem(moveTo, SWT.CASCADE);
							item.setText(channel.getName());
							item.setImage(folders.get(channel).getImage());
							item.addListener(SWT.Selection, new Listener() {
								public void handleEvent(Event ev) {
									tryMoveFeed(feed, channel);
								}
							});
						}
					}
				}
			});
		}
		private void createRenameItem() {
			final MenuItem renameItem = new MenuItem(popupMenu,SWT.CASCADE);
			renameItem.setText("Rename");
			renameItem.setImage(renameImage);
			renameItem.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					final TreeItem ti = treePane.getSelection()[0];
					final Object container = ti.getData(CONTAINER_KEY);
					
						final Text inplace = new Text(treePane, SWT.NONE);
						inplace.setText(container instanceof Feed ? ((Feed) container).getName() : ((Channel) container).getName());
						inplace.setFocus();
						inplace.selectAll();
						inplace.addListener(SWT.KeyDown, new Listener() {
							public void handleEvent(Event event) {
								if (event.keyCode == SWT.ESC) inplace.dispose();
								else if (event.keyCode == SWT.CR) {
									String newname = inplace.getText();
									try {
										if (container instanceof Feed) {
											Feed feed = (Feed) container; 
											controller.getDataManager().renameFeed(feed, newname);
											if (feed.isLocal()) feedUpdated(feed, false);
											else {
												feeds.get(feed).dispose();
												addFeed(feed);
											}
										} else {
											controller.getDataManager().renameChannel((Channel) container, newname);
											ti.setText(newname);
										}
										inplace.dispose();
									} catch (DataException e) {
									}
								}
							}

						});
						inplace.addListener(SWT.FocusOut, new Listener() {
							public void handleEvent(Event event) {
								inplace.dispose();
							}

						});			
						treeEditor.setEditor(inplace, ti);
					
				}
			});
		}
		private void createDeleteItem() {
			final MenuItem deleteItem = new MenuItem(popupMenu,SWT.CASCADE);
			deleteItem.setText("Delete");
			deleteItem.setImage(deleteImage);
			deleteItem.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					final TreeItem ti = treePane.getSelection()[0];
					final Object container = ti.getData(CONTAINER_KEY);
					try {
						if (container instanceof Feed) {
							controller.getDataManager().removeFeed((Feed) container);
							feeds.remove((Feed) container);
						} else if (container instanceof Virtual) {
							//Virtual v = (Virtual) container;
							controller.getDataManager().removeVirtual((Virtual) container);
							virtualCount --;
						} else if (container instanceof Channel) {
							controller.getDataManager().removeChannel((Channel) container);
							folderCount --;
							folders.remove((Channel) container);
						} else 
							return;

						ti.dispose();

					} catch (DataException e) {
						if (e.isReferenceViolation() && (container instanceof Channel)) {
							MessageBox mb = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.YES | SWT.NO | SWT.PRIMARY_MODAL);
							mb.setText("Error");						
							mb.setMessage("The folder contains feeds. Would you like to remove them first?");
							if (mb.open() == SWT.YES) {
								try {
									for (Feed child : controller.getDataManager().getFeeds()) 
										if (child.getChannel().equals((Channel) container)) {
											controller.getDataManager().removeFeed(child);
											feeds.get(child).dispose();
											feeds.remove(child);
										}
									controller.getDataManager().removeChannel((Channel) container);
									folderCount --;
									folders.remove((Channel) container);
									ti.dispose();
								} catch (DataException x) {
									controller.reportError(e.getMessage());
								}
							}
						} else {
							controller.reportError(e.getMessage());
						}
					}				
				}
			});
		}
		private void createUpdateItem() {
			final MenuItem updateItem = new MenuItem(popupMenu, SWT.CASCADE);
			updateItem.setText("Update");
			updateItem.setImage(updateImage);
			updateItem.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					controller.getDownloadManager().forceDownload((Feed) treePane.getSelection()[0].getData(CONTAINER_KEY));
				}
			});
		}
		private void createTimeItem() {
			final MenuItem changeIntervalItem = new MenuItem(popupMenu, SWT.CASCADE);
			changeIntervalItem.setText("Set update interval");
			changeIntervalItem.setImage(timeImage);
			changeIntervalItem.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					new ChangeIntervalDialog(getShell(),controller,(Feed) treePane.getSelection()[0].getData(CONTAINER_KEY));
				}
			});
		}
		private void createLJItem() {
			final MenuItem ljItem = new MenuItem(popupMenu, SWT.CASCADE);
			ljItem.setText("Post to LiveJournal");
			ljItem.setImage(ljImage);
			ljItem.addListener(SWT.Selection, controller.getActionManager().getAction(ActionManager.Actions.POST_OUTBOX));
		}
		private void createSaveItem() {
			final MenuItem saveItem = new MenuItem(popupMenu, SWT.CASCADE);
			saveItem.setText("Save as...");
			saveItem.setImage(saveImage);
			saveItem.addListener(SWT.Selection, controller.getActionManager().getAction(ActionManager.Actions.WRITE_OUTBOX));
		}
	}

//	setup DND capabilities of feeds and articles
	private void setupDND() {
		DragSource ds = new DragSource(treePane, DND.DROP_MOVE);
		ds.setTransfer(new Transfer[] {FeedTransfer.instance()});
		ds.addDragListener(new DragSourceListener() {

			public void dragFinished(DragSourceEvent event) {
			}

			public void dragSetData(DragSourceEvent event) {
				TreeItem[] selection = treePane.getSelection();
				if (selection.length > 0) {
					Object container = selection[0].getData(CONTAINER_KEY);
					if (container instanceof Feed) event.data =((Feed) container).getURL();
				}
			}

			public void dragStart(DragSourceEvent event) {
				TreeItem[] selection = treePane.getSelection();
				if (selection.length == 0) event.doit = false;
				else {
					Object container = selection[0].getData(CONTAINER_KEY);
					if (container instanceof Feed) {
						if (((Feed) container).isLocal()) event.doit = false;
					} else event.doit = false;
				}
			}

		});

		DropTarget dt = new DropTarget(treePane, DND.DROP_MOVE | DND.DROP_LINK);
		dt.setTransfer(new Transfer[]{IEURLTransfer.instance(),TextTransfer.getInstance(),HTMLTransfer.getInstance(),ArticleTransfer.instance(), FeedTransfer.instance()});
		dt.addDropListener(new DropTargetAdapter() {
			public void dragOver(DropTargetEvent event) {
				if (event.item == null) event.detail = DND.DROP_NONE;
				else {
					final Object o = event.item.getData(CONTAINER_KEY);
					if (ArticleTransfer.instance().isSupportedType(event.currentDataType)) {
						if (o instanceof Feed && ((Feed) o).isLocal()) {
							event.detail = DND.DROP_MOVE;
							//event.feedback = DND.FEEDBACK_SELECT;
						} else { 
							event.detail = DND.DROP_NONE;
							//event.feedback = DND.FEEDBACK_NONE;
						}
					} else if (!(o instanceof Channel)) {
						event.detail = DND.DROP_NONE;
						//event.feedback = DND.FEEDBACK_NONE;
					} else if (IEURLTransfer.instance().isSupportedType(event.currentDataType)) {
						event.detail = DND.DROP_LINK;
					} else event.detail = DND.DROP_MOVE;
					//event.feedback = DND.FEEDBACK_SELECT;
				}
			}
			public void drop(final DropTargetEvent event) {
				if (event.item == null) event.detail = DND.DROP_NONE;
				else {
					final Object o = event.item.getData(CONTAINER_KEY);
					if (FeedTransfer.instance().isSupportedType(event.currentDataType)) {
						if (o instanceof Channel) tryMoveFeed(controller.getDataManager().getFeed((String) event.data), (Channel) o);
					} else {
						if (ArticleTransfer.instance().isSupportedType(event.currentDataType)) {
							if (o instanceof Feed && ((Feed) o).isLocal()) {
								if (event.data != null) {
									Object[] objects = (Object[]) event.data;
									for (Object dndArticle: objects) {
										try {
											controller.getDataManager().addArticle(
													DNDArticle.toArticleInfo(((DNDArticle) dndArticle),controller.getDataManager(), 
															controller.getDataManager().getFeed("rss-reader://outbox")), 
															controller.getDataManager().getContent(DNDArticle.toArticleInfo((DNDArticle) dndArticle,controller.getDataManager())));
										} catch (DataException e) {
										}
									}
								}
							}
						} else if (o instanceof Channel) {
							if (IEURLTransfer.instance().isSupportedType(event.currentDataType))  {
								// IE url DND - simply convert to String
								final String url = new String((byte[]) event.data);
								getDisplay().asyncExec(new Runnable() {
									public void run() {
										int idx = url.indexOf('\n'); 
										new SubscribeDialog(getShell(),controller,idx == -1? url : url.substring(0, idx),(Channel) o, "");
									}
								});
							} else getDisplay().asyncExec(new Runnable() {
								public void run() {
									// parse the <a href=... string
									StringTokenizer st = new StringTokenizer((String)event.data, "\n\r");
									while (st.hasMoreTokens()) {
										String content = st.nextToken();
										int idx = content.indexOf("href=");
										if (idx >= 0) {
											if (content.charAt(idx + 5) == '\"') {
												int end = content.indexOf('\"', idx + 6);
												if (end >= 0) content = content.substring(idx+6,end);
												else content = content.substring(idx+6);
											} else {
												int end = content.indexOf(' ', idx + 5);
												if (end >= 0) content = content.substring(idx+5,end);
												else content = content.substring(idx+5);
											}
										}
										if (content.startsWith("http://") || content.startsWith("https://") || content.startsWith("ftp://")) {
											new SubscribeDialog(treePane.getShell(),controller,content,(Channel)o,"");
											break;
										}
									}
								}
							});
						}
					}
				}
			}
		});
	}
}
