package view.components;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.derby.iapi.util.StringUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.beans.StringBean;
import org.htmlparser.filters.NodeClassFilter;
import org.htmlparser.tags.LinkTag;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;

import search.SearchObserver;
import util.Trace;
import webapi.GeoCoder;
import webapi.GoogleMapsWidget;
import webapi.OnLoadListener;
import webapi.WebConnectionException;
import webapi.GoogleMapsWidget.ZoomControl;
import data.ArticleInfo;
import data.Channel;
import data.Container;
import data.DataException;
import data.DataManager;
import data.Feed;
import data.GeoPoint;
import data.Virtual;
import data.VirtualFactory;

/**
 * A map component which can extract geographical locations from RSS articles and display them on a map.
 * @author kuat
 */
public class MapPane extends Composite implements ContainerSelectionObserver, SearchObserver {
	private final Label status;
	private List<ArticleInfo> cache;
	private Container currentView;
	private int limit;
	private Thread worker;
	private final GoogleMapsWidget mapPane;
	private final DataManager dataManager;
	
	public MapPane(Composite parent, final DataManager dataManager, final ContainerSelector selector, Label status, int limit) {
		super(parent, SWT.NONE);
		setLayout(new FillLayout());
		cache = new ArrayList<ArticleInfo>();
		this.status = status;
		this.limit = limit;
		this.dataManager = dataManager;
		mapPane = new GoogleMapsWidget(this, SWT.NONE, ZoomControl.HTML_SMALL, new OnLoadListener() {
			public void onLoad() {
				mapPane.addMapTypeControl();
			}			
		});
		mapPane.addLocationListener(new LocationListener() {

			public void changed(LocationEvent event) {
			}

			public void changing(LocationEvent event) {
				if (!event.location.startsWith("file://")) {
					if (event.location.startsWith("follow://")) {
						event.doit = false;
						Program.launch(event.location.substring("follow://".length()));
					} else
					if ((event.location.startsWith("dostar://")) || (event.location.startsWith("unstar://"))) {						
						event.doit= false;						
						final boolean star = event.location.startsWith("dostar://");
						int id = extractInteger(event.location);
						try {
							ArticleInfo article;
							synchronized(cache) {
								article = cache.get(id);
							}
							
							if (article == null) return;
							
							dataManager.setStarredArticle(article, star);
							selector.feedUpdated(article.getFeed(),false);
						} catch (DataException e) {
							e.printStackTrace();
						}
						if (star)
							post("Item starred");
						else 
							post("Item unstarred");
					} else
					if (event.location.startsWith("delete://")) {
						event.doit= false;											
						int id = extractInteger(event.location);
						try {
							ArticleInfo article;
							synchronized(cache) {
								article = cache.get(id);
							}
							
							if (article == null) return;
							
							dataManager.removeArticle(article);
							selector.feedUpdated(article.getFeed(),false);
						} catch (DataException e) {
							e.printStackTrace();
						}
						post("Item deleted");
					}
				}

			}			
		});
		
	}
	
	public void refresh() {
		if (currentView != null) {
			if (currentView instanceof Virtual)
				virtualSelected((Virtual) currentView, true);
			else if (currentView instanceof Feed) {
				feedSelected((Feed) currentView);
			}
		}
	}
	public void setLimit(int limit) {
		this.limit = limit;
	}
	public Container currentView() {
		return currentView;
	}

	public void displayResults(Virtual search) {
		load(dataManager.getArticles(search));
	}

	public void channelSelected(Channel channel) {
		// not yet designed		
	}
	
	public void feedSelected(Feed feed) {
		currentView = feed;
		load(dataManager.getArticles(VirtualFactory.limitVirtual(VirtualFactory.createVirtualFromFeed(feed, false), limit)));		
	}

	public void outboxSelected(Feed feed) {
		feedSelected(feed);		
	}

	public void virtualSelected(Virtual virtual, boolean keepSelection) {
		currentView = virtual;
		load(dataManager.getArticles(VirtualFactory.limitVirtual(virtual, limit)));		
	}

	public void cancelSearch() {
		if (currentView instanceof Virtual) {
			virtualSelected((Virtual) currentView, true);
		} else if (currentView instanceof Feed) {
			feedSelected((Feed) currentView);
		} else if (currentView instanceof Channel) {
			channelSelected((Channel) currentView);			
		}
	}
	public void stopDisplayOperation() {
		if (worker != null) {
			worker = null;
		}		
	}
	private final String googleMapsKey = "http://maps.google.com/?q=loc%3A";
	private GeoPoint heuristics(ArticleInfo current) {
		String url = current.getURL();
		
		if (url == null) return GeoPoint.INFINITY;
		
        Parser parser;
        NodeFilter filter;
        NodeList list;
        
        filter = new NodeClassFilter (LinkTag.class);
        
        GeoPoint point = null;
        Trace.printVerbose("Parsing " + current.getTitle());
        try
        {
            parser = new Parser (url);
            list = parser.extractAllNodesThatMatch (filter);
            for (int i = 0; i < list.size (); i++) {
                String link = ((LinkTag) list.elementAt (i)).extractLink();
        		if (link.startsWith(googleMapsKey)) {
        			link = link.substring(googleMapsKey.length()).replace("+", " ");        			        								
        			try {
						point = GeoCoder.getLocation(link);
					} catch (WebConnectionException e1) {
						// ignore
					}
					
					break;
        		}
            }
        }
        catch (ParserException e)
        {
        	// ignore
        }
        
        if (point == null) point = GeoPoint.INFINITY;
        
        Trace.printVerbose(point.toString());
        
        return point;		
	}
	
	private String formatHTML(ArticleInfo article, int id) {
		String summary = null;
		try {
			StringBean sb = new StringBean ();
			sb.setLinks (false);
			Parser parser;
			parser = new Parser ();
			parser.setInputHTML(dataManager.getContent(article).getSummary());
			parser.visitAllNodesWith (sb);
			summary = sb.getStrings ();
		} catch (ParserException e) {
			Trace.printException(e);
		}				
		
		if (summary == null) summary = "";		
		
		StringBuffer info = new StringBuffer("<div style=\"width: 14em;\"><a style=\"margin-right: 6px; font-size: 11pt; font-weight: bold;\" href=\"follow://");
		info.append(article.getURL()).append("\">").append(article.getTitle()).append("</a><br>");
		info.append("<p style=\"font: 8pt Tahoma\">").append(StringUtil.truncate(summary, 300) + "...").append("</p>");
		info.append("<a style=\"font: 8pt Tahoma\" href=\"javascript: map.setCenter(new GLatLng(").append(article.getPoint().getLatitude()).append(",").append(article.getPoint().getLongitude()).append("), 10);\">").append("Zoom").append("</a> / ");
		info.append("<a style=\"font: 8pt Tahoma\" href=\"javascript: map.setCenter(new GLatLng(40,0),1);\">").append("Unzoom").append("</a> / ");
		info.append("<a style=\"font: 8pt Tahoma\" href=\"delete://").append(id).append("\">").append("Delete").append("</a> / ");
		info.append("<a style=\"font: 8pt Tahoma\" href=\"dostar://").append(id).append("\">").append("Star").append("</a> / ");
		info.append("<a style=\"font: 8pt Tahoma\" href=\"unstar://").append(id).append("\">").append("Unstar").append("</a>").append("</div>");
		return info.toString();
	}
	
	private void post(final String line) {
		if ((this == null) || (getDisplay() == null)) return;
		getDisplay().asyncExec(new Runnable() {public void run() {
			if (!isDisposed()) status.setText(line);
		}});
	}
	
	private int extractInteger(String s) {
		if (s == null) return 0;
		
		StringBuffer sb = new StringBuffer();
		char[] bytes = s.toCharArray();
		for(char c : bytes) {
			if (Character.isDigit(c)) {
				sb.append(c);
			}
		}
		
		return (Integer.parseInt(sb.toString()));
	}
	
	/**
	 * Creates the worker thread.
	 * @param iterator
	 * @param zoom
	 */
	private void load(final Iterator<ArticleInfo> iterator) {
		if (iterator == null)
			return;			

		stopDisplayOperation();

		worker = new Thread() {public void run () {
			getDisplay().syncExec(new Runnable() {public void run() {
			mapPane.clearMarkers();
			}});			
			
			synchronized(cache) {
				cache.clear();
			}

			while((worker == this) && iterator.hasNext()) {
				final ArticleInfo current = iterator.next();
				
				GeoPoint geo = current.getPoint();
				if (geo == null) {
					geo = heuristics(current);
						
					try {
						if (worker == this)
							dataManager.setLocation(current, geo);
					} catch (DataException e) {
						// ignore
					}
				}			

				if (!geo.equals(GeoPoint.INFINITY)) {
					synchronized(cache) {
						if (worker == this) {
							int id = cache.size();
							cache.add(current);
							mapPane.addMarker(Double.toString(geo.getLatitude()), Double.toString(geo.getLongitude()), formatHTML(current,id));
						}
					}
				}
			};				
		}};
		
		worker.start();
	}
	
	
}
