package view.components;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import search.SearchObserver;
import settings.MailPaneAppearance;
import util.ArticleCollator;
import util.IconFactory;
import util.SortUtils;
import util.IconFactory.ICON;
import view.dnd.ArticleTransfer;
import controller.Controller;
import data.ArticleContent;
import data.ArticleInfo;
import data.Channel;
import data.Container;
import data.DNDArticle;
import data.DataException;
import data.DataManager;
import data.Feed;
import data.Virtual;
import data.VirtualFactory;

/**
 * A form with article list and a browser component separated with a splitter.  
 * @author oleg
 */
public class RSSMailPane extends SashForm implements ContainerSelectionObserver, SearchObserver {
	private final Browser browser;
	private final Table table;
	private final DataManager dataManager;
	private final ContainerSelector folderPane;
	private final Image starImage;
	private final Image unreadImage;
	private final Controller controller;
	private final Map<Integer, int[]> savedWeights;
	
	private Container currentContainer = null;

	private MailPaneAppearance appearance;

	private final Font normalTableFont;
	private final Font boldTableFont;

	private final MenuItem articleCopyItem; 
	private final MenuItem articleUnreadItem;
	private final MenuItem articleDeleteItem;
	private final MenuItem articleStarItem;
	private final MenuItem articleUnStarItem;

	private final Map<Integer, TableColumn> columns = new HashMap<Integer, TableColumn>();
	private final Map<TableColumn, Integer> columnTypes = new HashMap<TableColumn, Integer>();

	// memory cache for the virtual table
	private List<ArticleInfo> shortTermMemory = new ArrayList<ArticleInfo>();
	private SimpleDateFormat tableDateFormatter = new SimpleDateFormat("MM/dd/yy hh:mm");

	private int feedColumnWidth;
	private final int starColumnWidth;

	public MailPaneAppearance getAppearance() {
		return appearance;
	}

	public void setAppearance(MailPaneAppearance appearance) {
		this.appearance = appearance;
		forceBrowserRefresh();
	}
	
	/**
	 * Retrieves column width data.
	 * @return a map from <code>ArticleCollator.SORT_*</code> constants to pixel sizes. No width is specified for <code>SORT_TITLE</code> as it is calculated automatically. 
	 */
	public Map<Integer, Integer> getColumnWidthData() {
		Map<Integer, Integer> sizes = new HashMap<Integer, Integer>();
		sizes.put(ArticleCollator.SORT_FEED, currentContainer instanceof Virtual? columns.get(ArticleCollator.SORT_FEED).getWidth() : feedColumnWidth);
		sizes.put(ArticleCollator.SORT_STAR, starColumnWidth);
		sizes.put(ArticleCollator.SORT_READ, columns.get(ArticleCollator.SORT_READ).getWidth());
		sizes.put(ArticleCollator.SORT_DATE, columns.get(ArticleCollator.SORT_DATE).getWidth());
		return sizes;
	}

//	public void setColumnWidthData(Map<Integer, Integer> sizes) {
//		Trace.printVerbose("SORT_STAR " + sizes.get(ArticleCollator.SORT_STAR));
//		Trace.printVerbose("SORT_DATE " + sizes.get(ArticleCollator.SORT_DATE));
//		Trace.printVerbose("SORT_READ " + sizes.get(ArticleCollator.SORT_READ));
//		Trace.printVerbose("SORT_FEED " + sizes.get(ArticleCollator.SORT_FEED));
//		
//		feedColumnWidth = sizes.get(ArticleCollator.SORT_FEED);
//		if (currentContainer instanceof Virtual) columns.get(ArticleCollator.SORT_FEED).setWidth(feedColumnWidth);
//		starColumnWidth = sizes.get(ArticleCollator.SORT_STAR);
//		if (! (currentContainer instanceof Virtual) || currentContainer.getName().equals("Starred")) {
//			columns.get(ArticleCollator.SORT_STAR).setWidth(starColumnWidth);
//		}
//		columns.get(ArticleCollator.SORT_READ).setWidth(sizes.get(ArticleCollator.SORT_READ));
//		columns.get(ArticleCollator.SORT_DATE).setWidth(sizes.get(ArticleCollator.SORT_DATE));
//	}

	/**
	 * Returns current sort criterion (one of the <code>ArticleCollator.SORT_*</code> constants).
	 * @return
	 */
	public int getSortCriterion() {
		return columnTypes.get(table.getSortColumn());
	}

	/**
	 * Returns current sort direction (<code>SWT.UP</code> or <code>SWT.DOWN</code>). 
	 * @return
	 */
	public int getSortDirection() {
		return table.getSortDirection();
	}

	/**
	 * Returns the article being currently displayed. 
	 * @return article being currently displayed or null if no article is displayed
	 */
	public ArticleInfo getSelectedArticle() {
		int index = table.getSelectionIndex();
		if (index >= 0) return shortTermMemory.get(index);
		else return null;
	}

	/**
	 * Sets the article being displayed and selects it in the table. 
	 * @param info article to select 
	 */
	public void setSelectedArticle(ArticleInfo info) {
		if (info != null) {
			int index = Collections.binarySearch(shortTermMemory, info, ArticleCollator.instance().getComparator(columnTypes.get(table.getSortColumn()), table.getSortDirection()));
			if (index > 0) {
				table.forceFocus();
				table.setSelection(index);
				lazyBrowse(table.getItem(index), info, 10);
			}
		}
	}

	/**
	 * Displays a new article in the list.
	 * @param info the new article
	 */
	public void newArticles(final Collection<ArticleInfo> infos, final Feed feed) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				if (feed.equals(currentContainer)) {
					SortUtils.addToSorted(shortTermMemory, infos, ArticleCollator.instance().getComparator(columnTypes.get(table.getSortColumn()), table.getSortDirection()));
					table.setItemCount(shortTermMemory.size());
					table.clearAll();
				} else if (currentContainer instanceof Virtual && shortTermMemory.contains(infos.iterator().next())) {
					virtualSelected((Virtual) currentContainer,true);
				} else if (feed.getChannel().equals(currentContainer)) {
					browser.setText(generateFolderSummary(feed.getChannel()));
				}
			}
		});
	}
	// formats article to HTML
	private String formatContent(ArticleInfo info, ArticleContent content) {
		StringBuffer bf = new StringBuffer("<html><title></title><style type=\"text/css\">\n");
		bf.append(appearance.getArticleCSS()).append("</style></head>\n");
		bf.append("<body><p class=\"rssheader\"><a class=\"rsslink\" href=\"").append(info.getURL()).append("\">").append(info.getTitle()).append("</a><br/>");
		if (!info.getAuthors().equals("")) bf.append("By ").append(info.getAuthors()).append(", ");
		assert info != null;
		assert content != null;
		return bf.append(new Date(info.getPublishedDate())).append("</p>").append(content.getSummary()).append("</body></html>").toString();
	}

	private final Listener articleCopyListener = new Listener() {
		public void handleEvent(Event event) {
			for (int index: table.getSelectionIndices()) {
				ArticleInfo info =  shortTermMemory.get(index);
				ArticleInfo newInfo = new ArticleInfo(info.getTitle(), dataManager.getFeed("rss-reader://outbox"),info.getPublishedDate(), info.getAuthors(), info.getURL(), info.isRead(), info.isStarred(), info.getPoint());
				try {
					dataManager.addArticle(newInfo, dataManager.getContent(info));
				} catch (DataException e) {

				}
			}
		}
	};
	private final Listener articleUnreadListener = new Listener() {
		public void handleEvent(Event event) {
			for (int index: table.getSelectionIndices()) {
				ArticleInfo info = shortTermMemory.get(index);
				if (info.isRead()) {
					try {
						dataManager.setReadArticle(info, false);
						TableItem item = table.getItem(index);
						item.setFont(boldTableFont);
						item.setImage(2, unreadImage);
						folderPane.feedUpdated(info.getFeed(),false);
					} catch (DataException e) {

						e.printStackTrace();
					}
				}
			}
		}
	};

	private void deleteSelectedArticles() {
		int[] si = table.getSelectionIndices();
		if (si.length > 0) {
			ArticleInfo[] infos = new ArticleInfo[si.length];
			for (int i=0; i< si.length; i++) {
				infos[i] = shortTermMemory.get(si[i]);
				try {
					dataManager.removeArticle(infos[i]);
					folderPane.feedUpdated(infos[i].getFeed(),false);
				} catch (DataException e) {
					controller.reportError("Deleting an article caused an error:\n" + e.getMessage());
				}
			}
			SortUtils.removeAll(shortTermMemory, si);
			table.setItemCount(shortTermMemory.size());
			table.clearAll();
			if (shortTermMemory.isEmpty()) browser.setText("");
			else {
				int newselected = si[0] > shortTermMemory.size() - 1 ? shortTermMemory.size() - 1 : si[0];
				table.setSelection(newselected);
				lazyBrowse(table.getItem(newselected), shortTermMemory.get(newselected), 10L);
			}
		}
	}

	private final Listener articleDeleteListener = new Listener() {
		public void handleEvent(Event event) {
			deleteSelectedArticles();
		}
	};
	private final Listener articleStarListener = new Listener() {
		public void handleEvent(Event event) {
			for (int i: table.getSelectionIndices()) {
				try {
					dataManager.setStarredArticle(shortTermMemory.get(i), true);
					table.getItem(i).setImage(starImage);
				} catch (DataException e) {
					controller.reportError("Starring an article caused an error:\n" + e.getMessage());
				}
			}
		}
	};
	private final Listener articleUnStarListener = new Listener() {
		public void handleEvent(Event event) {
			for (int i: table.getSelectionIndices()) {
				try {
					dataManager.setStarredArticle(shortTermMemory.get(i), false);
					table.getItem(i).setImage((Image)null);
				} catch (DataException e) {
					controller.reportError("Unstarring an article caused an error:\n" + e.getMessage());
				}
			}
		}
	};

	// adjusts columns' sizes
	private void adjustColumns() {
		int width = table.getSize().x - table.getBorderWidth()*2;
		ScrollBar vb = table.getVerticalBar();
		// This is black magic
		if (vb != null && vb.isVisible() && vb.getThumb() < vb.getMaximum()) {
			width -= vb.getSize().x;
		}
		int total = columns.get(ArticleCollator.SORT_STAR).getWidth()
			+ columns.get(ArticleCollator.SORT_DATE).getWidth()
			+ columns.get(ArticleCollator.SORT_READ).getWidth()
			+ columns.get(ArticleCollator.SORT_FEED).getWidth();
		int newWidth =  width - total;
		if (newWidth > 0) columns.get(ArticleCollator.SORT_TITLE).setWidth(newWidth); 
	}

	/**
	 * Creates a new pane.
	 * @param parent parent pane
	 * @param style SWT style
	 * @param dataManager data manager to use
	 * @param folderPane container selector 
	 * @param defaultAppearance font and color settings for the browser component
	 * @param criterion sort criterion (one of the <code>ArticleCollator.SORT_*</code> constants
	 * @param direction sort direction (<code>SWT.UP</code> or <code>SWT.DOWN</code>)
	 * @param columnWidthData column sizes
	 * @param orientation the orientation of mail pane as a sash form (<code>SWT.VERTICAL</code> or <code>SWT.HORIZONTAL</code>)
	 * @param weights preferred sash form weights for both orientations 
	 */
	public RSSMailPane(Composite parent, 
			final Controller controller, 
			final ContainerSelector folderPane, 
			MailPaneAppearance defaultAppearance, 
			int criterion, int direction, 
			Map<Integer, Integer> columnWidthData, 
			int orientation, Map<Integer, int[]> weights) {
		super(parent,orientation);	
		this.dataManager = controller.getDataManager();
		this.controller = controller;
		this.appearance = defaultAppearance;
		this.savedWeights = weights;
		table = new Table(this,SWT.FULL_SELECTION | SWT.MULTI | SWT.VIRTUAL);

		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		starImage = IconFactory.getIcon(getDisplay(), ICON.STAR);
		unreadImage = IconFactory.getIcon(getDisplay(), ICON.UNREAD);

		feedColumnWidth = columnWidthData.get(ArticleCollator.SORT_FEED);
		starColumnWidth = columnWidthData.get(ArticleCollator.SORT_STAR);

		TableColumn column = new TableColumn(table,SWT.NONE);
		column.setResizable(false);
		columns.put(ArticleCollator.SORT_STAR, column);
		column.setWidth(columnWidthData.get(ArticleCollator.SORT_STAR));
		columnTypes.put(column, ArticleCollator.SORT_STAR);

		column = new TableColumn(table,SWT.NONE);
		column.setText("Date");
		columns.put(ArticleCollator.SORT_DATE, column);
		column.setWidth(columnWidthData.get(ArticleCollator.SORT_DATE));
		columnTypes.put(column, ArticleCollator.SORT_DATE);

		column = new TableColumn(table, SWT.NONE);
		column.setResizable(false);
		columns.put(ArticleCollator.SORT_READ, column);
		column.setWidth(columnWidthData.get(ArticleCollator.SORT_READ));
		columnTypes.put(column, ArticleCollator.SORT_READ);

		column = new TableColumn(table, SWT.NONE);
		column.setText("Feed");
		columns.put(ArticleCollator.SORT_FEED, column);
		column.setWidth(columnWidthData.get(ArticleCollator.SORT_FEED));
		columnTypes.put(column, ArticleCollator.SORT_FEED);

		column = new TableColumn(table, SWT.NONE);
		column.setText("Title");
		columns.put(ArticleCollator.SORT_TITLE, column);
		columnTypes.put(column, ArticleCollator.SORT_TITLE);
		column.setWidth(100);

		
		if (!SWT.getPlatform().equals("gtk")) {
			Listener adjustListener = new Listener() {
				public void handleEvent(Event event) {
					adjustColumns();
				}
			};

			column.setResizable(false);

			table.addListener(SWT.Resize, adjustListener);
			int count = table.getColumnCount() - 1;
			for (int i = 0; i< count; i++) table.getColumn(i).addListener(SWT.Resize, adjustListener);
		}

		table.setSortColumn(columns.get(criterion));
		table.setSortDirection(direction);

		final Listener columnSortListener = new Listener() {
			public void handleEvent(Event event) {
				int sortDirection;
				TableColumn sortCol = (TableColumn) event.widget;
				if (table.getSortColumn() == sortCol) {
					sortDirection = table.getSortDirection() == SWT.UP? SWT.DOWN : SWT.UP;
					table.setSortDirection(sortDirection);
				} else {
					table.setSortColumn(sortCol);
					sortDirection = SWT.UP;
					table.setSortDirection(sortDirection);
				}
				Collections.sort(shortTermMemory, ArticleCollator.instance().getComparator(columnTypes.get(sortCol), sortDirection));
				table.clearAll();
			}
		};

		for (TableColumn ti: columnTypes.keySet()) ti.addListener(SWT.Selection, columnSortListener);

		

		this.folderPane = folderPane;
		folderPane.addContainerSelectionObserver(this);

		table.addListener(SWT.KeyDown, new Listener() {
			public void handleEvent(Event event) {
				if (event.keyCode == SWT.DEL) deleteSelectedArticles();
			}
		});

		DragSource ds = new DragSource(table,DND.DROP_MOVE);
		ds.setTransfer(new Transfer[] {ArticleTransfer.instance()});
		ds.addDragListener(new DragSourceAdapter() {
			public void dragSetData(DragSourceEvent event) {
				if (currentContainer instanceof Feed && !((Feed) currentContainer).isLocal()) {
					int[] indices = table.getSelectionIndices();
					DNDArticle articles[] = new DNDArticle[indices.length];
					for (int i=0; i<indices.length; i++) articles[i] = DNDArticle.fromArticleInfo(shortTermMemory.get(indices[i]));
					event.data = articles;
				}
			}
		});

		normalTableFont = table.getFont();
		FontData fontData = normalTableFont.getFontData()[0];
		fontData.setStyle(SWT.BOLD);
		boldTableFont = new Font(this.getDisplay(),fontData);

		table.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				TableItem ti = (TableItem) e.item;
				ArticleInfo info = shortTermMemory.get(table.indexOf(ti));
				lazyBrowse(ti, info, 500L);
			}
		});
		table.addListener(SWT.SetData, new Listener() {
			public void handleEvent(Event event) {
				ArticleInfo info = shortTermMemory.get(event.index);
				TableItem ti = (TableItem) event.item; 
				if (info.isStarred()) ti.setImage(0,starImage);
				ti.setText(1, tableDateFormatter.format(new Date(info.getPublishedDate())));
				if (!info.isRead()) ti.setImage(2, unreadImage);
				ti.setText(3, info.getFeed().getName());
				ti.setText(4, info.getTitle());
				if (!info.isRead()) ti.setFont(boldTableFont);
			}
		});


		Menu articleMenu = new Menu(table);
		table.setMenu(articleMenu);
		articleCopyItem = new MenuItem(articleMenu,SWT.CASCADE);
		articleCopyItem.setText("Copy to outbox");
		articleCopyItem.setImage(new Image(this.getDisplay(), IconFactory.getIcon(IconFactory.ICON.FOLDER_OUTBOX)));
		articleCopyItem.addListener(SWT.Selection, articleCopyListener);
		articleUnreadItem = new MenuItem(articleMenu,SWT.CASCADE);
		articleUnreadItem.setText("Mark as unread");
		articleUnreadItem.setImage(new Image(this.getDisplay(), IconFactory.getIcon(IconFactory.ICON.UNDO)));
		articleUnreadItem.addListener(SWT.Selection, articleUnreadListener);
		articleDeleteItem = new MenuItem(articleMenu,SWT.CASCADE);
		articleDeleteItem.setText("Delete");
		articleDeleteItem.setImage(new Image(this.getDisplay(), IconFactory.getIcon(IconFactory.ICON.DELETE)));
		articleDeleteItem.addListener(SWT.Selection, articleDeleteListener);
		articleStarItem = new MenuItem(articleMenu, SWT.CASCADE);
		articleStarItem.setText("Star");
		articleStarItem.setImage(starImage);
		articleStarItem.addListener(SWT.Selection, articleStarListener);
		articleUnStarItem = new MenuItem(articleMenu, SWT.CASCADE);
		articleUnStarItem.setText("Unstar");
		articleUnStarItem.setImage(new Image(getDisplay(),IconFactory.getIcon(IconFactory.ICON.UNSTAR)));
		articleUnStarItem.addListener(SWT.Selection, articleUnStarListener);
		browser = new Browser(this,SWT.NONE);
		browser.addLocationListener(new LocationListener() {

			public void changed(LocationEvent event) {
				// do nothin'
			}

			public void changing(LocationEvent event) {
				if (!event.location.equals("about:blank")) {
					Program.launch(event.location);
					event.doit = false;
				}
			}

		});
		this.setWeights(savedWeights.get(orientation));
	}
	private void showArticles(Iterator<ArticleInfo> iter) {
		if (iter == null)
			return;

		table.removeAll();
		shortTermMemory = new ArrayList<ArticleInfo>(20);
		while (iter.hasNext()) {
			shortTermMemory.add(iter.next());
		}
		Collections.sort(shortTermMemory, ArticleCollator.instance().getComparator(columnTypes.get(table.getSortColumn()), table.getSortDirection()));
		table.setItemCount(shortTermMemory.size());
	}
	/**
	 * Displays the contents of the feed.
	 */
	public void feedSelected(Feed feed) {
		currentContainer = feed;
		articleCopyItem.setEnabled(true);
		showArticles(dataManager.getArticles(feed));
		TableColumn starCol = columns.get(ArticleCollator.SORT_STAR);
		if (starCol.getWidth() == 0) starCol.setWidth(starColumnWidth);
		TableColumn feedCol = columns.get(ArticleCollator.SORT_FEED);
		if (feedCol.getWidth() != 0) {
			feedColumnWidth = feedCol.getWidth();
			feedCol.setWidth(0);
			feedCol.setResizable(false);
		}
		browser.setText("");
		setWeights(savedWeights.get(getOrientation()));
		
	}
	/**
	 * Displays the contents of the outbox.
	 */
	public void outboxSelected(Feed feed) {
		currentContainer = feed;
		articleCopyItem.setEnabled(false);
		showArticles(dataManager.getArticles(feed));
		TableColumn starCol = columns.get(ArticleCollator.SORT_STAR);
		if (starCol.getWidth() == 0) starCol.setWidth(starColumnWidth);
		TableColumn feedCol = columns.get(ArticleCollator.SORT_FEED);
		if (feedCol.getWidth() != 0) {
			feedColumnWidth = feedCol.getWidth();
			feedCol.setWidth(0);
			feedCol.setResizable(false);
		}
		browser.setText("");
		setWeights(savedWeights.get(getOrientation()));
	}
	/**
	 * Displays the contents of the virtual.
	 */
	public void virtualSelected(Virtual virtual, boolean keepSelection) {
		boolean wasChannel = currentContainer instanceof Channel;
		ArticleInfo selected = null;
		if (keepSelection) {
			int selection = table.getSelectionIndex();
			if (selection >= 0) selected = shortTermMemory.get(selection); 
		}
		currentContainer = virtual;
		articleCopyItem.setEnabled(true);
		showArticles(dataManager.getArticles(virtual));
		TableColumn starCol = columns.get(ArticleCollator.SORT_STAR);
		if (virtual.getName().equals("Starred")) {
			//starColumnWidth = starCol.getWidth();
			starCol.setWidth(0);
		} else if (starCol.getWidth() == 0) starCol.setWidth(starColumnWidth);
		TableColumn feedCol = columns.get(ArticleCollator.SORT_FEED);
		if (feedCol.getWidth() == 0) {
			feedCol.setWidth(feedColumnWidth);
			feedCol.setResizable(true);
		}
		browser.setText("");
		if (keepSelection) setSelectedArticle(selected);
		if (wasChannel) setWeights(savedWeights.get(getOrientation()));
	}
	private void forceBrowserRefresh() {
		int selected = table.getSelectionIndex();
		if (selected != -1) {
			ArticleInfo info = shortTermMemory.get(selected);
			browser.setText(formatContent(info, dataManager.getContent(info)));
		}
	}

	// adds a pause before displaying articles
	private void lazyBrowse(final TableItem item, final ArticleInfo info, final long sleep) {
		new Thread() {
			public void run() {
				try {
					Thread.sleep(sleep);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				getDisplay().asyncExec(new Runnable() {
					public void run() {
						if (!isDisposed()) {
							TableItem[] selection = table.getSelection();
							if (selection.length > 0 && selection[0].equals(item)) {
								ArticleContent content = dataManager.getContent(info);
								if (content != null) {
									browser.setText(formatContent(info, content));
									if (!info.isRead()) {
										try {
											dataManager.setReadArticle(info, true);
											item.setFont(normalTableFont);
											item.setImage(2, null);
											folderPane.feedUpdated(info.getFeed(),false);
										} catch (DataException xcp) {
											xcp.printStackTrace();
										}
									}
								}
							}
						}
					}
				});
			}
		}.start();
	}

	public void cancelSearch() {
		if (currentContainer instanceof Feed) {
			Feed feed  = (Feed) currentContainer;
			if (feed.isLocal()) 
				outboxSelected(feed);
			else
				feedSelected(feed);
		} else if (currentContainer instanceof Virtual) {
			virtualSelected((Virtual) currentContainer, true);			
		}
	}

	public void displayResults(Virtual search) {
		articleCopyItem.setEnabled(true);
		if (search != null)
			showArticles(dataManager.getArticles(search));
	}

	public Container currentView() {
		if (currentContainer == null)
			return null;

		return currentContainer;
	}
	
	// generates a div for a feed
	private StringBuffer generateFeedSummary(Feed feed) {
		StringBuffer div= new StringBuffer("<div class=\"feedDiv\">\n");
		div.append("<p>").append(feed.getName()).append("</p>\n");
		int max = 5;
		Iterator<ArticleInfo> iter = dataManager.getArticles(VirtualFactory.limitVirtual(VirtualFactory.orderByDate(VirtualFactory.createVirtualFromFeed(feed, false)), max));
		if (iter != null) {
			while (iter.hasNext()) {
				ArticleInfo info = iter.next();
				div.append("<a href=\"").append(info.getURL()).append("\">");
				div.append(info.getTitle()).append("</a><br />\n");
			}
		}
		div.append("</div>");
		return div;
	}
	// generates a summary page for a folder
	private String generateFolderSummary(Channel channel) {
		Virtual vChannel = VirtualFactory.createVirtualFromChannel(channel);
		int total = dataManager.numberArticles(vChannel);
		int unread = dataManager.unreadArticles(vChannel);
		
		StringBuffer html = new StringBuffer("<html>\n<head>\n<style type=\"text/css\">\n");
		html.append(appearance.getFolderCSS());
		html.append("\n</style>\n</head>\n<body>");
		html.append("\n<div class=\"folderDiv\">");
		html.append(channel.getName()).append(" (").append(total);
		html.append(" total/").append(unread).append(" unread)");
		html.append("</div>");
		for (Feed feed: dataManager.getFeeds()) {
			if (feed.getChannel().equals(channel)) {
				html.append(generateFeedSummary(feed)).append("\n");
			}
		}
		html.append("</body>\n</html>");
		return html.toString();
	}
	
	public void channelSelected(Channel channel) {
		// clear everything 
		currentContainer = channel;
		
		setWeights(new int[] {0,1});
		browser.setText(generateFolderSummary(channel));
		shortTermMemory = new ArrayList<ArticleInfo>();
		table.setItemCount(0);
		table.clearAll();
		
	}
	
	/**
	 * Sets the orientation of the pane. Use this rather than the inherited <code>setOrientation</code>.
	 * @param orientation new orientation
	 */
	public void setMailPaneOrientation(int orientation) {
		if (orientation != getOrientation()) {
			boolean isContainer = currentContainer instanceof Channel;
			if (!isContainer) savedWeights.put(getOrientation(), getWeights());
			setOrientation(orientation);
			if (!isContainer) setWeights(savedWeights.get(orientation)); 
		}
	}
	/**
	 * Retrieves preferred weights for both orientations.
	 * @return a map orientation -> weights 
	 */
	public Map<Integer, int[]> getMailPaneWeights() {
		if (!(currentContainer instanceof Channel)) {
			savedWeights.put(getOrientation(), getWeights());
		}
		return savedWeights;
	}
}
