package view.dialogs;

import java.io.IOException;
import java.util.Collection;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import parser.OPMLParser;
import util.Pair;
import controller.Controller;
import data.Channel;
import data.DataException;

public class OPMLImportDialog {
	private final Text filename;
	private final Spinner syncSpinner;
	private final Shell shell;
	private final Controller controller;

	/**
	 * Creates a new dialog with the given parent shell and controller for OPML import.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public OPMLImportDialog(final Shell parent, final Controller controller) {
		this.controller = controller;
		shell = new Shell(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("OPML Import");
		shell.setLayout(new GridLayout(1,false));
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));

		input.setLayout(new GridLayout(4,false));

		Label label = new Label(input,SWT.NONE);
		label.setText("File name");

		filename = new Text(input, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 200;
		gd.horizontalSpan = 2;
		filename.setLayoutData(gd);

		Button button = new Button(input, SWT.PUSH);
		button.setText("...");
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				FileDialog fd = new FileDialog(parent, SWT.OPEN);
				fd.setText("Open");
		        String[] filterExt = { "*.opml", "*.*" };
		        fd.setFilterExtensions(filterExt);
		        shell.setEnabled(false);
				String name = fd.open();
				shell.setEnabled(true);
				if (name != null) 
					filename.setText(name);
			}
		});
				
		label = new Label(input,SWT.NONE);
		label.setText("Update every");

		syncSpinner = new Spinner(input,SWT.BORDER);
		syncSpinner.setMaximum(60);
		syncSpinner.setMinimum(1);
		syncSpinner.setSelection(5);

		label = new Label(input,SWT.NONE);
		label.setText("minutes");
		label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		buttons.setLayout(new GridLayout(2, false));
		button = new Button(buttons, SWT.NONE);
		button.setText("Import");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				OPMLImport();
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});

		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}
	}
	
	/**
	 * Imports feeds from OPML file.
	 *
	 */
	private void OPMLImport() {
		new Thread() {
			private final String fileName = filename.getText();
			private long updateInterval = 60000L*syncSpinner.getSelection();
			
			public void run() {
				Map<String, Collection<Pair<String, String>>> tree = null; 
				try {
					tree = OPMLParser.readOPML(fileName, 1,
							updateInterval, null);
				} catch (IOException e){ 
					e.printStackTrace();
					controller.reportError("Could not import file due to I/O failure");
					return;
				}
				
				if (tree != null)  {
					for (String name : tree.keySet()) {
						Channel channel = null;
						String channelName;
						if ((name == null) || (name.equals(""))) {
							channelName = fileName;
						} else
							channelName = name;

						channel = controller.getDataManager().getChannel(channelName);
						if (channel == null)  {
							try {
								channel = controller.getDataManager().addChannel(channelName);
							} catch(DataException e) {
								// stop trying to put into this channel
								continue;
							}
						}														

						if (channel != null) {
							for (Pair<String, String> pair : tree.get(name)) {
								if (controller.getDataManager().getFeed(pair.cdr) == null)
									controller.getDownloadManager().newSubscription(pair.cdr, pair.car, channel, updateInterval);
							}
						}
					}
				}				
			}			
		}.start();		
		shell.close();
	}
}
