package view.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import controller.Controller;
import data.Channel;

/**
 * A dialog for subscribing to a new feed.
 * @author oleg
 */
public class SubscribeDialog {
	private final Text feedName;
	private final Text url;
	private final Combo parentChannel;
	private final Spinner updateInterval;
	private final Shell shell;
	/**
	 * Creates a new dialog with the given parent shell and controller.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public SubscribeDialog(Shell parent, Controller controller) {
		this(parent, controller,"",null,"");
	}
	/**
	 * Creates a new dialog with the given parent shell and controller. Suggests name, url, and channel.
	 * @param parent parent shell
	 * @param controller controller
	 * @param suggestedUrl suggested url (may be empty)
	 * @param suggestedChannel suggested url (may be null)
	 * @param suggestedTitle suggested name (may be empty)
	 */
	public SubscribeDialog(Shell parent, final Controller controller, String suggestedUrl, Channel suggestedChannel, String suggestedTitle) {
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("New subscription");
		shell.setLayout(new GridLayout(1, false));
	
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		input.setLayout(new GridLayout(3,false));
		
		Label label = new Label(input, SWT.NONE);
		label.setText("Feed name");
		
		feedName = new Text(input, SWT.SINGLE | SWT.BORDER);
		feedName.setText(suggestedTitle);
		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gd.horizontalSpan = 2;
		gd.widthHint = 300;
		feedName.setLayoutData(gd);
		
		label = new Label(input, SWT.NONE);
		label.setText("Feed URL");
		
		url = new Text(input, SWT.SINGLE | SWT.BORDER);
		gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gd.horizontalSpan = 2;
		url.setLayoutData(gd);
		url.setText(suggestedUrl);
		
		label = new Label(input, SWT.NONE);
		label.setText("Folder");
		
		parentChannel = new Combo(input, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
		gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gd.horizontalSpan = 2;
		parentChannel.setLayoutData(gd);
		for (Channel channel: controller.getDataManager().getChannels()) {
			if (!channel.getName().equals("")) parentChannel.add(channel.getName());
		} 
		if (parentChannel.getItemCount() > 0) parentChannel.setText(parentChannel.getItem(0));
		if (suggestedChannel != null) parentChannel.setText(suggestedChannel.getName());
		
		label = new Label(input, SWT.NONE);
		label.setText("Update every");
		
		updateInterval = new Spinner(input, SWT.BORDER);
		updateInterval.setMaximum(60);
		updateInterval.setMinimum(1);
		updateInterval.setSelection(5);

		
		label = new Label(input, SWT.NONE);
		label.setText("minutes");
	
		gd = new GridData(GridData.FILL_HORIZONTAL);
		label.setLayoutData(gd);
		
		Composite buttons = new Composite(shell,SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		buttons.setLayout(new GridLayout(2, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Subscribe");
		
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (!parentChannel.getText().equals("")) {
					final String feedURL = url.getText();
					final String feedTitle = feedName.getText();
					final String channelName = parentChannel.getText();
					final long syncInterval = 60000L*updateInterval.getSelection(); 
					new Thread() {
						public void run() {
							controller.getDownloadManager().newSubscription(feedURL, feedTitle, controller.getDataManager().getChannel(channelName), syncInterval);
						}
					}.start();
					shell.close();
				}
			}
		});
		
		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");
		
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});
		
		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}
	}
	/**
	 * Checks if the dialog shell is disposed.
	 */
	public boolean isDisposed() {
		return shell.isDisposed();
	}
}
