package view.dialogs;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import util.IconFactory;
import util.ReflectionUtils;
import util.IconFactory.ICON;
import webapi.OnLoadListener;
import webapi.WebConnectionException;
import webapi.search.BlogDiggerSearch;
import webapi.search.FeedsterSearch;
import webapi.search.GoogleMapsSearchView;
import webapi.search.Syndic8Search;
import webapi.search.TableSearchView;
import webapi.search.TechnoratiSearch;
import webapi.search.WebSearchProvider;
import webapi.search.WebSearchView;
import controller.Controller;
/**
 * A dialog for searching the Web for new feeds. Allows pluggable search engines and search result views.
 * @author oleg
 */
public class WebSearchDialog implements OnLoadListener {
	private final Button doSearch;
	private final Shell shell;
	private final Controller controller;
	private final Combo providersCombo;
	private final Combo viewCombo;
	private final Map<String, WebSearchProvider> providers = new HashMap<String, WebSearchProvider>();
	private final Map<String, Image> providerImages = new HashMap<String, Image>();
	private final Map<String, Class> searchViews = new HashMap<String, Class>();
	private WebSearchView searchView = null;
	
	public void onLoad() {
		doSearch.setEnabled(true);
	}

	private void applyProviderSelection() {
		doSearch.setImage(providerImages.get(providersCombo.getText()));
	}
	
	private void applySearchViewSelection() {
		final GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 5;
		if (searchView != null) searchView.dispose();		
		try {
			searchView = (WebSearchView) searchViews.get(viewCombo.getText()).getConstructor(new Class[] {Composite.class, Controller.class}).newInstance(new Object[]{shell, controller});
			searchView.setLayoutData(gd);
		} catch (IllegalArgumentException e1) {
		} catch (SecurityException e1) {
		} catch (InstantiationException e1) {
		} catch (IllegalAccessException e1) {
		} catch (InvocationTargetException e1) {
		} catch (NoSuchMethodException e1) {
		}
		shell.layout();
	}

	private void addSearchProvider(WebSearchProvider wp) {
		String name = wp.getName();
		providersCombo.add(name);
		providers.put(name, wp);
		providerImages.put(name, new Image(providersCombo.getDisplay(), wp.getIcon()));
	}
	private void addSearchView(String name, Class newSearchView) {
		if (ReflectionUtils.isSuperClass(WebSearchView.class, newSearchView)) { 
			viewCombo.add(name);
			searchViews.put(name, newSearchView);
		}
	}
	/**
	 * Creates a new dialog with the given parent shell and controller.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public WebSearchDialog(final Shell parent, final Controller ctrl) {
		this.controller = ctrl;

		shell = new Shell(parent, SWT.APPLICATION_MODAL | SWT.RESIZE | SWT.DIALOG_TRIM);		
		shell.setLayout(new GridLayout(1, false));
		shell.setText("Find new feeds");

		shell.setImage(IconFactory.getIcon(shell.getDisplay(), ICON.WEB_SEARCH));
		
		shell.addListener(SWT.Close, new Listener() {
			public void handleEvent(Event event) {
				controller.saveSettings(WebSearchDialog.class.getName(), shell.getSize());
			}
		});

		Composite queryRow = new Composite(shell,SWT.NONE);
		queryRow.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		queryRow.setLayout(new GridLayout(4,false));

		Label label = new Label(queryRow, SWT.NONE);
		label.setText("Keyword");

		final Text searchBar = new Text(queryRow, SWT.SINGLE | SWT.BORDER);
		searchBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		label = new Label(queryRow, SWT.NONE);
		label.setText("limit");

		final Spinner limit = new Spinner(queryRow, SWT.NONE);
		limit.setMinimum(1);
		limit.setSelection(10);
		limit.setMaximum(100);

		final Composite searchRow = new Composite(shell, SWT.NONE);
		searchRow.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		searchRow.setLayout(new GridLayout(6,false));

		label = new Label(searchRow, SWT.NONE);
		label.setText("Search provider");

		providersCombo = new Combo(searchRow, SWT.BORDER | SWT.DROP_DOWN | SWT.READ_ONLY);
		providersCombo.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				applyProviderSelection();
			}
		});


		label = new Label(searchRow, SWT.NONE);
		label.setText("Display as a");

		viewCombo =  new Combo(searchRow, SWT.BORDER | SWT.DROP_DOWN | SWT.READ_ONLY);		

		final Button suggest = new Button(searchRow, SWT.PUSH);
		final Menu menu = new Menu(shell, SWT.POP_UP);
		new Thread() { public void run() {
			Collection<String> terms = controller.getDataManager().getMostFrequentWords(10);
			for (final String term : terms) {
				shell.getDisplay().asyncExec(new Runnable() {public void run() {
					MenuItem item = new MenuItem (menu, SWT.PUSH);
					item.setText (term);
					item.addListener(SWT.Selection, new Listener() {
						public void handleEvent(Event e) {
							searchBar.append(term + " ");
						}
					});
				}});
			}	
		}}.start();
		suggest.setImage(IconFactory.getIcon(parent.getDisplay(), IconFactory.ICON.LIGHT_BULB));
		suggest.setToolTipText("Suggest a keyword");
		suggest.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				Rectangle rect = suggest.getBounds ();
				Point pt = new Point (rect.x, rect.y + rect.height);
				pt = searchRow.toDisplay (pt);
				menu.setLocation (pt);
				menu.setVisible (true);
				menu.setVisible(true);
			}
		});
		
		doSearch = new Button(searchRow, SWT.NONE);
		doSearch.setText("Search");
		GridData searchButtonData = new GridData();
		searchButtonData.widthHint = 130;
		searchButtonData.horizontalAlignment = SWT.END;
		searchButtonData.grabExcessHorizontalSpace = true;
		doSearch.setLayoutData(searchButtonData);
		
		addSearchProvider(new FeedsterSearch());
		addSearchProvider(new BlogDiggerSearch());
		addSearchProvider(new TechnoratiSearch());
		addSearchProvider(new Syndic8Search());

		addSearchView("Map",GoogleMapsSearchView.class);
		addSearchView("List",TableSearchView.class);

		if (viewCombo.getItemCount() == 0) {
			controller.reportError("No search views are found");
		} else if (providersCombo.getItemCount() == 0) {
			controller.reportError("No search providers are found");
		} else {
			viewCombo.select(0);
			applySearchViewSelection();
			providersCombo.select(0);
			applyProviderSelection();

			viewCombo.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					applySearchViewSelection();
				}
			});


			doSearch.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					doSearch.setEnabled(false);
					final String keyword = searchBar.getText();
					final int maxresult = limit.getSelection(); 
					final WebSearchProvider provider = providers.get(providersCombo.getText());
					if (provider != null) {
						new Thread() {
							public void run() {
								try {
									searchView.show(provider.searchWeb(keyword, maxresult));
								} catch (WebConnectionException e) {
									e.printStackTrace();
								} finally {
									if (!shell.isDisposed()) {
										shell.getDisplay().asyncExec(new Runnable() {
											public void run() {
												if (!doSearch.isDisposed()) doSearch.setEnabled(true);
											}
										});
									}
								}
							}
						}.start();
					}
				}
			});

			Point size = (Point) controller.getSettings(this.getClass().getName());
			if (size == null) size = new Point(500,400);
			shell.setSize(size);
			//shell.pack();
			shell.open();

			while (!shell.isDisposed())
				if (!shell.getDisplay().readAndDispatch())
					shell.getDisplay().sleep();
		}
	}

}
