package webapi;

import java.io.IOException;
import java.util.Random;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.browser.TitleEvent;
import org.eclipse.swt.browser.TitleListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Scale;

import util.FileUtil;


/**
 * A composite with embedded Google Maps and zoom control.
 * @author oleg
 */
public class GoogleMapsWidget extends Composite {
	/**
	 * Zoom control types.
	 */
	public enum ZoomControl {
		/**
		 * SWT scale widget.
		 */
		SWT_SCALE,
		/**
		 * Google Maps large map control.
		 */
		HTML_LARGE,
		/**
		 * Google Maps small map control.
		 */
		HTML_SMALL
	}
	private static final int PAGE_BORDER_OFFSET = 50; 
	private final Random random = new Random(System.currentTimeMillis());
	private final Browser browser;
	/**
	 * Adds a location change listener to the browser.
	 * @param l location listener.
	 */
	public void addLocationListener(LocationListener l) {
		browser.addLocationListener(l);
	}
	private void adjustSize() {
		browser.execute("document.getElementById(\"map\").style.width=\"" + String.valueOf(browser.getSize().x - PAGE_BORDER_OFFSET) + "px\";");
		browser.execute("document.getElementById(\"map\").style.height=\"" + String.valueOf(browser.getSize().y - PAGE_BORDER_OFFSET) + "px\";");
	}
	/**
	 * Adds the map type control to the map page.
	 */
	public void addMapTypeControl() {
		browser.execute("map.addControl(new GMapTypeControl());");
	}
	/**
	 * Adds a marker to the map through an asynchronous call.
	 * @param latitude latitude
	 * @param longitude longitude
	 * @param infoHTML the contents of the info balloon
	 * @param shake whether to add small random numbers to the coordinates
	 */
	public void addMarker(String latitude, String longitude, String infoHTML, boolean shake) {
		final StringBuffer javascript = new StringBuffer("map.addOverlay(createMarker(new GLatLng(");
		String.valueOf((Double.parseDouble(latitude) + random.nextDouble()*0.1));
		javascript.append(shake? String.valueOf((Double.parseDouble(latitude) + random.nextDouble()*0.01)) : latitude).append(",").append(shake? String.valueOf((Double.parseDouble(longitude) + random.nextDouble()*0.01)) : longitude).append("),'").append(infoHTML).append("'));");
		if (!browser.isDisposed()) {
			browser.getDisplay().asyncExec(new Runnable() {
				public void run() {
					if (!browser.isDisposed()) browser.execute(javascript.toString());
				}
			});
		}
	}
	
	/**
	 * Adds a marker to the map through a synchronous call.
	 * @param latitude latitude	
	 * @param longitude longitude
	 * @param html the content of the balloon
	 */
	public void addMarker(String latitude, String longitude, String html) {
		final StringBuffer javascript = new StringBuffer("map.addOverlay(createMarker(new GLatLng(").append(latitude).append(",").append(longitude).append("),'").append(html).append("'));");
		if (!browser.isDisposed()) {
			browser.getDisplay().syncExec(new Runnable() {
				public void run() {
					if (!browser.isDisposed()) browser.execute(javascript.toString());
				}
			});
		}
	}

	/**
	 * Displays marker at each given address location and shows html page
	 * for each marker. 
	 * @param address desired location
	 * @param infoHTML html to be displayed
	 * @author Kuat
	 */
	public void showAddress(String address, String infoHTML) {
		final StringBuffer javascript = new StringBuffer("showAddress(\"").append(address).append("\",'").append(infoHTML).append("');");

		if (!browser.isDisposed()) {
			browser.getDisplay().asyncExec(new Runnable() {
				public void run() {
					browser.execute(javascript.toString());
				}
			});
		}		
	}

	/**
	 * Zooms the map to locate the specified address. 
	 * @param address desired location
	 * @author Kuat
	 */
	public void zoomAddress(String address) {
		final StringBuffer javascript = new StringBuffer("zoomAddress(\"").append(address).append("\");");

		if (!browser.isDisposed()) {
			browser.getDisplay().asyncExec(new Runnable() {
				public void run() {
					browser.execute(javascript.toString());
				}
			});
		}		
	}

	/**
	 * Removes all markers from the map.
	 */
	public void clearMarkers() {
		if ((browser != null) && !browser.isDisposed())
			browser.execute("map.clearOverlays()");
	}
	/**
	 * Creates a new Google maps pane on a parent pane.
	 * @param parent parent pane
	 * @param zoomControl a zoom control to display
	 * @param style SWT style
	 */
	public GoogleMapsWidget(Composite parent, int style, final ZoomControl zoomControl, final OnLoadListener ll) {
		super(parent, style);
		setLayout(new GridLayout(zoomControl == ZoomControl.SWT_SCALE? 2 : 1,false));

////		JavaScript input for debugging

//		final Text js = new Text(this, SWT.SINGLE);
//		js.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

//		Button exec = new Button(this, SWT.NONE);
//		exec.setText("exec");
//		exec.addListener(SWT.Selection, new Listener() {

//		public void handleEvent(Event event) {
//		browser.execute(js.getText());

//		}

//		});



		browser = new Browser(this, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
//		gd.widthHint = 520;
//		gd.heightHint = 320;
		browser.setLayoutData(gd);

		//if (ll != null)  {
		browser.addTitleListener(new TitleListener() {
			public void changed(TitleEvent event) {
				if (event.title.equals("loaded")) { 
					if (ll != null) ll.onLoad();
					//Trace.printVerbose("adjusting");
					getDisplay().asyncExec(new Runnable() {
						public void run() {
							if (browser.isDisposed()) return;
							if (zoomControl == ZoomControl.HTML_LARGE) {
								browser.execute("map.addControl(new GLargeMapControl());");
							} else if (zoomControl == ZoomControl.HTML_SMALL) {
								browser.execute("map.addControl(new GSmallMapControl());");
							}
							adjustSize();
						}
					});

				}
			}
		});
		//}

		browser.addListener(SWT.Resize, new Listener() {
			public void handleEvent(Event event) {
				adjustSize();
			}
		});

		if (zoomControl == ZoomControl.SWT_SCALE) {
			final Scale zoom =  new Scale(this,SWT.VERTICAL);
			zoom.setMaximum(17);
			zoom.setMinimum(0);
			zoom.setIncrement(1);
			zoom.setSelection(1);
			zoom.setLayoutData(new GridData(GridData.FILL_VERTICAL));

			zoom.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					browser.execute("map.setZoom(" + zoom.getSelection() + ");");
				}
			});
		}
		try {
			browser.setUrl("file://" + FileUtil.extractResource(getClass(), "/html/maps.html", "rssmaps", "html"));
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
}
