package webapi;

import java.io.IOException;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;

import util.Pair;
/**
 * Accesses the LiveJournal Web API.
 * @author oleg
 */
public class LiveJournal {
	/**
	 * Obtains the list of friends' feeds.
	 * @param user user
	 * @param password password
	 * @return a collection of pairs (account, feed url)
	 * @throws WebConnectionException if an connection/authentication error occurs
	 */
	@SuppressWarnings({ "unchecked" })
	public static Collection<Pair<String,String>> getFriendsFeeds(String user, String password) throws WebConnectionException{
		XmlRpcClient ljclient = null;
		try {
			ljclient = ljConnect();
		} catch (MalformedURLException e) {
			throw new WebConnectionException("The LiveJournal service URL is malformed");
		}
		final List<Pair<String,String>> friendsFeeds = new LinkedList<Pair<String,String>>();
		final Vector arg = new Vector(1);
		Pair<String, String> authToken = produceAuthToken(ljclient, password);
		Hashtable hashtable = new Hashtable();
		hashtable.put("username", user);
		hashtable.put("auth_method", "challenge");
		hashtable.put("auth_challenge", authToken.car);
		hashtable.put("auth_response", authToken.cdr);
		arg.add(hashtable);
		Object result = null;
		try {
			result = ljclient.execute("LJ.XMLRPC.getfriends",arg);
		} catch (XmlRpcException e) {
			throw new WebConnectionException("Unable to retrieve the list of friends: error communicating with the LiveJournal service");
		} catch (IOException e) {
			throw new WebConnectionException("Unable to connect to the LiveJournal service");
		}
		if (result != null) {

			Vector friends = (Vector)((Hashtable) result).get("friends");
			for (Object friend: friends) {
				if (!"deleted".equals(((Hashtable) friend).get("status"))) {
					String account = (String)((Hashtable) friend).get("username");
					friendsFeeds.add(new Pair<String,String>(account, new StringBuffer("http://").append(account).append(".livejournal.com/data/atom").toString()));
				}
			}
		}
		return friendsFeeds;
	}
	/**
	 * Posts an entry to a LiveJournal journal.
	 * @param user username
	 * @param password password
	 * @param title the title of the entry
	 * @param entry the content of the entry
	 * @throws WebConnectionException if a connection/authentication error occurs
	 */
	@SuppressWarnings("unchecked")
	public static void postEntry(String user, String password, String title, String entry) throws WebConnectionException {
		XmlRpcClient ljclient = null;
		try {
			ljclient = ljConnect();
		} catch (MalformedURLException e) {
			throw new WebConnectionException("The LiveJournal service URL is malformed");
		}
		Pair<String, String> authToken = produceAuthToken(ljclient,password);
		final Vector arg = new Vector(1);
		Hashtable hashtable = new Hashtable();
		Calendar calendar = GregorianCalendar.getInstance();
		hashtable.put("username", user);
		hashtable.put("auth_method", "challenge");
		hashtable.put("auth_challenge", authToken.car);
		hashtable.put("auth_response", authToken.cdr);
		hashtable.put("event", entry);
		hashtable.put("subject", title);
		hashtable.put("year", String.valueOf(calendar.get(Calendar.YEAR)));
		hashtable.put("mon", String.valueOf(calendar.get(Calendar.MONTH) + 1));
		hashtable.put("day", String.valueOf(calendar.get(Calendar.DAY_OF_MONTH)));
		hashtable.put("hour", String.valueOf(calendar.get(Calendar.HOUR_OF_DAY)));
		hashtable.put("min", String.valueOf(calendar.get(Calendar.MINUTE)));
		arg.add(hashtable);
		try {
			ljclient.execute("LJ.XMLRPC.postevent", arg);
		} catch (XmlRpcException e) {
			throw new WebConnectionException("Unable to post an entry: error communicating with the LiveJournal service");
		} catch (IOException e) {
			throw new WebConnectionException("Unable to connect to the LiveJournal service");
		}
	}
	// authenticates with the LiveJournal server
	private static Pair<String, String> produceAuthToken(XmlRpcClient ljclient, String password) throws WebConnectionException {
		final Vector arg = new Vector(1);
		Object result;
		try {
			result = ljclient.execute("LJ.XMLRPC.getchallenge", arg);
		} catch (XmlRpcException e) {
			throw new WebConnectionException("Unable to authenticate: error communicating with the LiveJournal service");
		} catch (IOException e) {
			throw new WebConnectionException("Unable to connect to the LiveJournal service");
		}
		String challenge = (String)((Hashtable) result).get("challenge");
		MessageDigest md5hasher;
		try {
			md5hasher = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException e) {
			throw new WebConnectionException("Unable to authenticate: MD5 module is not available");
		}
		md5hasher.update(password.getBytes());
		md5hasher.update((challenge+hashToHex(md5hasher.digest())).getBytes());
		return new Pair<String, String>(challenge, hashToHex(md5hasher.digest()));
	}
	// instantiates an xml-rpc client 
	private static XmlRpcClient ljConnect() throws MalformedURLException {
		return new XmlRpcClient("http://www.livejournal.com/interface/xmlrpc");
	}
	// converts a byte array into a string with its hex representation
	private static String hashToHex(byte[] hash) {
		return new BigInteger(1, hash).toString(16);
	}
}
